{-# OPTIONS_GHC -fglasgow-exts #-}

module Ceta where {


data Nat = Zero_nat | Suc Nat;

data Scg a b = Null | Scg a a [(b, b)] [(b, b)];

data Shp a = Sharp a | Plain a;

data Xml = Xml [Char] [([Char], [Char])] [Xml] (Maybe [Char]);

leta :: forall a b. a -> (a -> b) -> b;
leta s f = f s;

class Orda a where {
  less_eq :: a -> a -> Bool;
  less :: a -> a -> Bool;
};

nat_aux :: Integer -> Nat -> Nat;
nat_aux i n = (if i <= 0 then n else nat_aux (i - 1) (Suc n));

nat :: Integer -> Nat;
nat i = nat_aux i Zero_nat;

hd :: forall a. [a] -> a;
hd (x : xs) = x;

tl :: forall a. [a] -> [a];
tl [] = [];
tl (x : xs) = xs;

split :: forall a b c. (a -> b -> c) -> (a, b) -> c;
split f (a, b) = f a b;

lex_ext_unbounded ::
  forall a. (a -> a -> (Bool, Bool)) -> [a] -> [a] -> (Bool, Bool);
lex_ext_unbounded f [] [] = (False, True);
lex_ext_unbounded f (uu : uv) [] = (True, True);
lex_ext_unbounded f [] (uw : ux) = (False, False);
lex_ext_unbounded f (a : asa) (b : bs) =
  case f a b of {
    (True, nstri) -> (True, True);
    (False, True) -> lex_ext_unbounded f asa bs;
    (False, False) -> (False, False);
  };

less_nat :: Nat -> Nat -> Bool;
less_nat m (Suc n) = less_eq_nat m n;
less_nat n Zero_nat = False;

less_eq_nat :: Nat -> Nat -> Bool;
less_eq_nat (Suc m) n = less_nat m n;
less_eq_nat Zero_nat n = True;

eq_nat :: Nat -> Nat -> Bool;
eq_nat Zero_nat Zero_nat = True;
eq_nat (Suc nat) (Suc nat') = eq_nat nat nat';
eq_nat Zero_nat (Suc nat') = False;
eq_nat (Suc nat') Zero_nat = False;

plus_nat :: Nat -> Nat -> Nat;
plus_nat (Suc m) n = plus_nat m (Suc n);
plus_nat Zero_nat n = n;

size_list :: forall a. [a] -> Nat;
size_list [] = Zero_nat;
size_list (a : list) = plus_nat (size_list list) (Suc Zero_nat);

lex_ext ::
  forall a. (a -> a -> (Bool, Bool)) -> Nat -> [a] -> [a] -> (Bool, Bool);
lex_ext f n ss ts =
  let {
    lts = size_list ts;
  } in (if eq_nat (size_list ss) lts || less_eq_nat lts n
         then lex_ext_unbounded f ss ts else (False, False));

foralla :: forall a. (a -> Bool) -> [a] -> Bool;
foralla p [] = True;
foralla p (x : xs) = p x && foralla p xs;

anya :: forall a. (a -> Bool) -> [a] -> Bool;
anya uu [] = False;
anya p (x : xs) = p x || anya p xs;

nulla :: forall a. [a] -> Bool;
nulla [] = True;
nulla (x : xs) = False;

data Term a b = Var b | Fun a [(Term a b)];

lpo ::
  forall a b.
    (Eq b) => (a -> Nat -> Nat) -> Nat -> Term a b -> Term a b -> (Bool, Bool);
lpo uu uv (Var x) (Var y) = (False, x == y);
lpo pr uw (Var x) (Fun g ts) =
  (False, nulla ts && eq_nat (pr g Zero_nat) Zero_nat);
lpo pr n (Fun f ss) (Var y) =
  let {
    con = anya (\ s -> snd (lpo pr n s (Var y))) ss;
  } in (con, con);
lpo pr n (Fun f ss) (Fun g ts) =
  (if anya (\ s -> snd (lpo pr n s (Fun g ts))) ss then (True, True)
    else let {
           prl = pr f (size_list ss);
           prr = pr g (size_list ts);
         } in (if less_eq_nat prr prl &&
                    foralla (\ t -> fst (lpo pr n (Fun f ss) t)) ts
                then (if less_nat prr prl then (True, True)
                       else lex_ext (lpo pr n) n ss ts)
                else (False, False)));

newtype Sub a b = Sub (b -> Term a b);

foldla :: forall a b. (a -> b -> a) -> a -> [b] -> a;
foldla f a [] = a;
foldla f a (x : xs) = foldla f (f a x) xs;

one_nat :: Nat;
one_nat = Suc Zero_nat;

sz :: forall a b. Term a b -> Nat;
sz (Var x) = one_nat;
sz (Fun f ts) =
  plus_nat one_nat (foldla (\ a x -> plus_nat a (sz x)) Zero_nat ts);

data Poly a = PVar Nat | PNum a | PSum [(Poly a)] | PMult [(Poly a)];

option_case :: forall a b. a -> (b -> a) -> Maybe b -> a;
option_case f1 f2 (Just a) = f2 a;
option_case f1 f2 Nothing = f1;

decomp :: forall a b. [a] -> [b] -> Maybe [(a, b)];
decomp [] [] = Just [];
decomp (uu : uv) [] = Nothing;
decomp [] (uw : ux) = Nothing;
decomp (s : ss) (t : ts) =
  case decomp ss ts of {
    Nothing -> Nothing;
    Just pairs -> Just ((s, t) : pairs);
  };

append :: forall a. [a] -> [a] -> [a];
append [] ys = ys;
append (x : xs) ys = x : append xs ys;

data Hterm a b = HHole | HFun a [(Hterm a b)];

gcMatchList ::
  forall a b. (Eq a) => [(Hterm a b, Term a b)] -> Maybe [(Hterm a b, b)];
gcMatchList [] = Just [];
gcMatchList ((HHole, uu) : ps) = gcMatchList ps;
gcMatchList ((HFun f hs, Fun g ss) : ps) =
  (if f == g
    then case decomp hs ss of {
           Nothing -> Nothing;
           Just pairs -> gcMatchList (append pairs ps);
         }
    else Nothing);
gcMatchList ((HFun f hs, Var x) : ps) =
  case gcMatchList ps of {
    Nothing -> Nothing;
    Just pairs -> Just ((HFun f hs, x) : pairs);
  };

mergeHTerms ::
  forall a b. (Eq a) => [(Hterm a b)] -> [(Hterm a b)] -> Maybe [(Hterm a b)];
mergeHTerms [] [] = Just [];
mergeHTerms [] (uu : uv) = Nothing;
mergeHTerms (uw : ux) [] = Nothing;
mergeHTerms (HHole : hsl) (h : hsr) =
  case mergeHTerms hsl hsr of {
    Nothing -> Nothing;
    Just res -> Just (h : res);
  };
mergeHTerms (HFun f hs : hsl) (HHole : hsr) =
  case mergeHTerms hsl hsr of {
    Nothing -> Nothing;
    Just res -> Just (HFun f hs : res);
  };
mergeHTerms (HFun f fs : hsl) (HFun g gs : hsr) =
  (if f == g
    then case mergeHTerms fs gs of {
           Nothing -> Nothing;
           Just mhs ->
             case mergeHTerms hsl hsr of {
               Nothing -> Nothing;
               Just res -> Just (HFun f mhs : res);
             };
         }
    else Nothing);

nth :: forall a. [a] -> Nat -> a;
nth (x : xs) (Suc n) = nth xs n;
nth (x : xs) Zero_nat = x;

merge :: forall a b. (Eq a) => Hterm a b -> Hterm a b -> Maybe (Hterm a b);
merge hl hr =
  case mergeHTerms [hl] [hr] of {
    Nothing -> Nothing;
    Just res -> Just (nth res Zero_nat);
  };

mergeX ::
  forall a b.
    (Eq a,
      Eq b) => a -> Hterm b a -> [(Hterm b a, a)] -> Maybe [(Hterm b a, a)];
mergeX x h [] = Just [];
mergeX x h ((ht, y) : ps) =
  (if x == y
    then case merge h ht of {
           Nothing -> Nothing;
           Just m -> mergeX x m ps;
         }
    else case mergeX x h ps of {
           Nothing -> Nothing;
           Just pairs -> Just ((ht, y) : pairs);
         });

mergeAll :: forall a b. (Eq a, Eq b) => [(Hterm a b, b)] -> Bool;
mergeAll [] = True;
mergeAll ((h, x) : ps) =
  case mergeX x h ps of {
    Nothing -> False;
    Just a -> mergeAll a;
  };

gcMatchI :: forall a b. (Eq a, Eq b) => Hterm a b -> Term a b -> Bool;
gcMatchI h t =
  case gcMatchList [(h, t)] of {
    Nothing -> False;
    Just a -> mergeAll a;
  };

mapa :: forall a b. (a -> b) -> [a] -> [b];
mapa f [] = [];
mapa f (x : xs) = f x : mapa f xs;

capI ::
  forall a b. (Eq a, Eq b) => [(Term a b, Term a b)] -> Term a b -> Hterm a b;
capI uu (Var uv) = HHole;
capI r (Fun f ts) =
  let {
    h = HFun f (mapa (capI r) ts);
  } in (if anya (\ ra -> gcMatchI h (fst ra)) r then HHole else h);

rev :: forall a. [a] -> [a];
rev xs = foldla (\ xsa x -> x : xsa) [] xs;

upt :: Nat -> Nat -> [Nat];
upt i j = (if less_nat i j then i : upt (Suc i) j else []);

list_case :: forall a b. a -> (b -> [b] -> a) -> [b] -> a;
list_case f1 f2 (a : list) = f2 a list;
list_case f1 f2 [] = f1;

zipa :: forall a b. [a] -> [b] -> [(a, b)];
zipa xs [] = [];
zipa xs (y : ys) = case xs of {
                     [] -> [];
                     z : zs -> (z, y) : zipa zs ys;
                   };

data Pos = Empty | PCons Nat Pos;

concat_map :: forall a b. (a -> [b]) -> [a] -> [b];
concat_map f [] = [];
concat_map f (x : xs) = append (f x) (concat_map f xs);

poss_list :: forall a b. Term a b -> [Pos];
poss_list (Var x) = [Empty];
poss_list (Fun f ss) =
  Empty :
    concat_map (\ (i, a) -> mapa (PCons i) a)
      (zipa (upt Zero_nat (size_list ss)) (mapa poss_list ss));

subt_at :: forall a b. Term a b -> Pos -> Term a b;
subt_at s Empty = s;
subt_at (Fun f ss) (PCons i p) = subt_at (nth ss i) p;

decompose_one ::
  forall a b. (Eq a) => Term a b -> Term a b -> Maybe [(Term a b, Term a b)];
decompose_one (Var x) t = Just [(Var x, t)];
decompose_one (Fun v va) (Var y) = Nothing;
decompose_one (Fun f ts) (Fun g ss) =
  (if f == g then decomp ts ss else Nothing);

decompose_n ::
  forall a b.
    (Eq a) => Nat -> Term a b -> Term a b -> Maybe [(Term a b, Term a b)];
decompose_n Zero_nat s t = decompose_one s t;
decompose_n (Suc n) s t =
  case decompose_n n s t of {
    Nothing -> Nothing;
    Just a ->
      foldla
        (\ aa (sa, ta) ->
          case aa of {
            Nothing -> Nothing;
            Just ys ->
              case decompose_one sa ta of {
                Nothing -> Nothing;
                Just xs -> Just (append xs ys);
              };
          })
        (Just []) a;
  };

decompose ::
  forall a b. (Eq a) => Term a b -> Term a b -> Maybe [(Term a b, Term a b)];
decompose s t = decompose_n (sz s) s t;

decompose_list ::
  forall a b. (Eq a) => [(Term a b, Term a b)] -> Maybe [(Term a b, Term a b)];
decompose_list ts =
  foldla
    (\ a (s, t) ->
      case a of {
        Nothing -> Nothing;
        Just ls ->
          case decompose s t of {
            Nothing -> Nothing;
            Just ys -> Just (append ls ys);
          };
      })
    (Just []) ts;

sub_empty :: forall a b. Sub a b;
sub_empty = Sub Var;

sub_case :: forall a b c. ((a -> Term b a) -> c) -> Sub b a -> c;
sub_case f1 (Sub fun) = f1 fun;

get_sub :: forall a b. Sub a b -> b -> Term a b;
get_sub (Sub sigma) = sigma;

sub_apply_term :: forall a b. Term a b -> Sub a b -> Term a b;
sub_apply_term (Var x) sigma = get_sub sigma x;
sub_apply_term (Fun f ss) sigma =
  Fun f (mapa (\ t -> sub_apply_term t sigma) ss);

list_all2 :: forall a b. (a -> b -> Bool) -> [a] -> [b] -> Bool;
list_all2 p (x : xs) (y : ys) = p x y && list_all2 p xs ys;
list_all2 p xs [] = nulla xs;
list_all2 p [] ys = nulla ys;

eq_term :: forall a b. (Eq a, Eq b) => Term a b -> Term a b -> Bool;
eq_term (Fun f ts) (Fun g us) = f == g && list_all2 eq_term ts us;
eq_term (Var x) (Var y) = x == y;
eq_term (Var v) (Fun f' list') = False;
eq_term (Fun f' list') (Var v) = False;

instance (Eq a, Eq b) => Eq (Term a b) where {
  a == b = eq_term a b;
};

member :: forall a. (Eq a) => a -> [a] -> Bool;
member x [] = False;
member x (y : ys) = x == y || member x ys;

subst_safe ::
  forall a b.
    (Eq a, Eq b) => [a] -> [(Term b a, Term b a)] -> Sub b a -> Maybe (Sub b a);
subst_safe vs [] sigma = Just sigma;
subst_safe vs ((Var x, t) : ts) sigma =
  (if member (Var x, t) ts then subst_safe vs ts sigma
    else (if member x vs && not (eq_term (sub_apply_term (Var x) sigma) t)
           then Nothing
           else case subst_safe (x : vs) ts sigma of {
                  Nothing -> Nothing;
                  Just (Sub s) ->
                    Just (Sub (\ y -> (if x == y then t else s y)));
                }));

match_all ::
  forall a b. (Eq a, Eq b) => [(Term a b, Term a b)] -> Maybe (Sub a b);
match_all ts =
  case decompose_list ts of {
    Nothing -> Nothing;
    Just ls -> subst_safe [] ls sub_empty;
  };

nat_case :: forall a. a -> (Nat -> a) -> Nat -> a;
nat_case f1 f2 (Suc nat) = f2 nat;
nat_case f1 f2 Zero_nat = f1;

dropa :: forall a. Nat -> [a] -> [a];
dropa n [] = [];
dropa n (x : xs) = case n of {
                     Zero_nat -> x : xs;
                     Suc m -> dropa m xs;
                   };

takea :: forall a. Nat -> [a] -> [a];
takea n [] = [];
takea n (x : xs) = case n of {
                     Zero_nat -> [];
                     Suc m -> x : takea m xs;
                   };

data Ctxt a b = Hole | More a [(Term a b)] (Ctxt a b) [(Term a b)];

ctxt_of_pos_term :: forall a b. Pos -> Term a b -> Ctxt a b;
ctxt_of_pos_term Empty t = Hole;
ctxt_of_pos_term (PCons i ps) (Fun f ts) =
  More f (takea i ts) (ctxt_of_pos_term ps (nth ts i)) (dropa (Suc i) ts);

data Nibble = Nibble0 | Nibble1 | Nibble2 | Nibble3 | Nibble4 | Nibble5
  | Nibble6 | Nibble7 | Nibble8 | Nibble9 | NibbleA | NibbleB | NibbleC
  | NibbleD | NibbleE | NibbleF;

shows_string :: [Char] -> [Char] -> [Char];
shows_string = append;

shows_sep ::
  forall a.
    (a -> [Char] -> [Char]) -> ([Char] -> [Char]) -> [a] -> [Char] -> [Char];
shows_sep s sep [] = shows_string [];
shows_sep s sep [x] = s x;
shows_sep s sep (x : v : va) = (s x . sep) . shows_sep s sep (v : va);

shows_between ::
  ([Char] -> [Char]) ->
    ([Char] -> [Char]) -> ([Char] -> [Char]) -> [Char] -> [Char];
shows_between l p r = l . p . r;

shows_list_gen ::
  forall a.
    (a -> [Char] -> [Char]) ->
      [Char] -> [Char] -> [Char] -> [Char] -> [a] -> [Char] -> [Char];
shows_list_gen elt e l s r xs =
  (if nulla xs then shows_string e
    else shows_between (shows_string l) (shows_sep elt (shows_string s) xs)
           (shows_string r));

class Shows a where {
  showsb :: a -> [Char] -> [Char];
  shows_list :: [a] -> [Char] -> [Char];
};

shows_term :: forall a b. (Shows a, Shows b) => Term a b -> [Char] -> [Char];
shows_term (Var x) = showsb x;
shows_term (Fun f ts) =
  showsb f . shows_list_gen id [] ['('] [','] [')'] (mapa shows_term ts);

ctxt_apply :: forall a b. Ctxt a b -> Term a b -> Term a b;
ctxt_apply Hole s = s;
ctxt_apply (More f ss1 c ss2) s = Fun f (append ss1 (ctxt_apply c s : ss2));

data Eithera a b = Lefta a | Righta b;

errora :: forall a b. a -> Eithera a b;
errora = Lefta;

returna :: forall a b. a -> Eithera b a;
returna = Righta;

checkB :: forall a. Bool -> a -> Eithera a ();
checkB b e = (if b then returna () else errora e);

rewrites ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => (Term a b, Term a b) ->
                    Ctxt a b ->
                      Sub a b ->
                        (Term a b, Term a b) ->
                          [(Term a b, Term a b)] ->
                            Eithera ([Char] -> [Char]) ();
rewrites (s, t) c sigma (l, ra) r =
  checkB
    (eq_term s (ctxt_apply c (sub_apply_term l sigma)) &&
      (eq_term t (ctxt_apply c (sub_apply_term ra sigma)) && member (l, ra) r))
    (shows_string ['T', 'e', 'r', 'm', ' '] .
      shows_term s .
        shows_string
          [' ', 'd', 'o', 'e', 's', ' ', 'n', 'o', 't', ' ', 'r', 'e', 'w', 'r',
            'i', 't', 'e', ' ', 't', 'o', ' ', 't', 'e', 'r', 'm', ' ']
          . shows_term t);

either_case :: forall a b c. (a -> b) -> (c -> b) -> Eithera a c -> b;
either_case f1 f2 (Righta b) = f2 b;
either_case f1 f2 (Lefta a) = f1 a;

isValue :: forall a b. Eithera a b -> Bool;
isValue p = case p of {
              Lefta _ -> False;
              Righta _ -> True;
            };

getpr ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => Term a b ->
                    Term a b ->
                      Pos ->
                        (Term a b, Term a b) ->
                          Maybe (Term a b,
                                  (Ctxt a b, (Sub a b, (Term a b, Term a b))));
getpr s t p (l, r) =
  case match_all [(l, subt_at s p), (r, subt_at t p)] of {
    Nothing -> Nothing;
    Just sigma ->
      (if isValue (rewrites (s, t) (ctxt_of_pos_term p s) sigma (l, r) [(l, r)])
        then Just (s, (ctxt_of_pos_term p s, (sigma, (l, r)))) else Nothing);
  };

getp ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => Term a b ->
                    Term a b ->
                      Pos ->
                        [(Term a b, Term a b)] ->
                          Maybe (Term a b,
                                  (Ctxt a b, (Sub a b, (Term a b, Term a b))));
getp s t p [] = Nothing;
getp s t p (r : rs) =
  case getpr s t p r of {
    Nothing -> getp s t p rs;
    Just a -> Just a;
  };

find_get_p ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => Term a b ->
                    Term a b ->
                      [Pos] ->
                        [(Term a b, Term a b)] ->
                          Maybe (Term a b,
                                  (Ctxt a b, (Sub a b, (Term a b, Term a b))));
find_get_p s t [] r = Nothing;
find_get_p s t (p : ps) r =
  case getp s t p r of {
    Nothing -> find_get_p s t ps r;
    Just a -> Just a;
  };

get ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => Term a b ->
                    Term a b ->
                      [(Term a b, Term a b)] ->
                        Eithera ([Char] -> [Char])
                          (Term a b,
                            (Ctxt a b, (Sub a b, (Term a b, Term a b))));
get s t r =
  case find_get_p s t (poss_list s) r of {
    Nothing ->
      errora
        (shows_string
           ['C', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'f', 'i', 'n',
             'd', ' ', 'r', 'e', 'd', 'u', 'c', 't', 'i', 'o', 'n', ' ', 'b',
             'e', 't', 'w', 'e', 'e', 'n', ' ', 't', 'e', 'r', 'm', ' ']
          . shows_term s .
              shows_string [' ', 'a', 'n', 'd', ' ', 't', 'e', 'r', 'm', ' '] .
                shows_term t . shows_string ['.']);
    Just a -> returna a;
  };

prod_case :: forall a b c. (a -> b -> c) -> (a, b) -> c;
prod_case f1 (a, b) = f1 a b;

remdups :: forall a. (Eq a) => [a] -> [a];
remdups [] = [];
remdups (x : xs) = (if member x xs then remdups xs else x : remdups xs);

comp :: forall a. (Eq a) => [(a, a)] -> [(a, a)] -> [(a, a)];
comp es es' =
  remdups
    (concat_map
      (\ (x, y) ->
        concat_map (\ (y', z) -> (if y == y' then [(x, z)] else [])) es')
      es);

data Alter a b c d e f g h = Alter1 a | Alter2 b | Alter3 c | Alter4 d
  | Alter5 e | Alter6 f | Alter7 g | Alter8 h;

newline :: Char;
newline = '\^J';

wspace :: [Char];
wspace = [newline, '\^I', '\^M', ' '];

trim :: [Char] -> [Char];
trim (c : s) = (if member c wspace then trim s else c : s);
trim [] = [];

capRM2 ::
  forall a b.
    (Eq a, Eq b) => (a -> [(Term a b, Term a b)]) -> Term a b -> Hterm a b;
capRM2 uu (Var uv) = HHole;
capRM2 rm (Fun f ts) =
  let {
    h = HFun f (mapa (capRM2 rm) ts);
  } in (if anya (\ r -> gcMatchI h (fst r)) (rm f) then HHole else h);

capRM ::
  forall a b.
    (Eq a,
      Eq b) => Maybe (a -> [(Term a b, Term a b)]) -> Term a b -> Hterm a b;
capRM r = case r of {
            Nothing -> (\ _ -> HHole);
            Just a -> capRM2 a;
          };

instance Eq Nat where {
  a == b = eq_nat a b;
};

combine :: forall a b. (Eq b) => Scg a b -> Scg a b -> Scg a b;
combine (Scg p q str wk) (Scg p' q' str' wk') =
  Scg p q (remdups (append str str')) (remdups (append wk wk'));
combine Null s = Null;
combine (Scg v va vb vc) Null = Null;

list_all :: forall a. (a -> Bool) -> [a] -> Bool;
list_all p [] = True;
list_all p (x : xs) = p x && list_all p xs;

subsumes :: forall a b. (Eq a, Eq b) => Scg a b -> Scg a b -> Bool;
subsumes (Scg p q str wk) (Scg p' q' str' wk') =
  p == p' &&
    (q == q' &&
      (list_all (\ x -> member x str') str &&
        list_all (\ x -> member x (append str' wk')) wk));
subsumes g Null = True;
subsumes Null (Scg v va vb vc) = False;

filtera :: forall a. (a -> Bool) -> [a] -> [a];
filtera p [] = [];
filtera p (x : xs) = (if p x then x : filtera p xs else filtera p xs);

subtract_list :: forall a. (Eq a) => [a] -> [a] -> [a];
subtract_list xs ys = filtera (\ x -> not (member x ys)) xs;

scg_comp ::
  forall a b. (Eq b) => (a -> a -> Bool) -> Scg a b -> Scg a b -> Scg a b;
scg_comp conn (Scg p q str wk) (Scg p' q' str' wk') =
  (if not (conn q p') then Null
    else let {
           strs =
             remdups
               (append (comp str str') (append (comp str wk') (comp wk str')));
           a = subtract_list (remdups (comp wk wk')) strs;
         } in Scg p q' strs a);
scg_comp conn Null g = Null;
scg_comp conn (Scg v va vb vc) Null = Null;

list_ex :: forall a. (a -> Bool) -> [a] -> Bool;
list_ex p [] = False;
list_ex p (x : xs) = p x || list_ex p xs;

in_situ :: forall a b. (Eq b) => Scg a b -> Bool;
in_situ Null = True;
in_situ (Scg p q str wk) = list_ex (\ (a, b) -> a == b) str;

sagiv :: forall a b. (Eq a, Eq b) => (a -> a -> Bool) -> Scg a b -> Bool;
sagiv conn g =
  (if in_situ g then True
    else let {
           gg = scg_comp conn g g;
         } in (if subsumes gg g then False else sagiv conn (combine g gg)));

showa :: forall a. (Shows a) => a -> [Char];
showa x = showsb x [];

shows_char :: Char -> [Char] -> [Char];
shows_char c = (\ a -> c : a);

shows_paren :: ([Char] -> [Char]) -> [Char] -> [Char];
shows_paren p = shows_char '(' . p . shows_char ')';

showsa :: forall a b. (Shows a, Shows b) => (a, b) -> [Char] -> [Char];
showsa p =
  shows_paren (showsb (fst p :: a) . shows_char ',' . showsb (snd p :: b));

shows_list_aux :: forall a. (a -> [Char] -> [Char]) -> [a] -> [Char] -> [Char];
shows_list_aux s xs = shows_list_gen s ['[', ']'] ['['] [','] [']'] xs;

shows_listb :: forall a b. (Shows a, Shows b) => [(a, b)] -> [Char] -> [Char];
shows_listb ps = shows_list_aux showsa ps;

instance (Shows a, Shows b) => Shows (a, b) where {
  showsb = showsa;
  shows_list = shows_listb;
};

args :: forall a b. Term a b -> [(Term a b)];
args (Var uu) = [];
args (Fun uv ts) = ts;

root :: forall a b. Term a b -> Maybe a;
root (Var x) = Nothing;
root (Fun f ts) = Just f;

eq_shp :: forall a. (Eq a) => Shp a -> Shp a -> Bool;
eq_shp (Sharp f) (Sharp f') = f == f';
eq_shp (Plain f) (Plain f') = f == f';
eq_shp (Sharp f) (Plain f') = False;
eq_shp (Plain f') (Sharp f) = False;

instance (Eq a) => Eq (Shp a) where {
  a == b = eq_shp a b;
};

data Rat = Fract Integer Integer;

data Domain = Natural | Arctic | Arctic_rat Rat | Int_mat Nat Nat | Arc_mat Nat
  | Arc_rat_mat Rat Nat | Rationala Rat | Rat_mat Rat Nat Nat;

data Xmldoc = Xmldoc [Char] Xml;

instance Orda Integer where {
  less_eq = (\ a b -> a <= b);
  less = (\ a b -> a < b);
};

foldra :: forall a b. (a -> b -> b) -> [a] -> b -> b;
foldra f [] a = a;
foldra f (x : xs) a = f x (foldra f xs a);

instance Orda Nat where {
  less_eq = less_eq_nat;
  less = less_nat;
};

the :: forall a. Maybe a -> a;
the (Just x) = x;

eq_scg :: forall a b. (Eq a, Eq b) => Scg a b -> Scg a b -> Bool;
eq_scg Null Null = True;
eq_scg (Scg pp1 pp2 list1 list2) (Scg pp1' pp2' list1' list2') =
  pp1 == pp1' && (pp2 == pp2' && (list1 == list1' && list2 == list2'));
eq_scg Null (Scg pp1' pp2' list1' list2') = False;
eq_scg (Scg pp1' pp2' list1' list2') Null = False;

term_case ::
  forall a b c. (a -> b) -> (c -> [(Term c a)] -> b) -> Term c a -> b;
term_case f1 f2 (Fun f list) = f2 f list;
term_case f1 f2 (Var v) = f1 v;

suptp :: forall a b. (Eq a, Eq b) => Term a b -> Term a b -> Bool;
suptp s t =
  case s of {
    Var _ -> False;
    Fun _ ts -> member t ts || anya (\ sa -> suptp sa t) ts;
  };

plaina :: forall a b. Term a b -> Term (Shp a) b;
plaina (Var x) = Var x;
plaina (Fun f ss) = Fun (Plain f) (mapa plaina ss);

sharpa :: forall a b. Term a b -> Term (Shp a) b;
sharpa (Var x) = Var x;
sharpa (Fun f ss) = Fun (Sharp f) (mapa plaina ss);

dquote :: [Char];
dquote = ['\"'];

eq_xml :: Xml -> Xml -> Bool;
eq_xml (Xml name1 attrs1 children1 text1) (Xml name2 attrs2 children2 text2) =
  name1 == name2 &&
    (attrs1 == attrs2 &&
      (list_all2 eq_xml children1 children2 && text1 == text2));

newtype SpL a = SimpProj [(a, Nat)];

minus_nat :: Nat -> Nat -> Nat;
minus_nat (Suc m) (Suc n) = minus_nat m n;
minus_nat Zero_nat n = Zero_nat;
minus_nat m Zero_nat = m;

times_nat :: Nat -> Nat -> Nat;
times_nat Zero_nat n = Zero_nat;
times_nat (Suc m) n = plus_nat n (times_nat m n);

digit2nat :: Char -> Eithera [Char] Nat;
digit2nat x =
  (if x == '0' then returna Zero_nat
    else (if x == '1' then returna one_nat
           else (if x == '2' then returna (nat 2)
                  else (if x == '3' then returna (nat 3)
                         else (if x == '4' then returna (nat 4)
                                else (if x == '5' then returna (nat 5)
                                       else (if x == '6' then returna (nat 6)
      else (if x == '7' then returna (nat 7)
             else (if x == '8' then returna (nat 8)
                    else (if x == '9' then returna (nat 9)
                           else errora
                                  (append [x]
                                    [' ', 'i', 's', ' ', 'n', 'o', 't', ' ',
                                      'a', ' ', 'd', 'i', 'g', 'i',
                                      't'])))))))))));

bind :: forall a b c. Eithera a b -> (b -> Eithera a c) -> Eithera a c;
bind p vq = case p of {
              Lefta a -> Lefta a;
              Righta a -> vq a;
            };

string2natHelp :: Nat -> [Char] -> Eithera [Char] Nat;
string2natHelp n [] = returna n;
string2natHelp n (d : s) =
  bind (digit2nat d)
    (\ m -> string2natHelp (plus_nat (times_nat (nat 10) n) m) s);

string2nat :: [Char] -> Eithera [Char] Nat;
string2nat s =
  (if nulla s
    then errora
           ['c', 'a', 'n', 'n', 'o', 't', ' ', 'c', 'o', 'n', 'v', 'e', 'r',
             't', ' ', 'e', 'm', 'p', 't', 'y', ' ', 's', 't', 'r', 'i', 'n',
             'g', ' ', 'i', 'n', 't', 'o', ' ', 'n', 'u', 'm', 'b', 'e', 'r']
    else string2natHelp Zero_nat s);

replicatea :: forall a. Nat -> a -> [a];
replicatea Zero_nat x = [];
replicatea (Suc n) x = x : replicatea n x;

shows_map :: forall a. (a -> [Char] -> [Char]) -> [a] -> [Char] -> [Char];
shows_map s [] = id;
shows_map s (x : xs) = s x . shows_map s xs;

is_none :: forall a. Maybe a -> Bool;
is_none (Just x) = False;
is_none Nothing = True;

shows_list_char :: [Char] -> [Char] -> [Char];
shows_list_char cs = shows_string cs;

instance Shows Char where {
  showsb = shows_char;
  shows_list = shows_list_char;
};

shows_lista :: forall a. (Shows a) => [a] -> [Char] -> [Char];
shows_lista l = shows_list l;

shows_attr :: ([Char], [Char]) -> [Char] -> [Char];
shows_attr av =
  shows_lista (fst av) .
    shows_string ['='] . shows_string (append dquote (append (snd av) dquote));

shows_attrs :: [([Char], [Char])] -> [Char] -> [Char];
shows_attrs asa = shows_map (\ a -> shows_string [' '] . shows_attr a) asa;

shows_XML_indent :: [Char] -> Nat -> Xml -> [Char] -> [Char];
shows_XML_indent ind i (Xml n a c t) =
  shows_string ind .
    shows_string ['<'] .
      shows_lista n .
        shows_attrs a .
          (if nulla c && is_none t then shows_string ['/', '>']
            else shows_string ['>'] .
                   (if not (nulla c)
                     then shows_map
                            (shows_XML_indent (append ind (replicatea i ' ')) i)
                            c
                            . shows_string ind
                     else id)
                     . (if is_none t then id else shows_string (the t)) .
                         shows_string ['<', '/'] .
                           shows_lista n . shows_string ['>']);

shows_xml :: Xml -> [Char] -> [Char];
shows_xml xml = shows_XML_indent [newline] (nat 2) xml;

shows_list_xml :: [Xml] -> [Char] -> [Char];
shows_list_xml xs = shows_list_aux shows_xml xs;

instance Shows Xml where {
  showsb = shows_xml;
  shows_list = shows_list_xml;
};

concata :: forall a. [[a]] -> [a];
concata [] = [];
concata (x : xs) = append x (concata xs);

xml2text :: [Char] -> Xml -> Eithera [Char] [Char];
xml2text reqName (Xml name atts cs text) =
  (if name == reqName &&
        (nulla atts &&
          (nulla cs && (not (is_none text) && not (nulla (the text)))))
    then returna (the text)
    else errora
           (concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'e', 'x', 't',
                'r', 'a', 'c', 't', ' ', 't', 'e', 'x', 't', ' ', 'f', 'o', 'r',
                ' '],
               reqName, [' ', 'f', 'r', 'o', 'm', ' ', ' '], [newline],
               showa (Xml name atts cs text)]));

xml2nat :: [Char] -> Xml -> Eithera [Char] Nat;
xml2nat reqName x = bind (xml2text reqName x) string2nat;

xml3elements ::
  forall a b c d.
    [Char] ->
      (Xml -> Eithera [Char] a) ->
        (Xml -> Eithera [Char] b) ->
          (Xml -> Eithera [Char] c) ->
            (a -> b -> c -> d) -> Xml -> Eithera [Char] d;
xml3elements reqName p1 p2 p3 combi (Xml name atts cs text) =
  (if name == reqName &&
        (nulla atts && (eq_nat (size_list cs) (nat 3) && is_none text))
    then bind (p1 (nth cs Zero_nat))
           (\ a ->
             bind (p2 (nth cs one_nat))
               (\ b -> bind (p3 (nth cs (nat 2))) (returna . combi a b)))
    else errora
           (concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], [newline], showa (Xml name atts cs text)]));

mapMonad :: forall a b. (Xml -> Eithera a b) -> [Xml] -> Eithera a [b];
mapMonad uu [] = returna [];
mapMonad f (xml : rest) =
  case f xml of {
    Lefta a -> errora a;
    Righta a ->
      case mapMonad f rest of {
        Lefta aa -> errora aa;
        Righta asa -> returna (a : asa);
      };
  };

xmlManyElements ::
  forall a b.
    [Char] ->
      (Xml -> Eithera [Char] a) -> ([a] -> b) -> Xml -> Eithera [Char] b;
xmlManyElements reqName p combi (Xml name atts cs text) =
  (if name == reqName && (nulla atts && is_none text)
    then bind (mapMonad p cs) (returna . combi)
    else errora
           (concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], [newline], showa (Xml name atts cs text)]));

xml2sp :: forall a. (Xml -> Eithera [Char] a) -> Xml -> Eithera [Char] (SpL a);
xml2sp xml2name =
  xmlManyElements
    ['a', 'r', 'g', 'u', 'm', 'e', 'n', 't', 'F', 'i', 'l', 't', 'e', 'r']
    (xml3elements
      ['a', 'r', 'g', 'u', 'm', 'e', 'n', 't', 'F', 'i', 'l', 't', 'e', 'r',
        'E', 'n', 't', 'r', 'y']
      xml2name (xml2nat ['a', 'r', 'i', 't', 'y'])
      (xml2nat ['c', 'o', 'l', 'l', 'a', 'p', 's', 'i', 'n', 'g'])
      (\ f _ n -> (f, minus_nat n one_nat)))
    SimpProj;

eq_hterm :: forall a b. (Eq a) => Hterm a b -> Hterm a b -> Bool;
eq_hterm HHole HHole = True;
eq_hterm (HFun f list) (HFun f' list') = f == f' && list == list';
eq_hterm HHole (HFun f' list') = False;
eq_hterm (HFun f' list') HHole = False;

instance (Eq a) => Eq (Hterm a b) where {
  a == b = eq_hterm a b;
};

fun_upd :: forall a b. (Eq a) => (a -> b) -> a -> b -> a -> b;
fun_upd f a b = (\ x -> (if x == a then b else f x));

abs_int :: Integer -> Integer;
abs_int i = (if i < 0 then negate i else i);

sgn_int :: Integer -> Integer;
sgn_int i = (if i == 0 then 0 else (if 0 < i then 1 else negate 1));

apsnd :: forall a b c. (a -> b) -> (c, a) -> (c, b);
apsnd f (x, y) = (x, f y);

divmod_int :: Integer -> Integer -> (Integer, Integer);
divmod_int k l =
  (if k == 0 then (0, 0)
    else (if l == 0 then (0, k)
           else apsnd (\ a -> sgn_int l * a)
                  (if sgn_int k == sgn_int l
                    then (\k l -> divMod (abs k) (abs l)) k l
                    else let {
                           (r, s) = (\k l -> divMod (abs k) (abs l)) k l;
                         } in (if s == 0 then (negate r, 0)
                                else (negate r - 1, abs_int l - s)))));

mod_int :: Integer -> Integer -> Integer;
mod_int a b = snd (divmod_int a b);

gcd_int :: Integer -> Integer -> Integer;
gcd_int k l =
  abs_int (if l == 0 then k else gcd_int l (mod_int (abs_int k) (abs_int l)));

implies :: Bool -> Bool -> Bool;
implies p True = True;
implies p False = not p;
implies True p = p;
implies False p = True;

is_var :: forall a b. Term a b -> Bool;
is_var (Var uu) = True;
is_var (Fun uv uw) = False;

shows_nl :: [Char] -> [Char];
shows_nl = shows_char newline;

shows_rule ::
  forall a b. (Shows a, Shows b) => (Term a b, Term a b) -> [Char] -> [Char];
shows_rule (l, r) =
  shows_term l . shows_string [' ', '-', '>', ' '] . shows_term r;

shows_rules ::
  forall a b. (Shows a, Shows b) => [(Term a b, Term a b)] -> [Char] -> [Char];
shows_rules trs = shows_list_gen shows_rule [] [] [newline] [] trs . shows_nl;

shows_p ::
  forall a b. (Shows a, Shows b) => [(Term a b, Term a b)] -> [Char] -> [Char];
shows_p p =
  shows_string
    ['d', 'e', 'p', 'e', 'n', 'd', 'e', 'n', 'c', 'y', ' ', 'p', 'a', 'i', 'r',
      's', ':']
    . shows_nl . shows_rules p;

lookupa :: forall a b. (Eq a) => a -> [(a, b)] -> Maybe b;
lookupa uu [] = Nothing;
lookupa a ((a', b) : xs) = (if a == a' then Just b else lookupa a xs);

alter1a :: forall a b c d e f g h. Alter a b c d e f g h -> a;
alter1a (Alter1 x) = x;

alter2a :: forall a b c d e f g h. Alter a b c d e f g h -> b;
alter2a (Alter2 x) = x;

alter3a :: forall a b c d e f g h. Alter a b c d e f g h -> c;
alter3a (Alter3 x) = x;

alter4a :: forall a b c d e f g h. Alter a b c d e f g h -> d;
alter4a (Alter4 x) = x;

alter5a :: forall a b c d e f g h. Alter a b c d e f g h -> e;
alter5a (Alter5 x) = x;

alter6a :: forall a b c d e f g h. Alter a b c d e f g h -> f;
alter6a (Alter6 x) = x;

alter7a :: forall a b c d e f g h. Alter a b c d e f g h -> g;
alter7a (Alter7 x) = x;

alter8a :: forall a b c d e f g h. Alter a b c d e f g h -> h;
alter8a (Alter8 x) = x;

parseError :: forall a. [Char] -> [Char] -> Eithera [Char] a;
parseError s allowed =
  errora
    (append
      ['e', 'x', 'p', 'e', 'c', 't', 'e', 'd', ' ', 's', 'y', 'm', 'b', 'o',
        'l', 's', ' ']
      (append allowed
        (append [' ', 'b', 'u', 't', ' ', 'f', 'o', 'u', 'n', 'd', ' ']
          (takea (nat 30) s))));

consume2 :: [[Char]] -> [[Char]] -> [Char] -> Eithera [Char] ([Char], [Char]);
consume2 [] allowed s =
  parseError s (concat_map (\ sa -> append sa [',']) allowed);
consume2 (t : asa) allowed s =
  (if mapa snd (zipa t s) == t then returna (t, trim (dropa (size_list t) s))
    else consume2 asa allowed s);

consume :: [[Char]] -> [Char] -> Eithera [Char] ([Char], [Char]);
consume allowed s = consume2 allowed allowed s;

letters :: [Char];
letters =
  ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
    'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', 'A', 'B', 'C', 'D',
    'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S',
    'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '_', '0', '1', '2', '3', '4', '5', '6',
    '7', '8', '9', '&', ';', ':'];

nat2int :: Nat -> Integer;
nat2int Zero_nat = 0;
nat2int (Suc n) = 1 + nat2int n;

data Af_entry = Collapse Nat | AFList [Nat];

xmlChange ::
  forall a b. (Xml -> Eithera [Char] a) -> (a -> b) -> Xml -> Eithera [Char] b;
xmlChange p f x = bind (p x) (\ a -> returna (f a));

xml2position :: Xml -> Eithera [Char] Nat;
xml2position =
  xmlChange (xml2nat ['p', 'o', 's', 'i', 't', 'i', 'o', 'n'])
    (\ n -> minus_nat n one_nat);

xmlGetName :: Xml -> [Char];
xmlGetName (Xml name uu uv uw) = name;

xmlOptions ::
  forall a. [([Char], Xml -> Eithera [Char] a)] -> Xml -> Eithera [Char] a;
xmlOptions ps x =
  case lookupa (xmlGetName x) ps of {
    Nothing ->
      errora
        (concata
          [['e', 'x', 'p', 'e', 'c', 't', 'e', 'd', ' ', 'o', 'n', 'e', ' ',
             'o', 'f', ':', ' '],
            concat_map (\ p -> append (fst p) [' ']) ps, [newline],
            ['b', 'u', 't', ' ', 'f', 'o', 'u', 'n', 'd'], [newline], showa x]);
    Just p -> p x;
  };

xml2afs ::
  forall a.
    (Xml -> Eithera [Char] a) -> Xml -> Eithera [Char] [((a, Nat), Af_entry)];
xml2afs xml2name =
  xmlManyElements
    ['a', 'r', 'g', 'u', 'm', 'e', 'n', 't', 'F', 'i', 'l', 't', 'e', 'r']
    (xml3elements
      ['a', 'r', 'g', 'u', 'm', 'e', 'n', 't', 'F', 'i', 'l', 't', 'e', 'r',
        'E', 'n', 't', 'r', 'y']
      xml2name (xml2nat ['a', 'r', 'i', 't', 'y'])
      (xmlOptions
        [(['c', 'o', 'l', 'l', 'a', 'p', 's', 'i', 'n', 'g'],
           xmlChange
             (xml2nat ['c', 'o', 'l', 'l', 'a', 'p', 's', 'i', 'n', 'g'])
             (\ i -> Collapse (minus_nat i one_nat))),
          (['n', 'o', 'n', 'C', 'o', 'l', 'l', 'a', 'p', 's', 'i', 'n', 'g'],
            xmlManyElements
              ['n', 'o', 'n', 'C', 'o', 'l', 'l', 'a', 'p', 's', 'i', 'n', 'g']
              xml2position AFList)])
      (\ f a -> (\ b -> ((f, a), b))))
    id;

string2int :: [Char] -> Eithera [Char] Integer;
string2int s =
  case s of {
    [] -> errora
            ['c', 'a', 'n', 'n', 'o', 't', ' ', 'c', 'o', 'n', 'v', 'e', 'r',
              't', ' ', 'e', 'm', 'p', 't', 'y', ' ', 's', 't', 'r', 'i', 'n',
              'g', ' ', 'i', 'n', 't', 'o', ' ', 'n', 'u', 'm', 'b', 'e', 'r'];
    h : t ->
      let {
        (pos, ss) = (if h == '-' then (False, t) else (True, s));
      } in bind (string2nat ss)
             (\ n ->
               (if pos || eq_nat n Zero_nat then returna (nat2int n)
                 else returna (0 - nat2int n)));
  };

xml2int :: [Char] -> Xml -> Eithera [Char] Integer;
xml2int reqName x = bind (xml2text reqName x) string2int;

list2position :: [Nat] -> Pos;
list2position [] = Empty;
list2position (n : ns) = PCons n (list2position ns);

xml2pos :: Xml -> Eithera [Char] Pos;
xml2pos =
  xmlManyElements
    ['p', 'o', 's', 'i', 't', 'i', 'o', 'n', 'I', 'n', 'T', 'e', 'r', 'm']
    xml2position list2position;

xml2elements ::
  forall a b c.
    [Char] ->
      (Xml -> Eithera [Char] a) ->
        (Xml -> Eithera [Char] b) -> (a -> b -> c) -> Xml -> Eithera [Char] c;
xml2elements reqName p1 p2 combi (Xml name atts cs text) =
  (if name == reqName &&
        (nulla atts && (eq_nat (size_list cs) (nat 2) && is_none text))
    then bind (p1 (nth cs Zero_nat))
           (\ a -> bind (p2 (nth cs one_nat)) (returna . combi a))
    else errora
           (concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], [newline], showa (Xml name atts cs text)]));

xml2rat :: Xml -> Eithera [Char] Rat;
xml2rat =
  xmlOptions
    [(['i', 'n', 't', 'e', 'g', 'e', 'r'],
       xmlChange (xml2int ['i', 'n', 't', 'e', 'g', 'e', 'r'])
         (\ x -> Fract x 1)),
      (['r', 'a', 't', 'i', 'o', 'n', 'a', 'l'],
        xml2elements ['r', 'a', 't', 'i', 'o', 'n', 'a', 'l']
          (xml2int ['n', 'u', 'm', 'e', 'r', 'a', 't', 'o', 'r'])
          (xml2int ['d', 'e', 'n', 'o', 'm', 'i', 'n', 'a', 't', 'o', 'r'])
          Fract)];

string2bool :: [Char] -> Eithera [Char] Bool;
string2bool s =
  (if s == ['t', 'r', 'u', 'e'] then returna True
    else (if s == ['f', 'a', 'l', 's', 'e'] then returna False
           else errora
                  (append
                    ['c', 'a', 'n', 'n', 'o', 't', ' ', 'c', 'o', 'n', 'v', 'e',
                      'r', 't', ' ']
                    (append s
                      [' ', 'i', 'n', 't', 'o', ' ', 'B', 'o', 'o', 'l', 'e',
                        'a', 'n']))));

xml2bool :: [Char] -> Xml -> Eithera [Char] Bool;
xml2bool reqName x = bind (xml2text reqName x) string2bool;

xml2scg_position :: Xml -> Eithera [Char] Nat;
xml2scg_position = xml2nat ['p', 'o', 's', 'i', 't', 'i', 'o', 'n'];

xml1element ::
  forall a b.
    [Char] -> (Xml -> Eithera [Char] a) -> (a -> b) -> Xml -> Eithera [Char] b;
xml1element reqName p1 combi (Xml name atts cs text) =
  (if name == reqName &&
        (nulla atts && (eq_nat (size_list cs) one_nat && is_none text))
    then bind (p1 (nth cs Zero_nat)) (returna . combi)
    else errora
           (concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], [newline], showa (Xml name atts cs text)]));

xml1CmanyElements ::
  forall a b c.
    [Char] ->
      (Xml -> Eithera [Char] a) ->
        (a -> Xml -> Eithera [Char] b) ->
          (a -> [b] -> c) -> Xml -> Eithera [Char] c;
xml1CmanyElements reqName p1 p2 combi (Xml name atts cs text) =
  (if name == reqName && (nulla atts && (is_none text && not (nulla cs)))
    then bind (p1 (hd cs))
           (\ a -> bind (mapMonad (p2 a) (tl cs)) (returna . combi a))
    else errora
           (concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], [newline], showa (Xml name atts cs text)]));

xml1manyElements ::
  forall a b c.
    [Char] ->
      (Xml -> Eithera [Char] a) ->
        (Xml -> Eithera [Char] b) -> (a -> [b] -> c) -> Xml -> Eithera [Char] c;
xml1manyElements reqName p1 p2 = xml1CmanyElements reqName p1 (\ _ -> p2);

xml2var :: forall a. Xml -> Eithera [Char] (Term a [Char]);
xml2var x = bind (xml2text ['v', 'a', 'r'] x) (returna . Var);

xml2term ::
  forall a. (Xml -> Eithera [Char] a) -> Xml -> Eithera [Char] (Term a [Char]);
xml2term xml2name x =
  xmlOptions
    [(['v', 'a', 'r'], xml2var),
      (['f', 'u', 'n', 'a', 'p', 'p'],
        xml1manyElements ['f', 'u', 'n', 'a', 'p', 'p'] xml2name
          (xml1element ['a', 'r', 'g'] (xml2term xml2name) id) Fun)]
    x;

xml2rule ::
  forall a.
    (Xml -> Eithera [Char] a) ->
      Xml -> Eithera [Char] (Term a [Char], Term a [Char]);
xml2rule xml2name =
  xml2elements ['r', 'u', 'l', 'e']
    (xml1element ['l', 'h', 's'] (xml2term xml2name) id)
    (xml1element ['r', 'h', 's'] (xml2term xml2name) id) (\ a b -> (a, b));

xml2scg ::
  forall a.
    (Xml -> Eithera [Char] a) ->
      Xml ->
        Eithera [Char]
          ((Term a [Char], Term a [Char]), ([(Nat, Nat)], [(Nat, Nat)]));
xml2scg xml2name =
  xml1manyElements
    ['s', 'i', 'z', 'e', 'C', 'h', 'a', 'n', 'g', 'e', 'G', 'r', 'a', 'p', 'h']
    (xml2rule xml2name)
    (xml3elements ['e', 'd', 'g', 'e'] xml2scg_position
      (xml2bool ['s', 't', 'r', 'i', 'c', 't']) xml2scg_position
      (\ p s q -> (s, (p, q))))
    (\ lr edges ->
      (lr, (mapa snd (filtera fst edges),
             mapa snd (filtera (\ x -> not (fst x)) edges))));

vec0I :: forall a. a -> Nat -> [a];
vec0I ze n = replicatea n ze;

mat0I :: forall a. a -> Nat -> Nat -> [[a]];
mat0I ze nr nc = replicatea nc (vec0I ze nr);

vec1I :: forall a. a -> a -> Nat -> Nat -> [a];
vec1I ze on n i =
  append (replicatea i ze)
    (on : replicatea (minus_nat (minus_nat n one_nat) i) ze);

mat1I :: forall a. a -> a -> Nat -> [[a]];
mat1I ze on n = mapa (vec1I ze on n) (upt Zero_nat n);

supteqp :: forall a b. (Eq a, Eq b) => Term a b -> Term a b -> Bool;
supteqp s t = eq_term s t || suptp s t;

the_var :: forall a b. Term a b -> b;
the_var (Var x) = x;

map_term :: forall a b c. (a -> b) -> Term a c -> Term b c;
map_term fg (Var x) = Var x;
map_term fg (Fun f ts) = Fun (fg f) (mapa (map_term fg) ts);

map_rule ::
  forall a b c. (a -> b) -> (Term a c, Term a c) -> (Term b c, Term b c);
map_rule fg lr = (map_term fg (fst lr), map_term fg (snd lr));

map_trsI ::
  forall a b c. (a -> b) -> [(Term a c, Term a c)] -> [(Term b c, Term b c)];
map_trsI fg r = mapa (map_rule fg) r;

shows_shp :: forall a. (Shows a) => Shp a -> [Char] -> [Char];
shows_shp (Sharp s) = showsb s . shows_string ['#'];
shows_shp (Plain s) = showsb s;

shows_list_shp :: forall a. (Shows a) => [(Shp a)] -> [Char] -> [Char];
shows_list_shp fs = shows_list_aux shows_shp fs;

instance (Shows a) => Shows (Shp a) where {
  showsb = shows_shp;
  shows_list = shows_list_shp;
};

splitterFun :: (Char -> Bool) -> [Char] -> ([Char], [Char]);
splitterFun allowed (c : s) =
  (if allowed c then let {
                       (l, a) = splitterFun allowed s;
                     } in (c : l, a)
    else ([], c : s));
splitterFun uu [] = ([], []);

parseText :: [Char] -> Eithera [Char] (Maybe [Char], [Char]);
parseText s =
  let {
    (l, r) = splitterFun (\ c -> not ([c] == ['<'])) s;
  } in (if nulla (trim l) then returna (Nothing, r)
         else returna (Just (rev (trim (rev (trim l)))), r));

parseAttribute :: [Char] -> Eithera [Char] ([Char], [Char]);
parseAttribute s =
  bind (consume [dquote] s)
    (\ (_, s1) ->
      let {
        (n, s2) = splitterFun (\ c -> not ([c] == dquote)) s1;
      } in bind (consume [dquote] s2) (\ (_, s3) -> returna (n, s3)));

splitter :: [Char] -> [Char] -> ([Char], [Char]);
splitter allowed s = splitterFun (\ c -> member c allowed) s;

parseName :: [Char] -> Eithera [Char] ([Char], [Char]);
parseName s =
  let {
    (l, r) = splitter letters s;
  } in (if nulla l then parseError s letters else returna (l, trim r));

parseAttributes :: [Char] -> Eithera [Char] ([([Char], [Char])], [Char]);
parseAttributes [] = returna ([], []);
parseAttributes (c : s) =
  (if member c ['/', '>'] then returna ([], c : s)
    else bind (parseName (c : s))
           (\ (n, s1) ->
             bind (consume [['=']] s1)
               (\ (_, s2) ->
                 bind (parseAttribute s2)
                   (\ (a, s3) ->
                     bind (parseAttributes s3)
                       (\ (nas, s4) -> returna ((n, a) : nas, s4))))));

parseXMLList :: [Char] -> Eithera [Char] ([Xml], [Char]);
parseXMLList s =
  (if nulla s || takea (nat 2) s == ['<', '/'] then returna ([], s)
    else bind (consume [['<']] s)
           (\ (_, s1) ->
             bind (parseName s1)
               (\ (n, s2) ->
                 bind (parseAttributes s2)
                   (\ (nas, s3) ->
                     bind (consume [['/', '>'], ['>']] s3)
                       (\ (clo, s4) ->
                         (if clo == ['/', '>']
                           then bind (parseXMLList s4)
                                  (\ (li, s5) ->
                                    returna (Xml n nas [] Nothing : li, s5))
                           else bind (parseText s4)
                                  (\ (t, s5) ->
                                    bind (parseXMLList s5)
                                      (\ (chi, s6) ->
bind (consume [append ['<', '/'] (append n ['>'])] s6)
  (\ (_, s7) ->
    bind (parseXMLList s7)
      (\ (li, s8) -> returna (Xml n nas chi t : li, s8)))))))))));

parseXML :: [Char] -> Eithera [Char] (Xml, [Char]);
parseXML s =
  bind (consume [['<']] s)
    (\ (_, s1) ->
      bind (parseName s1)
        (\ (n, s2) ->
          bind (parseAttributes s2)
            (\ (nas, s3) ->
              bind (consume [['/', '>'], ['>']] s3)
                (\ (clo, s4) ->
                  (if clo == ['/', '>']
                    then returna (Xml n nas [] Nothing, trim s4)
                    else bind (parseText s4)
                           (\ (t, s5) ->
                             bind (parseXMLList s5)
                               (\ (chi, s6) ->
                                 bind (consume
[append ['<', '/'] (append n ['>'])] s6)
                                   (\ (_, s7) ->
                                     returna (Xml n nas chi t, s7)))))))));

scanUpto :: [Char] -> [Char] -> Eithera [Char] ([Char], [Char]);
scanUpto end [] =
  errora
    (append
      ['d', 'i', 'd', ' ', 'n', 'o', 't', ' ', 'f', 'i', 'n', 'd', ' ', 'r',
        'e', 'q', 'u', 'i', 'r', 'e', 'd', ' ', 't', 'o', 'k', 'e', 'n', ' ']
      end);
scanUpto end (c : s) =
  (if mapa snd (zipa end (c : s)) == end
    then returna
           (takea (size_list end) (c : s), trim (dropa (size_list end) (c : s)))
    else bind (scanUpto end s) (\ (res, rest) -> returna (c : res, rest)));

xml2poly ::
  forall a. (Xml -> Eithera [Char] a) -> Xml -> Eithera [Char] (Poly a);
xml2poly xml2coeff x =
  xml1element ['p', 'o', 'l', 'y', 'n', 'o', 'm', 'i', 'a', 'l']
    (xmlOptions
      [(['c', 'o', 'e', 'f', 'f', 'i', 'c', 'i', 'e', 'n', 't'],
         xml1element ['c', 'o', 'e', 'f', 'f', 'i', 'c', 'i', 'e', 'n', 't']
           xml2coeff PNum),
        (['v', 'a', 'r', 'i', 'a', 'b', 'l', 'e'],
          xmlChange (xml2nat ['v', 'a', 'r', 'i', 'a', 'b', 'l', 'e'])
            (\ n -> PVar (minus_nat n one_nat))),
        (['s', 'u', 'm'],
          xmlManyElements ['s', 'u', 'm'] (xml2poly xml2coeff) PSum),
        (['p', 'r', 'o', 'd', 'u', 'c', 't'],
          xmlManyElements ['p', 'r', 'o', 'd', 'u', 'c', 't']
            (xml2poly xml2coeff) PMult)])
    id x;

xml_term :: forall a b. (Shows a, Shows b) => Term a b -> Xml;
xml_term (Var x) = Xml ['v', 'a', 'r'] [] [] (Just (showa x));
xml_term (Fun f ts) =
  Xml ['f', 'u', 'n', 'a', 'p', 'p'] []
    (Xml ['n', 'a', 'm', 'e'] [] [] (Just (showa f)) :
      mapa (\ t -> Xml ['a', 'r', 'g'] [] [xml_term t] Nothing) ts)
    Nothing;

isLeft :: forall a b. Eithera a b -> Bool;
isLeft (Lefta uu) = True;
isLeft (Righta uv) = False;

distinct :: forall a. (Eq a) => [a] -> Bool;
distinct [] = True;
distinct (x : xs) = not (member x xs) && distinct xs;

get_list ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => [(Term a b)] ->
                    Term a b ->
                      [(Term a b, Term a b)] ->
                        Eithera ([Char] -> [Char])
                          [(Term a b,
                             (Ctxt a b, (Sub a b, (Term a b, Term a b))))];
get_list [] u r =
  errora
    (shows_string
      ['L', 'o', 'o', 'p', ' ', 'm', 'u', 's', 't', ' ', 'b', 'e', ' ', 'n',
        'o', 'n', '-', 'e', 'm', 'p', 't', 'y']);
get_list [s] u r = bind (get s u r) (\ y -> returna [y]);
get_list (s : t : ts) u r =
  bind (get s t r)
    (\ d -> bind (get_list (t : ts) u r) (\ y -> returna (d : y)));

get_loop ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => [(Term a b)] ->
                    Ctxt a b ->
                      Sub a b ->
                        [(Term a b, Term a b)] ->
                          Eithera ([Char] -> [Char])
                            [(Term a b,
                               (Ctxt a b, (Sub a b, (Term a b, Term a b))))];
get_loop [] uu uv uw =
  errora
    (shows_string
      ['L', 'o', 'o', 'p', ' ', 'm', 'u', 's', 't', ' ', 'b', 'e', ' ', 'n',
        'o', 'n', '-', 'e', 'm', 'p', 't', 'y']);
get_loop (t : ts) d tau r =
  get_list (t : ts) (ctxt_apply d (sub_apply_term t tau)) r;

wf_vec :: forall a. Nat -> [a] -> Bool;
wf_vec n x = eq_nat (size_list x) n;

wf_mat :: forall a. Nat -> Nat -> [[a]] -> Bool;
wf_mat nr nc m = eq_nat (size_list m) nc && foralla (wf_vec nr) m;

class (Orda a) => Preorder a where {
};

class (Preorder a) => Bot a where {
  bot :: a;
};

maxaa :: forall a. (a -> a -> Bool) -> a -> a -> a;
maxaa less_eq a b = (if less_eq a b then b else a);

maxa :: forall a. (Orda a) => a -> a -> a;
maxa = maxaa less_eq;

minaa :: forall a. (a -> a -> Bool) -> a -> a -> a;
minaa less_eq a b = (if less_eq a b then a else b);

mina :: forall a. (Orda a) => a -> a -> a;
mina = minaa less_eq;

insert_scg ::
  forall a b. (Eq a, Eq b) => Scg a b -> [(Scg a b)] -> Maybe [(Scg a b)];
insert_scg g gs =
  (if eq_scg g Null then Nothing
    else (if list_ex (\ g' -> subsumes g' g) gs then Nothing
           else Just (g : filtera (\ g' -> not (subsumes g g')) gs)));

mk_trancl ::
  forall a b.
    (Eq a,
      Eq b) => (a -> a -> Bool) -> [(Scg a b)] -> [(Scg a b)] -> [(Scg a b)];
mk_trancl conn [] s = s;
mk_trancl conn (g : gs) s =
  case insert_scg g s of {
    Nothing -> mk_trancl conn gs s;
    Just s' ->
      mk_trancl conn
        (append (mapa (scg_comp conn g) s')
          (append (mapa (\ f -> scg_comp conn f g) s') gs))
        s';
  };

check_SCT ::
  forall a b. (Eq a, Eq b) => (a -> a -> Bool) -> [(Scg a b)] -> Bool;
check_SCT conn gs =
  let {
    a = mk_trancl conn gs [];
  } in list_all (sagiv conn) a;

div_int :: Integer -> Integer -> Integer;
div_int a b = fst (divmod_int a b);

normalize :: Rat -> (Integer, Integer);
normalize (Fract a b) =
  (if 0 < b then let {
                   g = gcd_int a b;
                 } in (div_int a g, div_int b g)
    else (if b == 0 then (0, 1)
           else let {
                  g = negate (gcd_int a b);
                } in (div_int a g, div_int b g)));

divmod_nat :: Nat -> Nat -> (Nat, Nat);
divmod_nat m n =
  (if eq_nat n Zero_nat || less_nat m n then (Zero_nat, m)
    else let {
           (q, a) = divmod_nat (minus_nat m n) n;
         } in (Suc q, a));

mod_nat :: Nat -> Nat -> Nat;
mod_nat m n = snd (divmod_nat m n);

div_nat :: Nat -> Nat -> Nat;
div_nat m n = fst (divmod_nat m n);

digit2string :: Nat -> [Char];
digit2string n =
  (if eq_nat n Zero_nat then ['0']
    else (if eq_nat n one_nat then ['1']
           else (if eq_nat n (nat 2) then ['2']
                  else (if eq_nat n (nat 3) then ['3']
                         else (if eq_nat n (nat 4) then ['4']
                                else (if eq_nat n (nat 5) then ['5']
                                       else (if eq_nat n (nat 6) then ['6']
      else (if eq_nat n (nat 7) then ['7']
             else (if eq_nat n (nat 8) then ['8']
                    else (if eq_nat n (nat 9) then ['9']
                           else ['n', 'o', 't', ' ', 'a', ' ', 'd', 'i', 'g',
                                  'i', 't']))))))))));

shows_nat :: Nat -> [Char] -> [Char];
shows_nat n =
  (if less_nat n (nat 10) then shows_string (digit2string n)
    else shows_nat (div_nat n (nat 10)) .
           shows_string (digit2string (mod_nat n (nat 10))));

shows_int :: Integer -> [Char] -> [Char];
shows_int i =
  (if i < 0 then shows_char '-' . shows_nat (nat (0 - i))
    else shows_nat (nat i));

show_rat :: Rat -> [Char];
show_rat x =
  let {
    (den, num) = normalize x;
  } in (if num == 1 then shows_int den []
         else ((shows_int den . shows_char '/') . shows_int num) []);

shows_list_int :: [Integer] -> [Char] -> [Char];
shows_list_int is = shows_list_aux shows_int is;

instance Shows Integer where {
  showsb = shows_int;
  shows_list = shows_list_int;
};

shows_list_nat :: [Nat] -> [Char] -> [Char];
shows_list_nat ns = shows_list_aux shows_nat ns;

instance Shows Nat where {
  showsb = shows_nat;
  shows_list = shows_list_nat;
};

shows_rat :: Rat -> [Char] -> [Char];
shows_rat x = shows_string (show_rat x);

shows_list_rat :: [Rat] -> [Char] -> [Char];
shows_list_rat rs = shows_list_aux shows_rat rs;

instance Shows Rat where {
  showsb = shows_rat;
  shows_list = shows_list_rat;
};

defs_list :: forall a b. (Eq a) => [(Term a b, Term a b)] -> [a];
defs_list [] = [];
defs_list ((Fun f uu, uv) : ts) =
  let {
    fs = defs_list ts;
  } in (if member f fs then fs else f : fs);
defs_list ((Var uw, ux) : ts) = defs_list ts;

shows_trs ::
  forall a b. (Shows a, Shows b) => [(Term a b, Term a b)] -> [Char] -> [Char];
shows_trs r =
  shows_string
    ['r', 'e', 'w', 'r', 'i', 't', 'e', ' ', 's', 'y', 's', 't', 'e', 'm', ':']
    . shows_nl . shows_rules r;

shows_dpp ::
  forall a b.
    (Shows a,
      Shows b) => [(Term a b, Term a b)] ->
                    [(Term a b, Term a b)] -> [Char] -> [Char];
shows_dpp p r = shows_p p . shows_trs r;

rev_append :: forall a. [a] -> [a] -> [a];
rev_append [] ys = ys;
rev_append (x : xs) ys = rev_append xs (x : ys);

flat_map_aux :: forall a b. (a -> [b]) -> [a] -> [b] -> [b];
flat_map_aux f [] ys = rev ys;
flat_map_aux f (x : xs) ys = flat_map_aux f xs (rev_append (f x) ys);

flat_map :: forall a b. (a -> [b]) -> [a] -> [b];
flat_map f xs = flat_map_aux f xs [];

max_list :: [Nat] -> Nat;
max_list [] = Zero_nat;
max_list (x : xs) = maxa x (max_list xs);

fitLength :: forall a. a -> Nat -> [a] -> [a];
fitLength uu Zero_nat uv = [];
fitLength a (Suc v) [] = replicatea (Suc v) a;
fitLength a (Suc n) (b : asa) = b : fitLength a n asa;

sum_lpoly :: forall a. (a -> a -> a) -> [a] -> [a] -> [a];
sum_lpoly uu [] ys = ys;
sum_lpoly uv (v : va) [] = v : va;
sum_lpoly pl (x : xs) (y : ys) = pl x y : sum_lpoly pl xs ys;

xml2label ::
  forall a.
    (Xml -> Eithera [Char] a) -> Xml -> Eithera [Char] (Eithera [Nat] [a]);
xml2label xml2name =
  xmlOptions
    [(['n', 'u', 'm', 'b', 'e', 'r', 'L', 'a', 'b', 'e', 'l'],
       xmlManyElements ['n', 'u', 'm', 'b', 'e', 'r', 'L', 'a', 'b', 'e', 'l']
         (xml2nat ['n', 'u', 'm', 'b', 'e', 'r']) Lefta),
      (['s', 'y', 'm', 'b', 'o', 'l', 'L', 'a', 'b', 'e', 'l'],
        xmlManyElements ['s', 'y', 'm', 'b', 'o', 'l', 'L', 'a', 'b', 'e', 'l']
          xml2name Righta)];

xml2rules ::
  forall a.
    (Xml -> Eithera [Char] a) ->
      Xml -> Eithera [Char] [(Term a [Char], Term a [Char])];
xml2rules xml2name =
  xmlManyElements ['r', 'u', 'l', 'e', 's'] (xml2rule xml2name) id;

xml_rules :: forall a b. (Shows a, Shows b) => [(Term a b, Term a b)] -> Xml;
xml_rules rls =
  Xml ['r', 'u', 'l', 'e', 's'] []
    (mapa (\ (l, r) ->
            Xml ['r', 'u', 'l', 'e'] []
              [Xml ['l', 'h', 's'] [] [xml_term l] Nothing,
                Xml ['r', 'h', 's'] [] [xml_term r] Nothing]
              Nothing)
      rls)
    Nothing;

partition :: forall a. (a -> Bool) -> [a] -> ([a], [a]);
partition p [] = ([], []);
partition p (x : xs) =
  let {
    (yes, no) = partition p xs;
  } in (if p x then (x : yes, no) else (yes, x : no));

removeAll :: forall a. (Eq a) => a -> [a] -> [a];
removeAll x [] = [];
removeAll x (y : xs) = (if x == y then removeAll x xs else y : removeAll x xs);

rewrites_to ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => [(Term a b, (Ctxt a b, (Sub a b, (Term a b, Term a b))))] ->
                    Term a b ->
                      [(Term a b, Term a b)] -> Eithera ([Char] -> [Char]) ();
rewrites_to [(s, (c, (sigma, rule)))] t r = rewrites (s, t) c sigma rule r;
rewrites_to ((s, (c, (sigma, rule))) : (t, (d, (tau, rule'))) : xs) u r =
  bind (rewrites (s, t) c sigma rule r)
    (\ _ -> rewrites_to ((t, (d, (tau, rule'))) : xs) u r);

check_loop_d ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => [(Term a b, (Ctxt a b, (Sub a b, (Term a b, Term a b))))] ->
                    Ctxt a b ->
                      Sub a b ->
                        [(Term a b, Term a b)] -> Eithera ([Char] -> [Char]) ();
check_loop_d [] uu uv uw =
  errora
    (shows_string
      ['T', 'h', 'e', 'r', 'e', ' ', 'm', 'u', 's', 't', ' ', 'b', 'e', ' ',
        'a', 't', ' ', 'l', 'e', 'a', 's', 't', ' ', 'o', 'n', 'e', ' ', 'r',
        'e', 'w', 'r', 'i', 't', 'e', ' ', 's', 't', 'e', 'p']);
check_loop_d (v : va) d tau r =
  rewrites_to (v : va) (ctxt_apply d (sub_apply_term (fst (hd (v : va))) tau))
    r;

check_loop ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => ([(Term a b)], (Ctxt a b, Sub a b)) ->
                    [(Term a b, Term a b)] -> Eithera ([Char] -> [Char]) ();
check_loop (ts, (c, sigma)) r =
  bind (get_loop ts c sigma r) (\ ls -> check_loop_d ls c sigma r);

catcha :: forall a b c. Eithera a b -> (a -> Eithera c b) -> Eithera c b;
catcha p eq = case p of {
                Lefta a -> eq a;
                Righta a -> Righta a;
              };

checkLoop ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => [(Term a b, Term a b)] ->
                    ([(Term a b)], (Ctxt a b, [(Term a b, Term a b)])) ->
                      Eithera ([Char] -> [Char]) ();
checkLoop r (ts, (c, sub)) =
  catcha
    (case subst_safe [] sub sub_empty of {
       Nothing ->
         errora
           (shows_string
             ['s', 'u', 'b', 's', 't', 'i', 't', 'u', 't', 'i', 'o', 'n', ' ',
               'n', 'o', 't', ' ', 'u', 'n', 'i', 'q', 'u', 'e']);
       Just sigma -> check_loop (ts, (c, sigma)) r;
     })
    (\ x ->
      errora
        (shows_string
           ['e', 'r', 'r', 'o', 'r', ' ', 'i', 'n', ' ', 'c', 'e', 'r', 't',
             'i', 'f', 'i', 'c', 'a', 't', 'i', 'o', 'n', ' ', 'o', 'f', ' ',
             'l', 'o', 'o', 'p']
          . shows_nl . x));

vec_geI :: forall a. (a -> a -> Bool) -> [a] -> [a] -> Bool;
vec_geI ge v w = foralla (\ (a, b) -> ge a b) (zipa v w);

mat_geI :: forall a. (a -> a -> Bool) -> [[a]] -> [[a]] -> Bool;
mat_geI ge m1 m2 = foralla (\ (a, b) -> vec_geI ge a b) (zipa m1 m2);

sub_vec :: forall a. Nat -> [a] -> [a];
sub_vec = takea;

sub_mat :: forall a. Nat -> Nat -> [[a]] -> [[a]];
sub_mat nr nc m = mapa (sub_vec nr) (takea nc m);

vec_pre_gtI :: forall a. (a -> a -> Bool) -> [a] -> [a] -> Bool;
vec_pre_gtI gt v w = anya (\ (a, b) -> gt a b) (zipa v w);

mat_pre_gtI :: forall a. (a -> a -> Bool) -> [[a]] -> [[a]] -> Bool;
mat_pre_gtI gt m1 m2 = anya (\ (a, b) -> vec_pre_gtI gt a b) (zipa m1 m2);

mat_gtI ::
  forall a.
    (a -> a -> Bool) -> (a -> a -> Bool) -> Nat -> [[a]] -> [[a]] -> Bool;
mat_gtI ge gt sd m1 m2 =
  mat_geI ge m1 m2 && mat_pre_gtI gt (sub_mat sd sd m1) (sub_mat sd sd m2);

lpo_strict ::
  forall a b.
    (Shows a, Eq b,
      Shows b) => (a -> Nat -> Nat) ->
                    Nat ->
                      (Term a b, Term a b) -> Eithera ([Char] -> [Char]) ();
lpo_strict pr n =
  (\ (s, t) ->
    checkB (fst (lpo pr n s t))
      (shows_string
         ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'o', 'r', 'i', 'e',
           'n', 't', ' ']
        . shows_term s .
            shows_string [' ', '>', 'L', 'P', 'O', ' '] .
              shows_term t . shows_nl));

eq_rat :: Rat -> Rat -> Bool;
eq_rat (Fract a b) (Fract c d) =
  (if b == 0 then c == 0 || d == 0
    else (if d == 0 then a == 0 || b == 0 else a * d == b * c));

instance Eq Rat where {
  a == b = eq_rat a b;
};

shows_list_list :: forall a. (Shows a) => [[a]] -> [Char] -> [Char];
shows_list_list ls = shows_list_aux shows_lista ls;

instance (Shows a) => Shows [a] where {
  showsb = shows_lista;
  shows_list = shows_list_list;
};

ctxt_case ::
  forall a b c.
    a -> (b -> [(Term b c)] -> Ctxt b c -> [(Term b c)] -> a) -> Ctxt b c -> a;
ctxt_case f1 f2 (More f list1 ctxt list2) = f2 f list1 ctxt list2;
ctxt_case f1 f2 Hole = f1;

shows_list_term ::
  forall a b. (Shows a, Shows b) => [(Term a b)] -> [Char] -> [Char];
shows_list_term is = shows_list_aux shows_term is;

instance (Shows a, Shows b) => Shows (Term a b) where {
  showsb = shows_term;
  shows_list = shows_list_term;
};

list_inter :: forall a. (Eq a) => [a] -> [a] -> [a];
list_inter [] bs = [];
list_inter (a : asa) bs =
  (if member a bs then a : list_inter asa bs else list_inter asa bs);

lpo_nstrict ::
  forall a b.
    (Shows a, Eq b,
      Shows b) => (a -> Nat -> Nat) ->
                    Nat ->
                      (Term a b, Term a b) -> Eithera ([Char] -> [Char]) ();
lpo_nstrict pr n =
  (\ (s, t) ->
    checkB (snd (lpo pr n s t))
      (shows_string
         ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'o', 'r', 'i', 'e',
           'n', 't', ' ']
        . shows_term s .
            shows_string [' ', '>', '=', 'L', 'P', 'O', ' '] .
              shows_term t . shows_nl));

less_rat :: Rat -> Rat -> Bool;
less_rat (Fract a b) (Fract c d) =
  (if b == 0 then 0 < sgn_int c * sgn_int d
    else (if d == 0 then sgn_int a * sgn_int b < 0
           else a * abs_int d * sgn_int b < c * abs_int b * sgn_int d));

less_eq_rat :: Rat -> Rat -> Bool;
less_eq_rat (Fract a b) (Fract c d) =
  (if b == 0 then 0 <= sgn_int c * sgn_int d
    else (if d == 0 then sgn_int a * sgn_int b <= 0
           else a * abs_int d * sgn_int b <= c * abs_int b * sgn_int d));

instance Orda Rat where {
  less_eq = less_eq_rat;
  less = less_rat;
};

data Arctic_rat = MinInfty_rat | Num_arc_rat Rat;

data Arctic = MinInfty | Num_arc Integer;

data Redpair a = Nat_carrier [((a, Nat), (Nat, [Nat]))]
  | Int_carrier [((a, Nat), (Integer, [Integer]))]
  | Arctic_carrier [((a, Nat), (Arctic, [Arctic]))]
  | Arctic_rat_carrier Rat [((a, Nat), (Arctic_rat, [Arctic_rat]))]
  | Int_mat_carrier Nat Nat [((a, Nat), ([[Integer]], [[[Integer]]]))]
  | Arc_mat_carrier Nat [((a, Nat), ([[Arctic]], [[[Arctic]]]))]
  | Arc_rat_mat_carrier Rat Nat [((a, Nat), ([[Arctic_rat]], [[[Arctic_rat]]]))]
  | Lpo [((a, Nat), Nat)] [((a, Nat), Af_entry)]
  | Rat_carrier Rat [((a, Nat), (Rat, [Rat]))]
  | Rat_mat_carrier Rat Nat Nat [((a, Nat), ([[Rat]], [[[Rat]]]))];

shows_ctxt :: forall a b. (Shows a, Shows b) => Ctxt a b -> [Char] -> [Char];
shows_ctxt Hole = shows_string ['[', ']'];
shows_ctxt (More f ss1 d ss2) =
  showsb f .
    shows_string ['('] .
      shows_list_gen shows_term [] [] [','] [','] ss1 .
        shows_ctxt d . shows_list_gen shows_term [')'] [','] [','] [')'] ss2;

list_union :: forall a. (Eq a) => [a] -> [a] -> [a];
list_union [] ys = ys;
list_union (x : xs) ys =
  let {
    zs = list_union xs ys;
  } in (if member x zs then zs else x : zs);

domain_case ::
  forall a.
    a -> a -> (Rat -> a) ->
                (Nat -> Nat -> a) ->
                  (Nat -> a) ->
                    (Rat -> Nat -> a) ->
                      (Rat -> a) -> (Rat -> Nat -> Nat -> a) -> Domain -> a;
domain_case f1 f2 f3 f4 f5 f6 f7 f8 (Rat_mat rat nat1 nat2) = f8 rat nat1 nat2;
domain_case f1 f2 f3 f4 f5 f6 f7 f8 (Rationala rat) = f7 rat;
domain_case f1 f2 f3 f4 f5 f6 f7 f8 (Arc_rat_mat rat nat) = f6 rat nat;
domain_case f1 f2 f3 f4 f5 f6 f7 f8 (Arc_mat nat) = f5 nat;
domain_case f1 f2 f3 f4 f5 f6 f7 f8 (Int_mat nat1 nat2) = f4 nat1 nat2;
domain_case f1 f2 f3 f4 f5 f6 f7 f8 (Arctic_rat rat) = f3 rat;
domain_case f1 f2 f3 f4 f5 f6 f7 f8 Arctic = f2;
domain_case f1 f2 f3 f4 f5 f6 f7 f8 Natural = f1;

data Lab a b = Lab (Lab a b) b | FunLab (Lab a b) [(Lab a b)] | UnLab a;

id_labeling :: forall a b. Lab a [b] -> [b] -> Eithera [b] [(Lab a [b])];
id_labeling uu cs = Lefta cs;

remove_comments_help :: Bool -> [Char] -> [Char];
remove_comments_help False (c : cs) =
  (if c == '<' && takea (nat 3) cs == ['!', '-', '-']
    then remove_comments_help True (tl cs)
    else c : remove_comments_help False cs);
remove_comments_help True (c : cs) =
  (if c == '-' && takea (nat 2) cs == ['-', '>']
    then remove_comments_help False (dropa (nat 2) cs)
    else remove_comments_help True cs);
remove_comments_help uu [] = [];

remove_comments :: [Char] -> [Char];
remove_comments = remove_comments_help False;

stripXMLHeader :: [Char] -> Eithera [Char] ([Char], [Char]);
stripXMLHeader s =
  (if takea (nat 2) (trim s) == ['<', '?']
    then bind (scanUpto ['?', '>'] s)
           (\ (header1, s1) ->
             bind (stripXMLHeader s1)
               (\ (header2, s2) -> returna (append header1 header2, trim s2)))
    else returna ([], trim s));

debug :: forall a. [Char] -> [Char] -> a -> a;
debug i t x = x;

parseXMLDoc :: [Char] -> Eithera [Char] Xmldoc;
parseXMLDoc s =
  debug ['0']
    ['p', 'a', 'r', 's', 'i', 'n', 'g', ' ', 't', 'x', 't', ' ', 't', 'o', ' ',
      'x', 'm', 'l']
    (bind (stripXMLHeader (remove_comments s))
      (\ (header, rest) ->
        bind (parseXML rest)
          (\ (xml, end) ->
            (if nulla end then returna (Xmldoc header xml)
              else errora
                     (append
                       ['t', 'h', 'e', 'r', 'e', ' ', 'i', 's', ' ', 't', 'e',
                         'x', 't', ' ', 'a', 'f', 't', 'e', 'r', ' ', 'r', 'o',
                         'o', 't', ' ', 'e', 'l', 'e', 'm', 'e', 'n', 't', ':',
                         ' ']
                       (newline : end))))));

xml4elements ::
  forall a b c d e.
    [Char] ->
      (Xml -> Eithera [Char] a) ->
        (Xml -> Eithera [Char] b) ->
          (Xml -> Eithera [Char] c) ->
            (Xml -> Eithera [Char] d) ->
              (a -> b -> c -> d -> e) -> Xml -> Eithera [Char] e;
xml4elements reqName p1 p2 p3 p4 combi (Xml name atts cs text) =
  (if name == reqName &&
        (nulla atts && (eq_nat (size_list cs) (nat 4) && is_none text))
    then bind (p1 (nth cs Zero_nat))
           (\ a ->
             bind (p2 (nth cs one_nat))
               (\ b ->
                 bind (p3 (nth cs (nat 2)))
                   (\ c -> bind (p4 (nth cs (nat 3))) (returna . combi a b c))))
    else errora
           (concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], [newline], showa (Xml name atts cs text)]));

xml0elements :: forall a. [Char] -> a -> Xml -> Eithera [Char] a;
xml0elements reqName combi (Xml name atts cs text) =
  (if name == reqName && (nulla atts && (nulla cs && is_none text))
    then returna combi
    else errora
           (concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], [newline], showa (Xml name atts cs text)]));

xml2context ::
  forall a. (Xml -> Eithera [Char] a) -> Xml -> Eithera [Char] (Ctxt a [Char]);
xml2context xml2name x =
  xmlOptions
    [(['b', 'o', 'x'], xml0elements ['b', 'o', 'x'] Hole),
      (['f', 'u', 'n', 'C', 'o', 'n', 't', 'e', 'x', 't'],
        xml4elements ['f', 'u', 'n', 'C', 'o', 'n', 't', 'e', 'x', 't'] xml2name
          (xmlManyElements ['b', 'e', 'f', 'o', 'r', 'e'] (xml2term xml2name)
            id)
          (xml2context xml2name)
          (xmlManyElements ['a', 'f', 't', 'e', 'r'] (xml2term xml2name) id)
          More)]
    x;

data ArithFun = Arg Nat | Const Nat | Sum [ArithFun] | Max [ArithFun]
  | Min [ArithFun] | Prod [ArithFun];

data Sl_inter a b = SL_Inter Nat [((Lab a b, Nat), ArithFun)];

data DPProof a b c = PisEmpty
  | RedPairProc (Redpair (Lab a b)) [(Term (Lab a b) c, Term (Lab a b) c)]
      (DPProof a b c)
  | RedPairUrProc (Redpair (Lab a b)) [(Term (Lab a b) c, Term (Lab a b) c)]
      [(Term (Lab a b) c, Term (Lab a b) c)] (DPProof a b c)
  | DepGraphProc
      [(Maybe (DPProof a b c), [(Term (Lab a b) c, Term (Lab a b) c)])]
  | MonoRedPairProc (Redpair (Lab a b)) [(Term (Lab a b) c, Term (Lab a b) c)]
      [(Term (Lab a b) c, Term (Lab a b) c)] (DPProof a b c)
  | MonoRedPairUrProc (Redpair (Lab a b)) [(Term (Lab a b) c, Term (Lab a b) c)]
      [(Term (Lab a b) c, Term (Lab a b) c)]
      [(Term (Lab a b) c, Term (Lab a b) c)] (DPProof a b c)
  | UnlabProc [(Term (Lab a b) c, Term (Lab a b) c)]
      [(Term (Lab a b) c, Term (Lab a b) c)] (DPProof a b c)
  | SemlabProc (Sl_inter a b) (Lab a b -> [Nat] -> Eithera b [(Lab a b)])
      [(Term (Lab a b) c, Term (Lab a b) c)]
      [(Term (Lab a b) c, Term (Lab a b) c)] (DPProof a b c)
  | RootlabProc [(Term (Lab a b) c, Term (Lab a b) c)]
      [(Term (Lab a b) c, Term (Lab a b) c)] (DPProof a b c)
  | SpscProc (SpL (Lab a b)) [(Term (Lab a b) c, Term (Lab a b) c)]
      (DPProof a b c)
  | SizeChangeSubTerm
      [((Term (Lab a b) c, Term (Lab a b) c), ([(Nat, Nat)], [(Nat, Nat)]))]
  | SizeChangeRedPair (Redpair (Lab a b)) [(Term (Lab a b) c, Term (Lab a b) c)]
      [((Term (Lab a b) c, Term (Lab a b) c), ([(Nat, Nat)], [(Nat, Nat)]))]
  | FccProc (Lab a b) [(Ctxt (Lab a b) c)]
      [(Term (Lab a b) c, Term (Lab a b) c)]
      [(Term (Lab a b) c, Term (Lab a b) c)] (DPProof a b c);

xml2flat_contexts ::
  forall a.
    (Xml -> Eithera [Char] a) -> Xml -> Eithera [Char] [(Ctxt a [Char])];
xml2flat_contexts xml2name =
  xmlManyElements ['f', 'l', 'a', 't', 'C', 'o', 'n', 't', 'e', 'x', 't', 's']
    (xml2context xml2name) id;

xml2arith_fun :: Xml -> Eithera [Char] ArithFun;
xml2arith_fun xml =
  xml1element ['a', 'r', 'i', 't', 'h', 'F', 'u', 'n', 'c', 't', 'i', 'o', 'n']
    (xmlOptions
      [(['n', 'a', 't', 'u', 'r', 'a', 'l'],
         xmlChange (xml2nat ['n', 'a', 't', 'u', 'r', 'a', 'l']) Const),
        (['v', 'a', 'r', 'i', 'a', 'b', 'l', 'e'],
          xmlChange (xml2nat ['v', 'a', 'r', 'i', 'a', 'b', 'l', 'e'])
            (\ n -> Arg (minus_nat n one_nat))),
        (['s', 'u', 'm'], xmlManyElements ['s', 'u', 'm'] xml2arith_fun Sum),
        (['p', 'r', 'o', 'd', 'u', 'c', 't'],
          xmlManyElements ['p', 'r', 'o', 'd', 'u', 'c', 't'] xml2arith_fun
            Prod),
        (['m', 'i', 'n'], xmlManyElements ['m', 'i', 'n'] xml2arith_fun Min),
        (['m', 'a', 'x'], xmlManyElements ['m', 'a', 'x'] xml2arith_fun Max)])
    id xml;

xml2sl_inter ::
  forall a b.
    (Xml -> Eithera [Char] (Lab a b)) ->
      Xml -> Eithera [Char] (Maybe (Sl_inter a b));
xml2sl_inter xml2name =
  xml1element ['m', 'o', 'd', 'e', 'l']
    (xmlOptions
      [(['f', 'i', 'n', 'i', 't', 'e', 'M', 'o', 'd', 'e', 'l'],
         xml1manyElements
           ['f', 'i', 'n', 'i', 't', 'e', 'M', 'o', 'd', 'e', 'l']
           (xmlChange
             (xml2nat ['c', 'a', 'r', 'r', 'i', 'e', 'r', 'S', 'i', 'z', 'e'])
             (\ n -> minus_nat n one_nat))
           (xml3elements ['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't'] xml2name
             (xml2nat ['a', 'r', 'i', 't', 'y']) xml2arith_fun
             (\ f a -> (\ b -> ((f, a), b))))
           (\ n xs -> Just (SL_Inter n xs))),
        (['r', 'o', 'o', 't', 'L', 'a', 'b', 'e', 'l', 'i', 'n', 'g'],
          xml0elements
            ['r', 'o', 'o', 't', 'L', 'a', 'b', 'e', 'l', 'i', 'n', 'g']
            Nothing)])
    id;

xml5elements ::
  forall a b c d e f.
    [Char] ->
      (Xml -> Eithera [Char] a) ->
        (Xml -> Eithera [Char] b) ->
          (Xml -> Eithera [Char] c) ->
            (Xml -> Eithera [Char] d) ->
              (Xml -> Eithera [Char] e) ->
                (a -> b -> c -> d -> e -> f) -> Xml -> Eithera [Char] f;
xml5elements reqName p1 p2 p3 p4 p5 combi (Xml name atts cs text) =
  (if name == reqName &&
        (nulla atts && (eq_nat (size_list cs) (nat 5) && is_none text))
    then bind (p1 (nth cs Zero_nat))
           (\ a ->
             bind (p2 (nth cs one_nat))
               (\ b ->
                 bind (p3 (nth cs (nat 2)))
                   (\ c ->
                     bind (p4 (nth cs (nat 3)))
                       (\ d ->
                         bind (p5 (nth cs (nat 4)))
                           (returna . combi a b c d)))))
    else errora
           (concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], [newline], showa (Xml name atts cs text)]));

xml2precedence ::
  forall a.
    (Xml -> Eithera [Char] a) -> Xml -> Eithera [Char] [((a, Nat), Nat)];
xml2precedence xml2name =
  xmlManyElements
    ['s', 't', 'a', 't', 'u', 's', 'P', 'r', 'e', 'c', 'e', 'd', 'e', 'n', 'c',
      'e']
    (xml4elements
      ['s', 't', 'a', 't', 'u', 's', 'P', 'r', 'e', 'c', 'e', 'd', 'e', 'n',
        'c', 'e', 'E', 'n', 't', 'r', 'y']
      xml2name (xml2nat ['a', 'r', 'i', 't', 'y'])
      (xml2nat ['p', 'r', 'e', 'c', 'e', 'd', 'e', 'n', 'c', 'e'])
      (xml0elements ['l', 'e', 'x'] ()) (\ f a p _ -> ((f, a), p)))
    id;

xml1to2elements ::
  forall a b c.
    [Char] ->
      (Xml -> Eithera [Char] a) ->
        (Xml -> Eithera [Char] b) ->
          (a -> Maybe b -> c) -> Xml -> Eithera [Char] c;
xml1to2elements reqName p1 p2 combi (Xml name atts cs text) =
  (if name == reqName &&
        (nulla atts &&
          (less_eq_nat one_nat (size_list cs) &&
            (less_eq_nat (size_list cs) (nat 2) && is_none text)))
    then bind (p1 (nth cs Zero_nat))
           (\ a ->
             (if eq_nat (size_list cs) (nat 2)
               then bind (p2 (nth cs one_nat))
                      (\ b -> returna (combi a (Just b)))
               else returna (combi a Nothing)))
    else errora
           (concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], [newline], showa (Xml name atts cs text)]));

xml2path_order ::
  forall a. (Xml -> Eithera [Char] a) -> Xml -> Eithera [Char] (Redpair a);
xml2path_order xml2name =
  xml1to2elements ['p', 'a', 't', 'h', 'O', 'r', 'd', 'e', 'r']
    (xml2precedence xml2name) (xml2afs xml2name)
    (\ pr a -> case a of {
                 Nothing -> Lpo pr [];
                 Just aa -> Lpo pr aa;
               });

rat_name :: [Char];
rat_name =
  ['r', 'a', 't', 'i', 'o', 'n', 'a', 'l', 's', ' ', '(', 'w', 'i', 't', 'h',
    ' ', 'n', 'e', 'g', 'a', 't', 'i', 'v', 'e', ' ', 'c', 'o', 'n', 's', 't',
    'a', 'n', 't', 's', ')'];

fract_norm :: Integer -> Integer -> Rat;
fract_norm a b =
  (if a == 0 || b == 0 then Fract 0 1
    else let {
           c = gcd_int a b;
         } in (if 0 < b then Fract (div_int a c) (div_int b c)
                else Fract (negate (div_int a c)) (negate (div_int b c))));

minus_rat :: Rat -> Rat -> Rat;
minus_rat (Fract a b) (Fract c d) =
  (if b == 0 then Fract (negate c) d
    else (if d == 0 then Fract a b else fract_norm (a * d - c * b) (b * d)));

rat_gt :: Rat -> Rat -> Rat -> Bool;
rat_gt delta x y =
  less_eq_rat
    (if less_rat (Fract 0 1) delta then mina delta (Fract 1 1) else Fract 1 1)
    (minus_rat x y);

ge_rat :: Rat -> Rat -> Bool;
ge_rat x y = less_eq_rat y x;

max0_rat :: Rat -> Rat;
max0_rat x = maxa (Fract 0 1) x;

times_rat :: Rat -> Rat -> Rat;
times_rat (Fract a b) (Fract c d) = fract_norm (a * c) (b * d);

plus_rat :: Rat -> Rat -> Rat;
plus_rat (Fract a b) (Fract c d) =
  (if b == 0 then Fract c d
    else (if d == 0 then Fract a b else fract_norm (a * d + c * b) (b * d)));

rat_components ::
  Rat ->
    (Rat -> Bool,
      (Rat, (Rat, (Rat -> Rat -> Rat,
                    (Rat -> Rat -> Rat,
                      (Rat -> Rat,
                        (Rat -> Rat -> Bool,
                          (Rat -> Rat -> Bool,
                            (Bool, ([Char], Rat -> Bool))))))))));
rat_components delta =
  ((\ _ -> True),
    (Fract 0 1,
      (Fract 1 1,
        (plus_rat,
          (times_rat,
            (max0_rat,
              (ge_rat, (rat_gt delta, (True, (rat_name, (\ _ -> True)))))))))));

get_comp_apos ::
  forall a.
    (a -> Bool,
      (a, (a, (a -> a -> a,
                (a -> a -> a,
                  (a -> a,
                    (a -> a -> Bool,
                      (a -> a -> Bool, (Bool, ([Char], a -> Bool))))))))))
      -> a -> Bool;
get_comp_apos (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) =
  ap;

mat_arc_posI :: forall a. (a -> Bool) -> [[a]] -> Bool;
mat_arc_posI ap m = ap (nth (nth m Zero_nat) Zero_nat);

get_comp_description ::
  forall a.
    (a -> Bool,
      (a, (a, (a -> a -> a,
                (a -> a -> a,
                  (a -> a,
                    (a -> a -> Bool,
                      (a -> a -> Bool, (Bool, ([Char], a -> Bool))))))))))
      -> [Char];
get_comp_description
  (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) = de;

get_comp_gt ::
  forall a.
    (a -> Bool,
      (a, (a, (a -> a -> a,
                (a -> a -> a,
                  (a -> a,
                    (a -> a -> Bool,
                      (a -> a -> Bool, (Bool, ([Char], a -> Bool))))))))))
      -> a -> a -> Bool;
get_comp_gt (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) =
  gt;

get_comp_ge ::
  forall a.
    (a -> Bool,
      (a, (a, (a -> a -> a,
                (a -> a -> a,
                  (a -> a,
                    (a -> a -> Bool,
                      (a -> a -> Bool, (Bool, ([Char], a -> Bool))))))))))
      -> a -> a -> Bool;
get_comp_ge (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) =
  ge;

get_comp_times ::
  forall a.
    (a -> Bool,
      (a, (a, (a -> a -> a,
                (a -> a -> a,
                  (a -> a,
                    (a -> a -> Bool,
                      (a -> a -> Bool, (Bool, ([Char], a -> Bool))))))))))
      -> a -> a -> a;
get_comp_times (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) =
  ti;

transpose :: forall a. Nat -> [[a]] -> [[a]];
transpose nr [] = replicatea nr [];
transpose nr (v : m) = mapa (\ (a, b) -> a : b) (zipa v (transpose nr m));

scalar_prodI ::
  forall a. a -> (a -> a -> a) -> (a -> a -> a) -> [a] -> [a] -> a;
scalar_prodI ze pl ti v w = foldra (\ (x, y) -> pl (ti x y)) (zipa v w) ze;

matT_vec_multI ::
  forall a. a -> (a -> a -> a) -> (a -> a -> a) -> [[a]] -> [a] -> [a];
matT_vec_multI ze pl ti m v = mapa (\ w -> scalar_prodI ze pl ti w v) m;

mat_multI ::
  forall a.
    a -> (a -> a -> a) -> (a -> a -> a) -> Nat -> [[a]] -> [[a]] -> [[a]];
mat_multI ze pl ti nr m1 m2 =
  mapa (matT_vec_multI ze pl ti (transpose nr m1)) m2;

get_comp_plus ::
  forall a.
    (a -> Bool,
      (a, (a, (a -> a -> a,
                (a -> a -> a,
                  (a -> a,
                    (a -> a -> Bool,
                      (a -> a -> Bool, (Bool, ([Char], a -> Bool))))))))))
      -> a -> a -> a;
get_comp_plus (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) =
  pl;

vec_plusI :: forall a. (a -> a -> a) -> [a] -> [a] -> [a];
vec_plusI pl v w = mapa (\ xy -> pl (fst xy) (snd xy)) (zipa v w);

mat_plusI :: forall a. (a -> a -> a) -> [[a]] -> [[a]] -> [[a]];
mat_plusI pl m1 m2 = mapa (\ uv -> vec_plusI pl (fst uv) (snd uv)) (zipa m1 m2);

get_comp_1 ::
  forall a.
    (a -> Bool,
      (a, (a, (a -> a -> a,
                (a -> a -> a,
                  (a -> a,
                    (a -> a -> Bool,
                      (a -> a -> Bool, (Bool, ([Char], a -> Bool))))))))))
      -> a;
get_comp_1 (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) = on;

get_comp_0 ::
  forall a.
    (a -> Bool,
      (a, (a, (a -> a -> a,
                (a -> a -> a,
                  (a -> a,
                    (a -> a -> Bool,
                      (a -> a -> Bool, (Bool, ([Char], a -> Bool))))))))))
      -> a;
get_comp_0 (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) = ze;

mat_arctic_components ::
  forall a.
    Nat ->
      (a -> Bool,
        (a, (a, (a -> a -> a,
                  (a -> a -> a,
                    (a -> a,
                      (a -> a -> Bool,
                        (a -> a -> Bool, (Bool, ([Char], a -> Bool))))))))))
        -> ([[a]] -> Bool,
             ([[a]],
               ([[a]],
                 ([[a]] -> [[a]] -> [[a]],
                   ([[a]] -> [[a]] -> [[a]],
                     ([[a]] -> [[a]],
                       ([[a]] -> [[a]] -> Bool,
                         ([[a]] -> [[a]] -> Bool,
                           (Bool, ([Char], [[a]] -> Bool))))))))));
mat_arctic_components n c =
  let {
    np = Suc (minus_nat n one_nat);
  } in (wf_mat np np,
         (mat0I (get_comp_0 c) np np,
           (mat1I (get_comp_0 c) (get_comp_1 c) np,
             (mat_plusI (get_comp_plus c),
               (mat_multI (get_comp_0 c) (get_comp_plus c) (get_comp_times c)
                  np,
                 (id, (mat_geI (get_comp_ge c),
                        (mat_geI (get_comp_gt c),
                          (False,
                            (concata
                               [showa np, ['x'], showa np,
                                 [' ', 'm', 'a', 't', 'r', 'i', 'c', 'e', 's',
                                   ' ', 'o', 'v', 'e', 'r', ' '],
                                 get_comp_description c],
                              mat_arc_posI (get_comp_apos c)))))))))));

get_comp_mono ::
  forall a.
    (a -> Bool,
      (a, (a, (a -> a -> a,
                (a -> a -> a,
                  (a -> a,
                    (a -> a -> Bool,
                      (a -> a -> Bool, (Bool, ([Char], a -> Bool))))))))))
      -> Bool;
get_comp_mono (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) =
  mo;

get_comp_max0 ::
  forall a.
    (a -> Bool,
      (a, (a, (a -> a -> a,
                (a -> a -> a,
                  (a -> a,
                    (a -> a -> Bool,
                      (a -> a -> Bool, (Bool, ([Char], a -> Bool))))))))))
      -> a -> a;
get_comp_max0 (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) =
  mz;

vec_max0I :: forall a. (a -> a) -> [a] -> [a];
vec_max0I = mapa;

mat_max0I :: forall a. (a -> a) -> [[a]] -> [[a]];
mat_max0I mz = mapa (vec_max0I mz);

mat_components ::
  forall a.
    Nat ->
      Nat ->
        (a -> Bool,
          (a, (a, (a -> a -> a,
                    (a -> a -> a,
                      (a -> a,
                        (a -> a -> Bool,
                          (a -> a -> Bool, (Bool, ([Char], a -> Bool))))))))))
          -> ([[a]] -> Bool,
               ([[a]],
                 ([[a]],
                   ([[a]] -> [[a]] -> [[a]],
                     ([[a]] -> [[a]] -> [[a]],
                       ([[a]] -> [[a]],
                         ([[a]] -> [[a]] -> Bool,
                           ([[a]] -> [[a]] -> Bool,
                             (Bool, ([Char], [[a]] -> Bool))))))))));
mat_components n sd c =
  let {
    sdp = Suc (minus_nat sd one_nat);
    np = maxa n sdp;
  } in (wf_mat np np,
         (mat0I (get_comp_0 c) np np,
           (mat1I (get_comp_0 c) (get_comp_1 c) np,
             (mat_plusI (get_comp_plus c),
               (mat_multI (get_comp_0 c) (get_comp_plus c) (get_comp_times c)
                  np,
                 (mat_max0I (get_comp_max0 c),
                   (mat_geI (get_comp_ge c),
                     (mat_gtI (get_comp_ge c) (get_comp_gt c) sdp,
                       (get_comp_mono c,
                         (concata
                            [showa np, ['x'], showa np,
                              [' ', 'm', 'a', 't', 'r', 'i', 'c', 'e', 's', ' ',
                                'w', 'i', 't', 'h', ' ', 's', 't', 'r', 'i',
                                'c', 't', ' ', 'd', 'i', 'm', 'e', 'n', 's',
                                'i', 'o', 'n', ' '],
                              showa sdp, [' ', 'o', 'v', 'e', 'r', ' '],
                              get_comp_description c],
                           (\ _ -> True)))))))))));

xml2vec_coeff ::
  forall a. (Xml -> Eithera [Char] a) -> Xml -> Eithera [Char] [a];
xml2vec_coeff xml2coeff =
  xmlManyElements ['v', 'e', 'c', 't', 'o', 'r']
    (xml1element ['c', 'o', 'e', 'f', 'f', 'i', 'c', 'i', 'e', 'n', 't']
      xml2coeff id)
    id;

xml2mat_coeff ::
  forall a. a -> (Xml -> Eithera [Char] a) -> Xml -> Eithera [Char] [[a]];
xml2mat_coeff ze xml2coeff =
  xmlOptions
    [(['m', 'a', 't', 'r', 'i', 'x'],
       xmlManyElements ['m', 'a', 't', 'r', 'i', 'x'] (xml2vec_coeff xml2coeff)
         id),
      (['v', 'e', 'c', 't', 'o', 'r'],
        xmlChange (xml2vec_coeff xml2coeff)
          (\ v ->
            v : replicatea (minus_nat (size_list v) one_nat)
                  (replicatea (size_list v) ze)))];

pos_arctic_rat :: Arctic_rat -> Bool;
pos_arctic_rat MinInfty_rat = False;
pos_arctic_rat (Num_arc_rat n) = less_eq_rat (Fract 0 1) n;

arctic_rat_name :: [Char];
arctic_rat_name =
  ['a', 'r', 'c', 't', 'i', 'c', ' ', 'r', 'a', 't', 'i', 'o', 'n', 'a', 'l',
    's'];

gt_arctic_rat :: Rat -> Arctic_rat -> Arctic_rat -> Bool;
gt_arctic_rat delta uu MinInfty_rat = True;
gt_arctic_rat delta MinInfty_rat (Num_arc_rat uv) = False;
gt_arctic_rat delta (Num_arc_rat x) (Num_arc_rat y) = rat_gt delta x y;

ge_arctic_rat :: Arctic_rat -> Arctic_rat -> Bool;
ge_arctic_rat x MinInfty_rat = True;
ge_arctic_rat MinInfty_rat (Num_arc_rat uu) = False;
ge_arctic_rat (Num_arc_rat x) (Num_arc_rat y) = less_eq_rat y x;

times_arctic_rat :: Arctic_rat -> Arctic_rat -> Arctic_rat;
times_arctic_rat MinInfty_rat y = MinInfty_rat;
times_arctic_rat (Num_arc_rat v) MinInfty_rat = MinInfty_rat;
times_arctic_rat (Num_arc_rat x) (Num_arc_rat y) = Num_arc_rat (plus_rat x y);

plus_arctic_rat :: Arctic_rat -> Arctic_rat -> Arctic_rat;
plus_arctic_rat MinInfty_rat y = y;
plus_arctic_rat (Num_arc_rat v) MinInfty_rat = Num_arc_rat v;
plus_arctic_rat (Num_arc_rat x) (Num_arc_rat y) = Num_arc_rat (maxa x y);

one_arctic_rat :: Arctic_rat;
one_arctic_rat = Num_arc_rat (Fract 0 1);

zero_arctic_rat :: Arctic_rat;
zero_arctic_rat = MinInfty_rat;

arctic_rat_components ::
  Rat ->
    (Arctic_rat -> Bool,
      (Arctic_rat,
        (Arctic_rat,
          (Arctic_rat -> Arctic_rat -> Arctic_rat,
            (Arctic_rat -> Arctic_rat -> Arctic_rat,
              (Arctic_rat -> Arctic_rat,
                (Arctic_rat -> Arctic_rat -> Bool,
                  (Arctic_rat -> Arctic_rat -> Bool,
                    (Bool, ([Char], Arctic_rat -> Bool))))))))));
arctic_rat_components delta =
  ((\ _ -> True),
    (zero_arctic_rat,
      (one_arctic_rat,
        (plus_arctic_rat,
          (times_arctic_rat,
            (id, (ge_arctic_rat,
                   (gt_arctic_rat delta,
                     (False, (arctic_rat_name, pos_arctic_rat))))))))));

eq_arctic_rat :: Arctic_rat -> Arctic_rat -> Bool;
eq_arctic_rat MinInfty_rat MinInfty_rat = True;
eq_arctic_rat (Num_arc_rat rat) (Num_arc_rat rat') = eq_rat rat rat';
eq_arctic_rat MinInfty_rat (Num_arc_rat rat') = False;
eq_arctic_rat (Num_arc_rat rat') MinInfty_rat = False;

instance Eq Arctic_rat where {
  a == b = eq_arctic_rat a b;
};

xml2arctic_rat_coeff :: Xml -> Eithera [Char] Arctic_rat;
xml2arctic_rat_coeff =
  xmlOptions
    [(['i', 'n', 't', 'e', 'g', 'e', 'r'], xmlChange xml2rat Num_arc_rat),
      (['r', 'a', 't', 'i', 'o', 'n', 'a', 'l'], xmlChange xml2rat Num_arc_rat),
      (['m', 'i', 'n', 'u', 's', 'I', 'n', 'f', 'i', 'n', 'i', 't', 'y'],
        xml0elements
          ['m', 'i', 'n', 'u', 's', 'I', 'n', 'f', 'i', 'n', 'i', 't', 'y']
          MinInfty_rat)];

pos_arctic :: Arctic -> Bool;
pos_arctic MinInfty = False;
pos_arctic (Num_arc n) = 0 <= n;

arctic_name :: [Char];
arctic_name =
  ['a', 'r', 'c', 't', 'i', 'c', ' ', 's', 'e', 'm', 'i', 'r', 'i', 'n', 'g'];

gt_arctic :: Arctic -> Arctic -> Bool;
gt_arctic uu MinInfty = True;
gt_arctic MinInfty (Num_arc uv) = False;
gt_arctic (Num_arc x) (Num_arc y) = y < x;

ge_arctic :: Arctic -> Arctic -> Bool;
ge_arctic x MinInfty = True;
ge_arctic MinInfty (Num_arc uu) = False;
ge_arctic (Num_arc x) (Num_arc y) = y <= x;

times_arctic :: Arctic -> Arctic -> Arctic;
times_arctic MinInfty y = MinInfty;
times_arctic (Num_arc v) MinInfty = MinInfty;
times_arctic (Num_arc x) (Num_arc y) = Num_arc (x + y);

plus_arctic :: Arctic -> Arctic -> Arctic;
plus_arctic MinInfty y = y;
plus_arctic (Num_arc v) MinInfty = Num_arc v;
plus_arctic (Num_arc x) (Num_arc y) = Num_arc (maxa x y);

one_arctic :: Arctic;
one_arctic = Num_arc 0;

zero_arctic :: Arctic;
zero_arctic = MinInfty;

arctic_components ::
  (Arctic -> Bool,
    (Arctic,
      (Arctic,
        (Arctic -> Arctic -> Arctic,
          (Arctic -> Arctic -> Arctic,
            (Arctic -> Arctic,
              (Arctic -> Arctic -> Bool,
                (Arctic -> Arctic -> Bool,
                  (Bool, ([Char], Arctic -> Bool))))))))));
arctic_components =
  ((\ _ -> True),
    (zero_arctic,
      (one_arctic,
        (plus_arctic,
          (times_arctic,
            (id, (ge_arctic,
                   (gt_arctic, (False, (arctic_name, pos_arctic))))))))));

eq_arctic :: Arctic -> Arctic -> Bool;
eq_arctic MinInfty MinInfty = True;
eq_arctic (Num_arc int) (Num_arc int') = int == int';
eq_arctic MinInfty (Num_arc int') = False;
eq_arctic (Num_arc int') MinInfty = False;

instance Eq Arctic where {
  a == b = eq_arctic a b;
};

xml2arctic_coeff :: Xml -> Eithera [Char] Arctic;
xml2arctic_coeff =
  xmlOptions
    [(['i', 'n', 't', 'e', 'g', 'e', 'r'],
       xmlChange (xml2int ['i', 'n', 't', 'e', 'g', 'e', 'r']) Num_arc),
      (['m', 'i', 'n', 'u', 's', 'I', 'n', 'f', 'i', 'n', 'i', 't', 'y'],
        xml0elements
          ['m', 'i', 'n', 'u', 's', 'I', 'n', 'f', 'i', 'n', 'i', 't', 'y']
          MinInfty)];

int_name :: [Char];
int_name =
  ['n', 'a', 't', 'u', 'r', 'a', 'l', 's', ' ', 'w', 'i', 't', 'h', ' ', 'n',
    'e', 'g', 'a', 't', 'i', 'v', 'e', ' ', 'c', 'o', 'n', 's', 't', 'a', 'n',
    't', 's'];

ge_int :: Integer -> Integer -> Bool;
ge_int x y = y <= x;

max0_int :: Integer -> Integer;
max0_int x = maxa 0 x;

int_components ::
  (Integer -> Bool,
    (Integer,
      (Integer,
        (Integer -> Integer -> Integer,
          (Integer -> Integer -> Integer,
            (Integer -> Integer,
              (Integer -> Integer -> Bool,
                (Integer -> Integer -> Bool,
                  (Bool, ([Char], Integer -> Bool))))))))));
int_components =
  ((\ _ -> True),
    (0, (1, ((\ a b -> a + b),
              ((\ a b -> a * b),
                (max0_int,
                  (ge_int,
                    ((\ x y -> y < x),
                      (True, (int_name, (\ _ -> True)))))))))));

poly_to_lpoly ::
  forall a.
    (Eq a) => a -> a -> (a -> a -> a) ->
                          (a -> a -> a) -> Poly a -> Eithera [Char] (a, [a]);
poly_to_lpoly uu uv uw ux (PNum i) = returna (i, []);
poly_to_lpoly ze on pl ti (PVar x) =
  returna (ze, append (replicatea x ze) [on]);
poly_to_lpoly ze on pl ti (PSum [p]) = poly_to_lpoly ze on pl ti p;
poly_to_lpoly ze on pl ti (PSum (p : q : ps)) =
  bind (poly_to_lpoly ze on pl ti p)
    (\ (cp, ncp) ->
      bind (poly_to_lpoly ze on pl ti (PSum (q : ps)))
        (\ (cq, ncq) -> returna (pl cp cq, sum_lpoly pl ncp ncq)));
poly_to_lpoly ze on pl ti (PMult [p]) = poly_to_lpoly ze on pl ti p;
poly_to_lpoly ze on pl ti (PMult (p : q : ps)) =
  bind (poly_to_lpoly ze on pl ti p)
    (\ (cp, ncp) ->
      bind (poly_to_lpoly ze on pl ti (PMult (q : ps)))
        (\ (cq, ncq) ->
          (if foralla (\ a -> ze == a) ncp
            then returna (ti cp cq, mapa (ti cp) ncq)
            else (if foralla (\ a -> ze == a) ncq
                   then returna (ti cp cq, mapa (\ x -> ti x cq) ncp)
                   else errora
                          ['n', 'o', 'n', '-', 'l', 'i', 'n', 'e', 'a', 'r',
                            ' ', 'p', 'o', 'l', 'y', 'n', 'o', 'm', 'i', 'a',
                            'l', 's', ' ', 'n', 'o', 't', ' ', 's', 'u', 'p',
                            'p', 'o', 'r', 't', 'e', 'd']))));

xml2int_coeff :: Xml -> Eithera [Char] Integer;
xml2int_coeff = xml2int ['i', 'n', 't', 'e', 'g', 'e', 'r'];

divide_rat :: Rat -> Rat -> Rat;
divide_rat (Fract a b) (Fract c d) = fract_norm (a * d) (b * c);

default_delta :: Rat;
default_delta =
  divide_rat (Fract 1 1) (Fract 1000000000000000000000000000000000000 1);

xml2interpretation_type :: Xml -> Eithera [Char] Domain;
xml2interpretation_type =
  xml1element ['t', 'y', 'p', 'e']
    (xmlOptions
      [(['p', 'o', 'l', 'y', 'n', 'o', 'm', 'i', 'a', 'l'],
         xml2elements ['p', 'o', 'l', 'y', 'n', 'o', 'm', 'i', 'a', 'l']
           (xml1element ['d', 'o', 'm', 'a', 'i', 'n']
             (xmlOptions
               [(['n', 'a', 't', 'u', 'r', 'a', 'l', 's'],
                  xml0elements ['n', 'a', 't', 'u', 'r', 'a', 'l', 's']
                    Natural),
                 (['a', 'r', 'c', 't', 'i', 'c'],
                   xml0elements ['a', 'r', 'c', 't', 'i', 'c'] Arctic),
                 (['a', 'r', 'c', 't', 'i', 'c', 'R', 'a', 't', 'i', 'o', 'n',
                    'a', 'l', 's'],
                   xml0elements
                     ['a', 'r', 'c', 't', 'i', 'c', 'R', 'a', 't', 'i', 'o',
                       'n', 'a', 'l', 's']
                     (Arctic_rat default_delta)),
                 (['a', 'r', 'c', 't', 'i', 'c', 'B', 'e', 'l', 'o', 'w', 'Z',
                    'e', 'r', 'o'],
                   xml0elements
                     ['a', 'r', 'c', 't', 'i', 'c', 'B', 'e', 'l', 'o', 'w',
                       'Z', 'e', 'r', 'o']
                     Arctic),
                 (['r', 'a', 't', 'i', 'o', 'n', 'a', 'l', 's'],
                   xml0elements ['r', 'a', 't', 'i', 'o', 'n', 'a', 'l', 's']
                     (Rationala default_delta)),
                 (['m', 'a', 't', 'r', 'i', 'c', 'e', 's'],
                   xml3elements ['m', 'a', 't', 'r', 'i', 'c', 'e', 's']
                     (xml2nat ['d', 'i', 'm', 'e', 'n', 's', 'i', 'o', 'n'])
                     (xml2nat
                       ['s', 't', 'r', 'i', 'c', 't', 'D', 'i', 'm', 'e', 'n',
                         's', 'i', 'o', 'n'])
                     (xml1element ['d', 'o', 'm', 'a', 'i', 'n']
                       (xmlOptions
                         [(['r', 'a', 't', 'i', 'o', 'n', 'a', 'l', 's'],
                            xml0elements
                              ['r', 'a', 't', 'i', 'o', 'n', 'a', 'l', 's']
                              Zero_nat),
                           (['n', 'a', 't', 'u', 'r', 'a', 'l', 's'],
                             xml0elements
                               ['n', 'a', 't', 'u', 'r', 'a', 'l', 's']
                               one_nat),
                           (['a', 'r', 'c', 't', 'i', 'c'],
                             xml0elements ['a', 'r', 'c', 't', 'i', 'c']
                               (nat 2)),
                           (['a', 'r', 'c', 't', 'i', 'c', 'R', 'a', 't', 'i',
                              'o', 'n', 'a', 'l', 's'],
                             xml0elements
                               ['a', 'r', 'c', 't', 'i', 'c', 'R', 'a', 't',
                                 'i', 'o', 'n', 'a', 'l', 's']
                               (nat 3)),
                           (['a', 'r', 'c', 't', 'i', 'c', 'B', 'e', 'l', 'o',
                              'w', 'Z', 'e', 'r', 'o'],
                             xml0elements
                               ['a', 'r', 'c', 't', 'i', 'c', 'B', 'e', 'l',
                                 'o', 'w', 'Z', 'e', 'r', 'o']
                               (nat 2))])
                       id)
                     (\ di sd doa ->
                       (if eq_nat doa one_nat then Int_mat di sd
                         else (if eq_nat doa Zero_nat
                                then Rat_mat default_delta di sd
                                else (if eq_nat doa (nat 2) then Arc_mat di
                                       else Arc_rat_mat default_delta di)))))])
             id)
           (xml2nat ['d', 'e', 'g', 'r', 'e', 'e']) (\ typea _ -> typea)),
        (['m', 'a', 't', 'r', 'i', 'x', 'I', 'n', 't', 'e', 'r', 'p', 'r', 'e',
           't', 'a', 't', 'i', 'o', 'n'],
          xml3elements
            ['m', 'a', 't', 'r', 'i', 'x', 'I', 'n', 't', 'e', 'r', 'p', 'r',
              'e', 't', 'a', 't', 'i', 'o', 'n']
            (xml1element ['d', 'o', 'm', 'a', 'i', 'n']
              (xmlOptions
                [(['r', 'a', 't', 'i', 'o', 'n', 'a', 'l', 's'],
                   xml0elements ['r', 'a', 't', 'i', 'o', 'n', 'a', 'l', 's']
                     Zero_nat),
                  (['n', 'a', 't', 'u', 'r', 'a', 'l', 's'],
                    xml0elements ['n', 'a', 't', 'u', 'r', 'a', 'l', 's']
                      one_nat),
                  (['a', 'r', 'c', 't', 'i', 'c'],
                    xml0elements ['a', 'r', 'c', 't', 'i', 'c'] (nat 2)),
                  (['a', 'r', 'c', 't', 'i', 'c', 'R', 'a', 't', 'i', 'o', 'n',
                     'a', 'l', 's'],
                    xml0elements
                      ['a', 'r', 'c', 't', 'i', 'c', 'R', 'a', 't', 'i', 'o',
                        'n', 'a', 'l', 's']
                      (nat 3)),
                  (['a', 'r', 'c', 't', 'i', 'c', 'B', 'e', 'l', 'o', 'w', 'Z',
                     'e', 'r', 'o'],
                    xml0elements
                      ['a', 'r', 'c', 't', 'i', 'c', 'B', 'e', 'l', 'o', 'w',
                        'Z', 'e', 'r', 'o']
                      (nat 2))])
              id)
            (xml2nat ['d', 'i', 'm', 'e', 'n', 's', 'i', 'o', 'n'])
            (xml2nat
              ['s', 't', 'r', 'i', 'c', 't', 'D', 'i', 'm', 'e', 'n', 's', 'i',
                'o', 'n'])
            (\ doa di sd ->
              (if eq_nat doa one_nat then Int_mat di sd
                else (if eq_nat doa Zero_nat then Rat_mat default_delta di sd
                       else (if eq_nat doa (nat 2) then Arc_mat di
                              else Arc_rat_mat default_delta di)))))])
    id;

xml2interpretation ::
  forall a. (Xml -> Eithera [Char] a) -> Xml -> Eithera [Char] (Redpair a);
xml2interpretation xml2name =
  xml1CmanyElements
    ['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't', 'a', 't', 'i', 'o', 'n']
    xml2interpretation_type
    (\ a ->
      case a of {
        Natural ->
          xml3elements ['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't'] xml2name
            (xml2nat ['a', 'r', 'i', 't', 'y'])
            (\ x ->
              bind (xml2poly xml2int_coeff x)
                (poly_to_lpoly (get_comp_0 int_components)
                  (get_comp_1 int_components) (get_comp_plus int_components)
                  (get_comp_times int_components)))
            (\ f aa poly ->
              Alter1
                ((f, aa),
                  (fst poly,
                    fitLength (get_comp_0 int_components) aa (snd poly))));
        Arctic ->
          xml3elements ['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't'] xml2name
            (xml2nat ['a', 'r', 'i', 't', 'y'])
            (\ x ->
              bind (xml2poly xml2arctic_coeff x)
                (poly_to_lpoly (get_comp_0 arctic_components)
                  (get_comp_1 arctic_components)
                  (get_comp_plus arctic_components)
                  (get_comp_times arctic_components)))
            (\ f aa poly ->
              Alter3
                ((f, aa),
                  (fst poly,
                    fitLength (get_comp_0 arctic_components) aa (snd poly))));
        Arctic_rat d ->
          xml3elements ['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't'] xml2name
            (xml2nat ['a', 'r', 'i', 't', 'y'])
            (\ x ->
              bind (xml2poly xml2arctic_rat_coeff x)
                (poly_to_lpoly (get_comp_0 (arctic_rat_components d))
                  (get_comp_1 (arctic_rat_components d))
                  (get_comp_plus (arctic_rat_components d))
                  (get_comp_times (arctic_rat_components d))))
            (\ f aa poly ->
              Alter7
                ((f, aa),
                  (fst poly,
                    fitLength (get_comp_0 (arctic_rat_components d)) aa
                      (snd poly))));
        Int_mat n sd ->
          xml3elements ['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't'] xml2name
            (xml2nat ['a', 'r', 'i', 't', 'y'])
            (\ x ->
              bind (xml2poly (xml2mat_coeff 0 xml2int_coeff) x)
                (poly_to_lpoly (get_comp_0 (mat_components n sd int_components))
                  (get_comp_1 (mat_components n sd int_components))
                  (get_comp_plus (mat_components n sd int_components))
                  (get_comp_times (mat_components n sd int_components))))
            (\ f aa poly ->
              Alter4
                ((f, aa),
                  (fst poly,
                    fitLength (get_comp_0 (mat_components n sd int_components))
                      aa (snd poly))));
        Arc_mat n ->
          xml3elements ['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't'] xml2name
            (xml2nat ['a', 'r', 'i', 't', 'y'])
            (\ x ->
              bind (xml2poly (xml2mat_coeff MinInfty xml2arctic_coeff) x)
                (poly_to_lpoly
                  (get_comp_0 (mat_arctic_components n arctic_components))
                  (get_comp_1 (mat_arctic_components n arctic_components))
                  (get_comp_plus (mat_arctic_components n arctic_components))
                  (get_comp_times (mat_arctic_components n arctic_components))))
            (\ f aa poly ->
              Alter6
                ((f, aa),
                  (fst poly,
                    fitLength
                      (get_comp_0 (mat_arctic_components n arctic_components))
                      aa (snd poly))));
        Arc_rat_mat d n ->
          xml3elements ['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't'] xml2name
            (xml2nat ['a', 'r', 'i', 't', 'y'])
            (\ x ->
              bind (xml2poly (xml2mat_coeff MinInfty_rat xml2arctic_rat_coeff)
                     x)
                (poly_to_lpoly
                  (get_comp_0
                    (mat_arctic_components n (arctic_rat_components d)))
                  (get_comp_1
                    (mat_arctic_components n (arctic_rat_components d)))
                  (get_comp_plus
                    (mat_arctic_components n (arctic_rat_components d)))
                  (get_comp_times
                    (mat_arctic_components n (arctic_rat_components d)))))
            (\ f aa poly ->
              Alter8
                ((f, aa),
                  (fst poly,
                    fitLength
                      (get_comp_0
                        (mat_arctic_components n (arctic_rat_components d)))
                      aa (snd poly))));
        Rationala d ->
          xml3elements ['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't'] xml2name
            (xml2nat ['a', 'r', 'i', 't', 'y'])
            (\ x ->
              bind (xml2poly xml2rat x)
                (poly_to_lpoly (get_comp_0 (rat_components d))
                  (get_comp_1 (rat_components d))
                  (get_comp_plus (rat_components d))
                  (get_comp_times (rat_components d))))
            (\ f aa poly ->
              Alter2
                ((f, aa),
                  (fst poly,
                    fitLength (get_comp_0 (rat_components d)) aa (snd poly))));
        Rat_mat d n sd ->
          xml3elements ['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't'] xml2name
            (xml2nat ['a', 'r', 'i', 't', 'y'])
            (\ x ->
              bind (xml2poly (xml2mat_coeff (Fract 0 1) xml2rat) x)
                (poly_to_lpoly
                  (get_comp_0 (mat_components n sd (rat_components d)))
                  (get_comp_1 (mat_components n sd (rat_components d)))
                  (get_comp_plus (mat_components n sd (rat_components d)))
                  (get_comp_times (mat_components n sd (rat_components d)))))
            (\ f aa poly ->
              Alter5
                ((f, aa),
                  (fst poly,
                    fitLength
                      (get_comp_0 (mat_components n sd (rat_components d))) aa
                      (snd poly))));
      })
    (\ typea pia ->
      case typea of {
        Natural -> Int_carrier (mapa alter1a pia);
        Arctic -> Arctic_carrier (mapa alter3a pia);
        Arctic_rat d -> Arctic_rat_carrier d (mapa alter7a pia);
        Int_mat n sd -> Int_mat_carrier n sd (mapa alter4a pia);
        Arc_mat n -> Arc_mat_carrier n (mapa alter6a pia);
        Arc_rat_mat d n -> Arc_rat_mat_carrier d n (mapa alter8a pia);
        Rationala d -> Rat_carrier d (mapa alter2a pia);
        Rat_mat d n sd -> Rat_mat_carrier d n sd (mapa alter5a pia);
      });

xml2redpair ::
  forall a. (Xml -> Eithera [Char] a) -> Xml -> Eithera [Char] (Redpair a);
xml2redpair xml2name =
  xml1element ['r', 'e', 'd', 'P', 'a', 'i', 'r']
    (xmlOptions
      [(['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't', 'a', 't', 'i', 'o', 'n'],
         xml2interpretation xml2name),
        (['p', 'a', 't', 'h', 'O', 'r', 'd', 'e', 'r'],
          xml2path_order xml2name)])
    id;

xml2ndChoice ::
  forall a b c.
    [Char] ->
      (Xml -> Eithera [Char] a) ->
        [Char] ->
          (Xml -> Eithera [Char] b) ->
            (a -> Maybe b -> c) -> Xml -> Eithera [Char] c;
xml2ndChoice reqName p1 choiceName p2 combi (Xml name atts cs text) =
  (if name == reqName &&
        (nulla atts && (less_eq_nat (nat 2) (size_list cs) && is_none text))
    then bind (p1 (nth cs Zero_nat))
           (\ a ->
             bind (xml2bool choiceName (nth cs one_nat))
               (\ choice ->
                 (if choice
                   then bind (p2 (nth cs (minus_nat (size_list cs) one_nat)))
                          (\ b -> returna (combi a (Just b)))
                   else returna (combi a Nothing))))
    else errora
           (concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], [newline], showa (Xml name atts cs text)]));

xml2dpProof ::
  forall a.
    Nat ->
      (Nat -> Xml -> Eithera [Char] (Lab a [Nat])) ->
        Xml -> Eithera [Char] (DPProof a [Nat] [Char]);
xml2dpProof ld xml2name x =
  xmlOptions
    [(['p', 'I', 's', 'E', 'm', 'p', 't', 'y'],
       xml0elements ['p', 'I', 's', 'E', 'm', 'p', 't', 'y'] PisEmpty),
      (['d', 'e', 'p', 'G', 'r', 'a', 'p', 'h', 'P', 'r', 'o', 'c'],
        xmlManyElements
          ['d', 'e', 'p', 'G', 'r', 'a', 'p', 'h', 'P', 'r', 'o', 'c']
          (xml2ndChoice ['c', 'o', 'm', 'p', 'o', 'n', 'e', 'n', 't']
            (xml1element ['d', 'p', 's'] (xml2rules (xml2name ld)) id)
            ['r', 'e', 'a', 'l', 'S', 'c', 'c'] (xml2dpProof ld xml2name)
            (\ dps prfOpt -> (prfOpt, dps)))
          DepGraphProc),
      (['r', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o', 'c'],
        xml3elements ['r', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o', 'c']
          (xml2redpair (xml2name ld))
          (xml1element ['d', 'p', 's'] (xml2rules (xml2name ld)) id)
          (xml2dpProof ld xml2name) RedPairProc),
      (['r', 'e', 'd', 'P', 'a', 'i', 'r', 'U', 'r', 'P', 'r', 'o', 'c'],
        xml4elements
          ['r', 'e', 'd', 'P', 'a', 'i', 'r', 'U', 'r', 'P', 'r', 'o', 'c']
          (xml2redpair (xml2name ld))
          (xml1element ['d', 'p', 's'] (xml2rules (xml2name ld)) id)
          (xml1element ['u', 's', 'a', 'b', 'l', 'e', 'R', 'u', 'l', 'e', 's']
            (xml2rules (xml2name ld)) id)
          (xml2dpProof ld xml2name) RedPairUrProc),
      (['m', 'o', 'n', 'o', 'R', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o',
         'c'],
        xml4elements
          ['m', 'o', 'n', 'o', 'R', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o',
            'c']
          (xml2redpair (xml2name ld))
          (xml1element ['d', 'p', 's'] (xml2rules (xml2name ld)) id)
          (xml1element ['t', 'r', 's'] (xml2rules (xml2name ld)) id)
          (xml2dpProof ld xml2name) MonoRedPairProc),
      (['m', 'o', 'n', 'o', 'R', 'e', 'd', 'P', 'a', 'i', 'r', 'U', 'r', 'P',
         'r', 'o', 'c'],
        xml5elements
          ['m', 'o', 'n', 'o', 'R', 'e', 'd', 'P', 'a', 'i', 'r', 'U', 'r', 'P',
            'r', 'o', 'c']
          (xml2redpair (xml2name ld))
          (xml1element ['d', 'p', 's'] (xml2rules (xml2name ld)) id)
          (xml1element ['t', 'r', 's'] (xml2rules (xml2name ld)) id)
          (xml1element ['u', 's', 'a', 'b', 'l', 'e', 'R', 'u', 'l', 'e', 's']
            (xml2rules (xml2name ld)) id)
          (xml2dpProof ld xml2name) MonoRedPairUrProc),
      (['s', 'e', 'm', 'l', 'a', 'b', 'P', 'r', 'o', 'c'],
        xml4elements ['s', 'e', 'm', 'l', 'a', 'b', 'P', 'r', 'o', 'c']
          (xml2sl_inter (xml2name ld))
          (xml1element ['d', 'p', 's'] (xml2rules (xml2name (Suc ld))) id)
          (xml1element ['t', 'r', 's'] (xml2rules (xml2name (Suc ld))) id)
          (xml2dpProof (Suc ld) xml2name)
          (\ a ->
            case a of {
              Nothing -> RootlabProc;
              Just i -> SemlabProc i id_labeling;
            })),
      (['u', 'n', 'l', 'a', 'b', 'P', 'r', 'o', 'c'],
        xml3elements ['u', 'n', 'l', 'a', 'b', 'P', 'r', 'o', 'c']
          (xml1element ['d', 'p', 's']
            (xml2rules (xml2name (minus_nat ld one_nat))) id)
          (xml1element ['t', 'r', 's']
            (xml2rules (xml2name (minus_nat ld one_nat))) id)
          (xml2dpProof (minus_nat ld one_nat) xml2name) UnlabProc),
      (['s', 'u', 'b', 't', 'e', 'r', 'm', 'P', 'r', 'o', 'c'],
        xml3elements ['s', 'u', 'b', 't', 'e', 'r', 'm', 'P', 'r', 'o', 'c']
          (xml2sp (xml2name ld))
          (xml1element ['d', 'p', 's'] (xml2rules (xml2name ld)) id)
          (xml2dpProof ld xml2name) SpscProc),
      (['f', 'l', 'a', 't', 'C', 'o', 'n', 't', 'e', 'x', 't', 'C', 'l', 'o',
         's', 'u', 'r', 'e', 'P', 'r', 'o', 'c'],
        xml5elements
          ['f', 'l', 'a', 't', 'C', 'o', 'n', 't', 'e', 'x', 't', 'C', 'l', 'o',
            's', 'u', 'r', 'e', 'P', 'r', 'o', 'c']
          (xml1element ['f', 'r', 'e', 's', 'h', 'S', 'y', 'm', 'b', 'o', 'l']
            (xml2name ld) id)
          (xml2flat_contexts (xml2name ld))
          (xml1element ['d', 'p', 's'] (xml2rules (xml2name ld)) id)
          (xml1element ['t', 'r', 's'] (xml2rules (xml2name ld)) id)
          (xml2dpProof ld xml2name) FccProc),
      (['s', 'i', 'z', 'e', 'C', 'h', 'a', 'n', 'g', 'e', 'C', 'r', 'i', 't',
         'e', 'r', 'i', 'o', 'n'],
        xml1manyElements
          ['s', 'i', 'z', 'e', 'C', 'h', 'a', 'n', 'g', 'e', 'C', 'r', 'i', 't',
            'e', 'r', 'i', 'o', 'n']
          (xmlOptions
            [(['s', 'u', 'b', 't', 'e', 'r', 'm', 'C', 'r', 'i', 't', 'e', 'r',
                'i', 'o', 'n'],
               xml0elements
                 ['s', 'u', 'b', 't', 'e', 'r', 'm', 'C', 'r', 'i', 't', 'e',
                   'r', 'i', 'o', 'n']
                 Nothing),
              (['r', 'e', 'd', 'u', 'c', 't', 'i', 'o', 'n', 'P', 'a', 'i',
                 'r'],
                xml2elements
                  ['r', 'e', 'd', 'u', 'c', 't', 'i', 'o', 'n', 'P', 'a', 'i',
                    'r']
                  (xml2redpair (xml2name ld))
                  (xml1element
                    ['u', 's', 'a', 'b', 'l', 'e', 'R', 'u', 'l', 'e', 's']
                    (xml2rules (xml2name ld)) id)
                  (\ redp ur -> Just (redp, ur)))])
          (xml2scg (xml2name ld))
          (\ a ->
            case a of {
              Nothing -> SizeChangeSubTerm;
              Just redp_ur -> SizeChangeRedPair (fst redp_ur) (snd redp_ur);
            }))]
    x;

foldR :: forall a b. (a -> [a] -> b -> b) -> [a] -> b -> b;
foldR f [] b = b;
foldR f (x : xs) b = f x xs (foldR f xs b);

mat_monoI :: forall a. (a -> Bool) -> Nat -> [[a]] -> Bool;
mat_monoI geq1 sd m = foralla (anya geq1) (sub_mat sd sd m);

mul_pvarsI ::
  forall a b. (Eq a) => a -> (a -> a -> a) -> a -> [(b, a)] -> [(b, a)];
mul_pvarsI uu uv a [] = [];
mul_pvarsI ze ti a ((x, b) : vas) =
  let {
    p = ti a b;
    res = mul_pvarsI ze ti a vas;
  } in (if p == ze then res else (x, p) : res);

data L_poly a b = LPoly b [(a, b)];

mul_lpolyI ::
  forall a b. (Eq a) => a -> (a -> a -> a) -> a -> L_poly b a -> L_poly b a;
mul_lpolyI ze ti a (LPoly b vas) = LPoly (ti a b) (mul_pvarsI ze ti a vas);

list_sumI :: forall a. a -> (a -> a -> a) -> [a] -> a;
list_sumI ze pl [] = ze;
list_sumI ze pl (x : xs) = pl x (list_sumI ze pl xs);

add_varI ::
  forall a b.
    (Eq a, Eq b) => a -> (a -> a -> a) -> b -> a -> [(b, a)] -> [(b, a)];
add_varI uu uv x a [] = [(x, a)];
add_varI ze pl x a ((y, b) : vas) =
  (if x == y
    then let {
           s = pl a b;
         } in (if s == ze then vas else (x, s) : vas)
    else (y, b) : add_varI ze pl x a vas);

sum_pvarsI ::
  forall a b.
    (Eq a, Eq b) => a -> (a -> a -> a) -> [(b, a)] -> [(b, a)] -> [(b, a)];
sum_pvarsI uu uv [] vbs = vbs;
sum_pvarsI ze pl ((x, a) : vas) vbs =
  (if a == ze then sum_pvarsI ze pl vas vbs
    else sum_pvarsI ze pl vas (add_varI ze pl x a vbs));

sum_lpolyI ::
  forall a b.
    (Eq a,
      Eq b) => a -> (a -> a -> a) -> L_poly b a -> L_poly b a -> L_poly b a;
sum_lpolyI ze pl (LPoly a vas) (LPoly b vbs) =
  LPoly (pl a b) (sum_pvarsI ze pl vas vbs);

l_poly_case :: forall a b c. (a -> [(b, a)] -> c) -> L_poly b a -> c;
l_poly_case f1 (LPoly a list) = f1 a list;

pleftI ::
  forall a b c.
    (Eq a,
      Eq c) => a -> a -> (a -> a -> a) ->
                           (a -> a -> a) ->
                             (a -> a) ->
                               (b -> Nat -> (a, [a])) -> Term b c -> L_poly c a;
pleftI ze on pl ti mz pia (Var x) = LPoly ze [(x, on)];
pleftI ze on pl ti mz pia (Fun f ts) =
  let {
    (c, asa) = pia f (size_list ts);
  } in case sum_lpolyI ze pl (LPoly c [])
              (list_sumI (LPoly ze []) (sum_lpolyI ze pl)
                (mapa (\ at ->
                        mul_lpolyI ze ti (fst at)
                          (pleftI ze on pl ti mz pia (snd at)))
                  (zipa asa ts)))
         of {
         LPoly d [] -> LPoly (mz d) [];
         LPoly d (ab : lista) -> LPoly d (ab : lista);
       };

data TRSProof a b c =
  DPTrans [(Term (Shp (Lab a b)) c, Term (Shp (Lab a b)) c)]
    (DPProof (Shp (Lab a b)) b c)
  | RuleRemoval (Redpair (Lab a b)) [(Term (Lab a b) c, Term (Lab a b) c)]
      (TRSProof a b c)
  | Unlab [(Term (Lab a b) c, Term (Lab a b) c)] (TRSProof a b c)
  | Semlab (Sl_inter a b) (Lab a b -> [Nat] -> Eithera b [(Lab a b)])
      [(Term (Lab a b) c, Term (Lab a b) c)] (TRSProof a b c)
  | Rootlab [(Term (Lab a b) c, Term (Lab a b) c)] (TRSProof a b c)
  | Fcc [(Ctxt (Lab a b) c)] [(Term (Lab a b) c, Term (Lab a b) c)]
      (TRSProof a b c)
  | StringReversal [(Term (Lab a b) c, Term (Lab a b) c)] (TRSProof a b c)
  | RisEmpty;

lpoS :: [Char] -> [Char];
lpoS = shows_string ['>', '_', 'L', 'P', 'O', '+', 'A', 'F'];

linear_term_impl :: forall a b. (Eq a) => [a] -> Term b a -> Maybe [a];
linear_term_impl xs (Var x) = (if member x xs then Nothing else Just (x : xs));
linear_term_impl xs (Fun uu []) = Just xs;
linear_term_impl xs (Fun f (t : ts)) =
  case linear_term_impl xs t of {
    Nothing -> Nothing;
    Just ys -> linear_term_impl ys (Fun f ts);
  };

linear_term :: forall a b. (Eq b) => Term a b -> Bool;
linear_term t =
  case linear_term_impl [] t of {
    Nothing -> False;
    Just _ -> True;
  };

xml2trsInput ::
  forall a.
    (Xml -> Eithera [Char] a) ->
      Xml -> Eithera [Char] [(Term a [Char], Term a [Char])];
xml2trsInput xml2name =
  xml1element ['i', 'n', 'p', 'u', 't']
    (xml1element ['t', 'r', 's', 'I', 'n', 'p', 'u', 't']
      (xml1element ['t', 'r', 's'] (xml2rules xml2name) id) id)
    id;

xml2trsProof ::
  forall a.
    (Xml -> Eithera [Char] (Lab a [Nat])) ->
      (Xml -> Eithera [Char] (Shp (Lab a [Nat]))) ->
        (Nat -> Xml -> Eithera [Char] (Lab (Shp (Lab a [Nat])) [Nat])) ->
          Xml -> Eithera [Char] (TRSProof a [Nat] [Char]);
xml2trsProof xml2name xml2sname xml2lsname x =
  xmlOptions
    [(['r', 'I', 's', 'E', 'm', 'p', 't', 'y'],
       xml0elements ['r', 'I', 's', 'E', 'm', 'p', 't', 'y'] RisEmpty),
      (['d', 'p', 'T', 'r', 'a', 'n', 's'],
        xml3elements ['d', 'p', 'T', 'r', 'a', 'n', 's']
          (xml1element ['d', 'p', 's'] (xml2rules xml2sname) id)
          (xml2bool
            ['m', 'a', 'r', 'k', 'e', 'd', 'S', 'y', 'm', 'b', 'o', 'l', 's'])
          (xml2dpProof Zero_nat xml2lsname) (\ dps _ -> DPTrans dps)),
      (['r', 'u', 'l', 'e', 'R', 'e', 'm', 'o', 'v', 'a', 'l'],
        xml3elements ['r', 'u', 'l', 'e', 'R', 'e', 'm', 'o', 'v', 'a', 'l']
          (xml2redpair xml2name)
          (xml1element ['t', 'r', 's'] (xml2rules xml2name) id)
          (xml2trsProof xml2name xml2sname xml2lsname) RuleRemoval),
      (['u', 'n', 'l', 'a', 'b'],
        xml2elements ['u', 'n', 'l', 'a', 'b']
          (xml1element ['t', 'r', 's'] (xml2rules xml2name) id)
          (xml2trsProof xml2name xml2sname xml2lsname) Unlab),
      (['s', 't', 'r', 'i', 'n', 'g', 'R', 'e', 'v', 'e', 'r', 's', 'a', 'l'],
        xml2elements
          ['s', 't', 'r', 'i', 'n', 'g', 'R', 'e', 'v', 'e', 'r', 's', 'a', 'l']
          (xml1element ['t', 'r', 's'] (xml2rules xml2name) id)
          (xml2trsProof xml2name xml2sname xml2lsname) StringReversal),
      (['s', 'e', 'm', 'l', 'a', 'b'],
        xml3elements ['s', 'e', 'm', 'l', 'a', 'b'] (xml2sl_inter xml2name)
          (xml1element ['t', 'r', 's'] (xml2rules xml2name) id)
          (xml2trsProof xml2name xml2sname xml2lsname)
          (\ a ->
            case a of {
              Nothing -> Rootlab;
              Just i -> Semlab i id_labeling;
            })),
      (['f', 'l', 'a', 't', 'C', 'o', 'n', 't', 'e', 'x', 't', 'C', 'l', 'o',
         's', 'u', 'r', 'e'],
        xml3elements
          ['f', 'l', 'a', 't', 'C', 'o', 'n', 't', 'e', 'x', 't', 'C', 'l', 'o',
            's', 'u', 'r', 'e']
          (xml2flat_contexts xml2name)
          (xml1element ['t', 'r', 's'] (xml2rules xml2name) id)
          (xml2trsProof xml2name xml2sname xml2lsname) Fcc)]
    x;

checkF :: forall a b. (a -> Bool) -> (a -> b) -> a -> Eithera b ();
checkF f g x = (if f x then returna () else errora (g x));

checkAllMI :: forall a b. (a -> Eithera b ()) -> [a] -> Eithera (a, b) ();
checkAllMI f xs =
  foldra (\ x iq -> bind (catcha (f x) (\ xa -> errora (x, xa))) (\ _ -> iq)) xs
    (returna ());

check_edges ::
  forall a.
    (a -> [Char] -> [Char]) ->
      ((a, a) -> Bool) -> [a] -> [a] -> Eithera ([Char] -> [Char]) ();
check_edges ss g c d =
  catcha
    (checkAllMI
      (\ n ->
        catcha
          (checkAllMI
            (\ m ->
              checkB (not (g (m, n)))
                (shows_string ['e', 'd', 'g', 'e', ' ', 'f', 'r', 'o', 'm', ' ']
                  . ss m . shows_string [' ', 't', 'o', ' '] . ss n))
            d)
          (\ x -> errora (snd x)))
      c)
    (\ x -> errora (snd x));

bot_bool :: Bool;
bot_bool = False;

less_bool :: Bool -> Bool -> Bool;
less_bool True b = False;
less_bool False b = b;

less_eq_bool :: Bool -> Bool -> Bool;
less_eq_bool True b = b;
less_eq_bool False b = True;

instance Orda Bool where {
  less_eq = less_eq_bool;
  less = less_bool;
};

instance Preorder Bool where {
};

instance Bot Bool where {
  bot = bot_bool;
};

bot_fun :: forall a b. (Bot b) => a -> b;
bot_fun = (\ _ -> bot);

prightI ::
  forall a b c.
    (Eq a,
      Eq c) => a -> a -> (a -> a -> a) ->
                           (a -> a -> a) ->
                             (a -> a) ->
                               (b -> Nat -> (a, [a])) -> Term b c -> L_poly c a;
prightI ze on pl ti mz pia (Var x) = LPoly ze [(x, on)];
prightI ze on pl ti mz pia (Fun f ts) =
  let {
    (c, asa) = pia f (size_list ts);
    (LPoly d a) =
      sum_lpolyI ze pl (LPoly c [])
        (list_sumI (LPoly ze []) (sum_lpolyI ze pl)
          (mapa (\ at ->
                  mul_lpolyI ze ti (fst at)
                    (prightI ze on pl ti mz pia (snd at)))
            (zipa asa ts)));
  } in LPoly (mz d) a;

data TRSDisproof a b = Loop ([(Term a b)], (Ctxt a b, [(Term a b, Term a b)]))
  | NotWellFormed;

lpoNS :: [Char] -> [Char];
lpoNS = shows_string ['>', '=', '_', 'L', 'P', 'O', '+', 'A', 'F'];

funs_term_list' :: forall a b. Term a b -> [a];
funs_term_list' (Var uu) = [];
funs_term_list' (Fun f ts) = f : flat_map funs_term_list' ts;

funs_rule_list' :: forall a b. (Term a b, Term a b) -> [a];
funs_rule_list' r = append (funs_term_list' (fst r)) (funs_term_list' (snd r));

funs_trs_list :: forall a b. (Eq a) => [(Term a b, Term a b)] -> [a];
funs_trs_list trs = remdups (flat_map funs_rule_list' trs);

vars_term_list' :: forall a b. Term a b -> [b];
vars_term_list' (Var x) = [x];
vars_term_list' (Fun uu ts) = flat_map vars_term_list' ts;

vars_rule_list' :: forall a b. (Term a b, Term a b) -> [b];
vars_rule_list' r = append (vars_term_list' (fst r)) (vars_term_list' (snd r));

vars_trs_list :: forall a b. (Eq b) => [(Term a b, Term a b)] -> [b];
vars_trs_list trs = remdups (flat_map vars_rule_list' trs);

xml2plainName :: Xml -> Eithera [Char] (Lab [Char] [Nat]);
xml2plainName x =
  xmlOptions
    [(['n', 'a', 'm', 'e'], xmlChange (xml2text ['n', 'a', 'm', 'e']) UnLab),
      (['l', 'a', 'b', 'e', 'l', 'e', 'd', 'S', 'y', 'm', 'b', 'o', 'l'],
        xml2elements
          ['l', 'a', 'b', 'e', 'l', 'e', 'd', 'S', 'y', 'm', 'b', 'o', 'l']
          xml2plainName (xml2label xml2plainName)
          (\ f a ->
            case a of {
              Lefta aa -> Lab f aa;
              Righta aa -> FunLab f aa;
            }))]
    x;

xml2sharpName :: Xml -> Eithera [Char] (Shp (Lab [Char] [Nat]));
xml2sharpName x =
  (if xmlGetName x == ['s', 'h', 'a', 'r', 'p']
    then xml1element ['s', 'h', 'a', 'r', 'p'] xml2plainName Sharp x
    else xmlChange xml2plainName Plain x);

length_unique :: forall a. (Eq a) => [a] -> Nat;
length_unique [] = Zero_nat;
length_unique (x : xs) =
  (if member x xs then length_unique xs else Suc (length_unique xs));

data CertificationResult = Certified Bool | Error [Char];

vars_term_list :: forall a b. (Eq b) => Term a b -> [b];
vars_term_list t = remdups (vars_term_list' t);

enum_vectors :: forall a b. [a] -> [b] -> [[(b, a)]];
enum_vectors c [] = [[]];
enum_vectors c (x : xs) =
  let {
    a = enum_vectors c xs;
  } in concat_map (\ vec -> mapa (\ ca -> (x, ca) : vec) c) a;

assign_from_vector :: forall a b. (Eq a) => [(a, b)] -> a -> b;
assign_from_vector vec x = the (lookupa x vec);

eq_lab :: forall a b. (Eq a, Eq b) => Lab a b -> Lab a b -> Bool;
eq_lab (FunLab f ls) (FunLab g ks) = eq_lab f g && list_all2 eq_lab ls ks;
eq_lab (Lab lab l) (Lab lab' l') = eq_lab lab lab' && l == l';
eq_lab (UnLab f) (UnLab f') = f == f';
eq_lab (Lab lab l) (FunLab lab' list') = False;
eq_lab (FunLab lab' list') (Lab lab l) = False;
eq_lab (Lab lab l) (UnLab f') = False;
eq_lab (UnLab f') (Lab lab l) = False;
eq_lab (FunLab lab list) (UnLab f') = False;
eq_lab (UnLab f') (FunLab lab list) = False;

instance (Eq a, Eq b) => Eq (Lab a b) where {
  a == b = eq_lab a b;
};

shows_lab :: forall a b. (Shows a, Shows b) => Lab a b -> [Char] -> [Char];
shows_lab (UnLab f) = showsb f;
shows_lab (Lab f l) =
  shows_lab f . shows_string ['['] . showsb l . shows_string [']'];
shows_lab (FunLab f l) =
  shows_lab f . shows_string ['['] . shows_map shows_lab l . shows_string [']'];

shows_list_lab ::
  forall a b. (Shows a, Shows b) => [(Lab a b)] -> [Char] -> [Char];
shows_list_lab fs = shows_list_aux shows_lab fs;

instance (Shows a, Shows b) => Shows (Lab a b) where {
  showsb = shows_lab;
  shows_list = shows_list_lab;
};

label :: forall a b. Lab a b -> Eithera b [(Lab a b)] -> Lab a b;
label f (Lefta l) = Lab f l;
label f (Righta l) = FunLab f l;

eval_lab ::
  forall a b c d.
    (Lab a b -> [c] -> c) ->
      (Lab a b -> [c] -> Eithera b [(Lab a b)]) ->
        (d -> c) -> Term (Lab a b) d -> (c, Term (Lab a b) d);
eval_lab i l alpha (Var x) = (alpha x, Var x);
eval_lab i l alpha (Fun f ts) =
  let {
    clts = mapa (eval_lab i l alpha) ts;
    cs = mapa fst clts;
    c = i f cs;
    lts = mapa snd clts;
    la = l f cs;
  } in (c, Fun (label f la) lts);

check_sl_rule_ass ::
  forall a b c d.
    (Eq a, Shows a, Eq b, Shows b, Eq c, Shows c, Eq d,
      Shows d) => Bool ->
                    (Lab a b -> [c] -> c) ->
                      (Lab a b -> [c] -> Eithera b [(Lab a b)]) ->
                        [(Term (Lab a b) d, Term (Lab a b) d)] ->
                          (d -> c) ->
                            (Term (Lab a b) d, Term (Lab a b) d) ->
                              Eithera ([Char] -> [Char]) ();
check_sl_rule_ass mc i la lR alpha (l, r) =
  let {
    cl_ll = eval_lab i la alpha l;
    cr_lr = eval_lab i la alpha r;
  } in bind (checkB (implies mc (fst cl_ll == fst cr_lr))
              (shows_string ['r', 'u', 'l', 'e', ' '] .
                shows_rule (l, r) .
                  shows_string
                    [' ', 'v', 'i', 'o', 'l', 'a', 't', 'e', 's', ' ', 't', 'h',
                      'e', ' ', 'm', 'o', 'd', 'e', 'l', ' ', 'c', 'o', 'n',
                      'd', 'i', 't', 'i', 'o', 'n', ',', ' ', '[', 'l', 'h',
                      's', ']', ' ', '=', ' ']
                    . showsb (fst cl_ll :: c) .
                        shows_string
                          [',', ' ', '[', 'r', 'h', 's', ']', ' ', '=', ' ']
                          . showsb (fst cr_lr :: c)))
         (\ _ ->
           checkB (member (snd cl_ll, snd cr_lr) lR)
             (shows_string
                ['l', 'a', 'b', 'e', 'l', 'e', 'd', ' ', 'r', 'u', 'l', 'e',
                  ' ']
               . shows_rule (snd cl_ll, snd cr_lr) .
                   shows_string [' ', 'm', 'i', 's', 's', 'i', 'n', 'g']));

check_sl_rule ::
  forall a b c d.
    (Eq a, Shows a, Eq b, Shows b, Eq c, Shows c, Eq d,
      Shows d) => (Lab a b -> [c] -> c) ->
                    (Lab a b -> [c] -> Eithera b [(Lab a b)]) ->
                      [c] ->
                        Bool ->
                          [(Term (Lab a b) d, Term (Lab a b) d)] ->
                            (Term (Lab a b) d, Term (Lab a b) d) ->
                              Eithera ([Char] -> [Char]) ();
check_sl_rule i la c mc lR (l, r) =
  catcha
    (checkAllMI (\ alpha -> check_sl_rule_ass mc i la lR alpha (l, r))
      (mapa assign_from_vector
        (enum_vectors c (list_union (vars_term_list l) (vars_term_list r)))))
    (\ x -> errora (snd x));

check_sl_model_lab_trs ::
  forall a b c d.
    (Eq a, Shows a, Eq b, Shows b, Eq c, Shows c, Eq d,
      Shows d) => (Lab a b -> [c] -> c) ->
                    (Lab a b -> [c] -> Eithera b [(Lab a b)]) ->
                      [c] ->
                        [(Term (Lab a b) d, Term (Lab a b) d)] ->
                          [(Term (Lab a b) d, Term (Lab a b) d)] ->
                            Eithera ([Char] -> [Char]) ();
check_sl_model_lab_trs i l c lR r =
  catcha (checkAllMI (check_sl_rule i l c True lR) r) (\ x -> errora (snd x));

rl_fun :: forall a b c. a -> b -> Eithera c b;
rl_fun uu cs = Righta cs;

rl_inter_list :: forall a b. (Eq a) => [a] -> a -> a -> b -> a;
rl_inter_list fs d g uu = (if member g fs then g else d);

check_sl_lab_trs ::
  forall a b c d.
    (Eq a, Shows a, Eq b, Shows b, Eq c, Shows c, Eq d,
      Shows d) => (Lab a b -> [c] -> c) ->
                    (Lab a b -> [c] -> Eithera b [(Lab a b)]) ->
                      [c] ->
                        [(Term (Lab a b) d, Term (Lab a b) d)] ->
                          [(Term (Lab a b) d, Term (Lab a b) d)] ->
                            Eithera ([Char] -> [Char]) ();
check_sl_lab_trs i l c lP p =
  catcha (checkAllMI (check_sl_rule i l c False lP) p) (\ x -> errora (snd x));

get_lab :: forall a b. Lab a b -> Eithera b [(Lab a b)];
get_lab (Lab f l) = Lefta l;
get_lab (FunLab f l) = Righta l;

unlab :: forall a b. Lab a b -> Lab a b;
unlab (Lab f l) = f;
unlab (FunLab f l) = f;
unlab (UnLab v) = UnLab v;

check_sl_only_lab_and_decr ::
  forall a b c.
    (Eq a, Shows a, Eq b, Shows b, Eq c,
      Shows c) => ((Eithera a [(Lab b a)], Eithera a [(Lab b a)]) -> Bool) ->
                    [(Term (Lab b a) c, Term (Lab b a) c)] ->
                      [(Term (Lab b a) c, Term (Lab b a) c)] ->
                        Eithera ([Char] -> [Char]) ();
check_sl_only_lab_and_decr gr r =
  (\ xs ->
    catcha
      (checkAllMI
        (\ (ll, lr) ->
          let {
            l = map_term unlab ll;
            ra = map_term unlab lr;
          } in checkB
                 (if eq_term l ra
                   then not (is_var ll) &&
                          (not (is_var lr) &&
                            gr (get_lab (the (root ll)),
                                 get_lab (the (root lr))))
                          || eq_term ll lr && member (l, ra) r
                   else member (l, ra) r)
                 (shows_string
                    ['l', 'a', 'b', 'e', 'l', 'e', 'd', ' ', 'r', 'u', 'l', 'e',
                      ' ']
                   . shows_rule (l, ra) .
                       shows_string
                         [' ', 'i', 's', ' ', 'n', 'e', 'i', 't', 'h', 'e', 'r',
                           ' ', 'd', 'e', 'c', 'r', 'e', 'a', 's', 'i', 'n',
                           'g', ' ', 'n', 'o', 'r', ' ', 'a', ' ', 'l', 'a',
                           'b', 'e', 'l', 'e', 'd', ' ', 'v', 'e', 'r', 's',
                           'i', 'o', 'n', ' ', 'o', 'f', ' ', 'o', 'r', 'i',
                           'g', 'i', 'n', 'a', 'l', ' ', 'r', 'u', 'l', 'e']))
        xs)
      (\ x -> errora (snd x)));

check_sem_lab_tt ::
  forall a b c d.
    (Eq a, Shows a, Eq b, Shows b, Eq c,
      Shows c) => ([(Term (Lab a b) c, Term (Lab a b) c)] ->
                    [(Term (Lab a b) c, Term (Lab a b) c)] ->
                      Eithera ([Char] -> [Char]) ())
                    -> ([(Term (Lab a b) c, Term (Lab a b) c)] ->
                         [(Term (Lab a b) c, Term (Lab a b) c)] ->
                           Eithera ([Char] -> [Char]) ())
                         -> ((Eithera b [(Lab a b)], Eithera b [(Lab a b)]) ->
                              Bool)
                              -> (Lab a b -> [d] -> d) ->
                                   (Lab a b -> [d] -> Eithera b [(Lab a b)]) ->
                                     [(Term (Lab a b) c, Term (Lab a b) c)] ->
                                       [(Term (Lab a b) c, Term (Lab a b) c)] ->
 Eithera ([Char] -> [Char]) ();
check_sem_lab_tt check_lab check_model_lab gr i l r lR =
  catcha
    (bind (check_model_lab lR r) (\ _ -> check_sl_only_lab_and_decr gr r lR))
    (\ x ->
      errora
        (shows_string
           ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'i', 't', 'h', ' ',
             'l', 'a', 'b', 'e', 'l', 'e', 'd', ' ', 'T', 'R', 'S', ':']
          . shows_nl . x));

rl_tt ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => [(Term (Lab a b) [Char], Term (Lab a b) [Char])] ->
                    [(Term (Lab a b) [Char], Term (Lab a b) [Char])] ->
                      Eithera ([Char] -> [Char]) ();
rl_tt r lR =
  let {
    fs = funs_trs_list r;
  } in bind (checkB (not (nulla fs))
              (shows_string
                 ['r', 'o', 'o', 't', '-', 'l', 'a', 'b', 'e', 'l', 'i', 'n',
                   'g', ' ', 'r', 'e', 'q', 'u', 'i', 'r', 'e', 's', ' ', 'a',
                   't', ' ', 'l', 'e', 'a', 's', 't', ' ', 'o', 'n', 'e', ' ',
                   'f', 'u', 'n', 'c', 't', 'i', 'o', 'n', ' ', 's', 'y', 'm',
                   'b', 'o', 'l', ' ', 'i', 'n', ' ', 't', 'h', 'e', ' ', 's',
                   'i', 'g', 'n', 'a', 't', 'u', 'r', 'e']
                . shows_nl))
         (\ _ ->
           let {
             d = hd fs;
           } in check_sem_lab_tt
                  (check_sl_lab_trs (rl_inter_list fs d) rl_fun fs)
                  (check_sl_model_lab_trs (rl_inter_list fs d) rl_fun fs)
                  bot_fun (rl_inter_list fs d) rl_fun r lR);

funas_term_list' :: forall a b. Term a b -> [(a, Nat)];
funas_term_list' (Var uu) = [];
funas_term_list' (Fun f ts) = (f, size_list ts) : flat_map funas_term_list' ts;

funas_rule_list' :: forall a b. (Term a b, Term a b) -> [(a, Nat)];
funas_rule_list' r =
  append (funas_term_list' (fst r)) (funas_term_list' (snd r));

funas_trs_list :: forall a b. (Eq a) => [(Term a b, Term a b)] -> [(a, Nat)];
funas_trs_list trs = remdups (flat_map funas_rule_list' trs);

supteq_list :: forall a b. Term a b -> [(Term a b)];
supteq_list (Var x) = [Var x];
supteq_list (Fun f ts) = Fun f ts : concat_map supteq_list ts;

dP_list ::
  forall a b.
    (Eq a,
      Eq b) => [(Term a b, Term a b)] -> [(Term (Shp a) b, Term (Shp a) b)];
dP_list r =
  let {
    d = defs_list r;
  } in concat_map
         (\ lr ->
           let {
             l = fst lr;
             s = sharpa l;
           } in mapa (\ u -> (s, sharpa u))
                  (filtera
                    (\ u ->
                      not (suptp l u) &&
                        (not (is_var u) && member (the (root u)) d))
                    (supteq_list (snd lr))))
         r;

iD_list ::
  forall a b. [(Term a b, Term a b)] -> [(Term (Shp a) b, Term (Shp a) b)];
iD_list r = mapa (\ lr -> (plaina (fst lr), plaina (snd lr))) r;

indent :: ([Char] -> [Char]) -> [Char] -> [Char];
indent p s =
  flat_map (\ c -> (if c == newline then [newline, ' '] else [c])) (p s);

check_rule_reflecting ::
  forall a.
    (Eq a,
      Shows a) => [(Ctxt a [Char])] ->
                    [(Term a [Char], Term a [Char])] ->
                      (Term a [Char], Term a [Char]) ->
                        Eithera ([Char] -> [Char]) ();
check_rule_reflecting fcs rs rule =
  checkB
    (anya (\ (l, r) ->
            anya (\ c ->
                   eq_term (fst rule) (ctxt_apply c l) &&
                     eq_term (snd rule) (ctxt_apply c r))
              (Hole : fcs))
      rs)
    (shows_string ['t', 'h', 'e', ' ', 'r', 'u', 'l', 'e', ' '] .
      shows_rule rule .
        shows_string
          [' ', 'i', 's', ' ', 'n', 'e', 'i', 't', 'h', 'e', 'r', ' ', 'c', 'o',
            'n', 't', 'a', 'i', 'n', 'e', 'd', ' ', 'i', 'n', ' ', 't', 'h',
            'e', ' ', 'o', 'r', 'i', 'g', 'i', 'n', 'a', 'l', ' ', 's', 'e',
            't', ' ', 'o', 'f', ' ', 'r', 'u', 'l', 'e', 's', ' ', 'n', 'o',
            'r', ' ', 'o', 'b', 't', 'a', 'i', 'n', 'e', 'd', ' ', 'b', 'y',
            ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 'a', ' ', 'f',
            'l', 'a', 't', ' ', 'c', 'o', 'n', 't', 'e', 'x', 't']
          . shows_nl);

check_rule_preserving ::
  forall a.
    (Eq a,
      Shows a) => [(Ctxt a [Char])] ->
                    [(Term a [Char], Term a [Char])] ->
                      (Term a [Char], Term a [Char]) ->
                        Eithera ([Char] -> [Char]) ();
check_rule_preserving fcs rs' rule =
  checkB
    (member rule rs' ||
      foralla
        (\ c -> member (ctxt_apply c (fst rule), ctxt_apply c (snd rule)) rs')
        fcs)
    (shows_string ['t', 'h', 'e', ' ', 'r', 'u', 'l', 'e', ' '] .
      shows_rule rule .
        shows_string
          [' ', 'i', 's', ' ', 'n', 'e', 'i', 't', 'h', 'e', 'r', ' ', 'c', 'o',
            'n', 't', 'a', 'i', 'n', 'e', 'd', ' ', 'i', 'n', ' ', 't', 'h',
            'e', ' ', 'r', 'e', 's', 'u', 'l', 't', 'i', 'n', 'g', ' ', 's',
            'e', 't', ' ', 'o', 'f', ' ', 'r', 'u', 'l', 'e', 's', ' ', 'n',
            'o', 'r', ' ', 'c', 'l', 'o', 's', 'e', 'd', ' ', 'u', 'n', 'd',
            'e', 'r', ' ', 'a', 'l', 'l', ' ', 'f', 'l', 'a', 't', ' ', 'c',
            'o', 'n', 't', 'e', 'x', 't', 's']
          . shows_nl);

hole_at :: forall a b. (Eq a) => Nat -> Nat -> a -> Ctxt a b -> Bool;
hole_at n i f (More g ss1 Hole ss2) =
  g == f &&
    (eq_nat (size_list ss1) i &&
      eq_nat (size_list ss2) (minus_nat (minus_nat n i) one_nat));
hole_at n i f Hole = False;
hole_at n i f (More v va (More vd ve vf vg) vc) = False;

check_flat_ctxt_complete ::
  forall a.
    (Eq a,
      Shows a) => [(Ctxt a [Char])] ->
                    (a, Nat) -> Eithera ([Char] -> [Char]) ();
check_flat_ctxt_complete fcs fa =
  checkB
    (foralla (\ i -> anya (hole_at (snd fa) i (fst fa)) fcs)
      (upt Zero_nat (snd fa)))
    (shows_string
       ['t', 'h', 'e', ' ', 'l', 'i', 's', 't', ' ', 'o', 'f', ' ', 'f', 'l',
         'a', 't', ' ', 'c', 'o', 'n', 't', 'e', 'x', 't', 's', ' ', 'i', 's',
         ' ', 'i', 'n', 'c', 'o', 'm', 'p', 'l', 'e', 't', 'e']
      . shows_nl);

is_flat_ctxt_list ::
  forall a b. (Eq a, Eq b) => [a] -> [(b, Nat)] -> Ctxt b a -> Bool;
is_flat_ctxt_list vs fas (More f ss1 Hole ss2) =
  member (f, Suc (size_list (append ss1 ss2))) fas &&
    (foralla is_var (append ss1 ss2) &&
      (distinct (append ss1 ss2) &&
        nulla (list_inter (mapa the_var (append ss1 ss2)) vs)));
is_flat_ctxt_list vs fas Hole = False;
is_flat_ctxt_list vs fas (More v va (More vd ve vf vg) vc) = False;

check_is_flat_ctxt ::
  forall a.
    (Eq a,
      Shows a) => [[Char]] ->
                    [(a, Nat)] ->
                      Ctxt a [Char] -> Eithera ([Char] -> [Char]) ();
check_is_flat_ctxt vs fas c =
  checkB (is_flat_ctxt_list vs fas c)
    (shows_ctxt c .
      shows_string
        [' ', 'i', 's', ' ', 'n', 'o', 't', ' ', 'a', ' ', 'f', 'l', 'a', 't',
          ' ', 'c', 'o', 'n', 't', 'e', 'x', 't']
        . shows_nl);

check_flat_ctxt ::
  forall a.
    (Eq a,
      Shows a) => [[Char]] -> Ctxt a [Char] -> Eithera ([Char] -> [Char]) ();
check_flat_ctxt vs c =
  case c of {
    Hole ->
      errora
        (shows_ctxt c .
          shows_string
            [' ', 'i', 's', ' ', 'n', 'o', 't', ' ', 'a', ' ', 'f', 'l', 'a',
              't', ' ', 'c', 'o', 'n', 't', 'e', 'x', 't']
            . shows_nl);
    More _ ss1 Hole ss2 ->
      bind (bind (checkB (distinct (append ss1 ss2))
                   (shows_ctxt c .
                     shows_string
                       [' ', 'c', 'o', 'n', 't', 'a', 'i', 'n', 's', ' ', 'd',
                         'u', 'p', 'l', 'i', 'c', 'a', 't', 'e', ' ', 'v', 'a',
                         'r', 'i', 'a', 'b', 'l', 'e', 's']
                       . shows_nl))
             (\ _ ->
               checkB (foralla is_var (append ss1 ss2))
                 (shows_ctxt c .
                   shows_string
                     [' ', 'i', 's', ' ', 'n', 'o', 't', ' ', 'f', 'l', 'a',
                       't', ',', ' ', 'i', '.', 'e', '.', ',', ' ', 'h', 'a',
                       's', ' ', 'd', 'e', 'p', 't', 'h', ' ', 'g', 'r', 'e',
                       'a', 't', 'e', 'r', ' ', 't', 'h', 'a', 'n', ' ', 'o',
                       'n', 'e']
                     . shows_nl)))
        (\ _ ->
          checkB (foralla (\ t -> not (member (the_var t) vs)) (append ss1 ss2))
            (shows_ctxt c .
              shows_string
                [' ', 'h', 'a', 's', ' ', 't', 'o', ' ', 'c', 'o', 'n', 't',
                  'a', 'i', 'n', ' ', 'o', 'n', 'l', 'y', ' ', 'f', 'r', 'e',
                  's', 'h', ' ', 'v', 'a', 'r', 'i', 'a', 'b', 'l', 'e', 's']
                . shows_nl));
    More _ ss1 (More _ _ _ _) ss2 ->
      errora
        (shows_ctxt c .
          shows_string
            [' ', 'i', 's', ' ', 'n', 'o', 't', ' ', 'a', ' ', 'f', 'l', 'a',
              't', ' ', 'c', 'o', 'n', 't', 'e', 'x', 't']
            . shows_nl);
  };

fcc_tt ::
  forall a.
    (Eq a,
      Shows a) => [(Term a [Char], Term a [Char])] ->
                    [(Ctxt a [Char])] ->
                      [(Term a [Char], Term a [Char])] ->
                        Eithera ([Char] -> [Char]) ();
fcc_tt r fcs lR =
  let {
    vs = vars_trs_list r;
    fas = funas_trs_list r;
  } in bind (bind (bind (bind (bind (checkB (not (nulla fcs))
                                      (shows_string
 ['a', 't', ' ', 'l', 'e', 'a', 's', 't', ' ', 'o', 'n', 'e', ' ', 'f', 'l',
   'a', 't', ' ', 'c', 'o', 'n', 't', 'e', 'x', 't', ' ', 'i', 's', ' ', 'r',
   'e', 'q', 'u', 'i', 'r', 'e', 'd', ' ', 'f', 'o', 'r', ' ', 't', 'h', 'e',
   ' ', 'c', 'l', 'o', 's', 'u', 'r', 'e', ' ', 'u', 'n', 'd', 'e', 'r', ' ',
   'f', 'l', 'a', 't', ' ', 'c', 'o', 'n', 't', 'e', 'x', 't', 's']
. shows_nl))
                                (\ _ ->
                                  catcha (checkAllMI (check_flat_ctxt vs) fcs)
                                    (\ x -> errora (snd x))))
                          (\ _ ->
                            catcha (checkAllMI (check_is_flat_ctxt vs fas) fcs)
                              (\ x -> errora (snd x))))
                    (\ _ ->
                      catcha (checkAllMI (check_flat_ctxt_complete fcs) fas)
                        (\ x -> errora (snd x))))
              (\ _ ->
                catcha (checkAllMI (check_rule_preserving fcs lR) r)
                  (\ x -> errora (snd x))))
         (\ _ ->
           catcha (checkAllMI (check_rule_reflecting fcs r) lR)
             (\ x -> errora (snd x)));

pi_full :: forall a. a -> Nat -> Nat -> Bool;
pi_full uu uv uw = True;

terminationGoal :: Xml;
terminationGoal =
  Xml ['g', 'o', 'a', 'l'] []
    [Xml ['t', 'e', 'r', 'm', 'i', 'n', 'a', 't', 'i', 'o', 'n'] [] [] Nothing]
    Nothing;

xml2substitution ::
  forall a.
    (Xml -> Eithera [Char] a) ->
      Xml -> Eithera [Char] [(Term a [Char], Term a [Char])];
xml2substitution xml2name =
  xmlManyElements ['s', 'u', 'b', 's', 't', 'i', 't', 'u', 't', 'i', 'o', 'n']
    (xml2elements ['s', 'u', 'b', 's', 't', 'E', 'n', 't', 'r', 'y'] xml2var
      (xml2term xml2name) (\ a b -> (a, b)))
    id;

xmlMany2Elements ::
  forall a b c d.
    [Char] ->
      (Xml -> Eithera [Char] a) ->
        (Xml -> Eithera [Char] b) ->
          (Xml -> Eithera [Char] c) ->
            ([a] -> b -> c -> d) -> Xml -> Eithera [Char] d;
xmlMany2Elements reqName p1 p2 p3 combi (Xml name atts cs text) =
  let {
    ds = rev cs;
  } in (if name == reqName &&
             (nulla atts &&
               (is_none text && less_eq_nat (nat 2) (size_list cs)))
         then bind (mapMonad p1 (rev (tl (tl ds))))
                (\ a ->
                  bind (p2 (nth ds one_nat))
                    (\ b -> bind (p3 (nth ds Zero_nat)) (returna . combi a b)))
         else errora
                (concata
                  [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r',
                     'a', 'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ',
                     'f', 'o', 'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm',
                     'l', ' ', 'e', 'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e',
                     'x', 'p', 'e', 'c', 't', 'e', 'd', ' '],
                    reqName, [')'], [newline], showa (Xml name atts cs text)]));

xml2trsDisproof ::
  forall a.
    (Xml -> Eithera [Char] a) -> Xml -> Eithera [Char] (TRSDisproof a [Char]);
xml2trsDisproof xml2name =
  xmlOptions
    [(['v', 'a', 'r', 'i', 'a', 'b', 'l', 'e', 'C', 'o', 'n', 'd', 'i', 't',
        'i', 'o', 'n', 'V', 'i', 'o', 'l', 'a', 't', 'e', 'd'],
       xml0elements
         ['v', 'a', 'r', 'i', 'a', 'b', 'l', 'e', 'C', 'o', 'n', 'd', 'i', 't',
           'i', 'o', 'n', 'V', 'i', 'o', 'l', 'a', 't', 'e', 'd']
         NotWellFormed),
      (['l', 'o', 'o', 'p'],
        xmlMany2Elements ['l', 'o', 'o', 'p']
          (xml3elements ['r', 'e', 'd', 'e', 'x'] (xml2term xml2name) xml2pos
            (xml2rule xml2name) (\ t _ _ -> t))
          (xml2substitution xml2name) (xml2context xml2name)
          (\ ts sigma c -> Loop (ts, (c, sigma))))];

nonTerminationGoal :: Xml;
nonTerminationGoal =
  Xml ['g', 'o', 'a', 'l'] []
    [Xml ['n', 'o', 'n', 'T', 'e', 'r', 'm', 'i', 'n', 'a', 't', 'i', 'o', 'n']
       [] [] Nothing]
    Nothing;

data CertificationProblem a b c =
  CertificationProblem Bool [(Term a c, Term a c)]
    (Eithera (TRSProof a b c) (TRSDisproof a c));

xml2certProblem ::
  forall a.
    (Xml -> Eithera [Char] a) ->
      (Xml -> Eithera [Char] (Lab a [Nat])) ->
        (Xml -> Eithera [Char] (Shp (Lab a [Nat]))) ->
          (Nat -> Xml -> Eithera [Char] (Lab (Shp (Lab a [Nat])) [Nat])) ->
            Xml -> Eithera [Char] (CertificationProblem a [Nat] [Char]);
xml2certProblem xml2uname xml2name xml2sname xml2lsname (Xml name egal cs text)
  = (if name ==
          ['c', 'e', 'r', 't', 'i', 'f', 'i', 'c', 'a', 't', 'i', 'o', 'n', 'P',
            'r', 'o', 'b', 'l', 'e', 'm']
          && (is_none text && less_eq_nat (nat 3) (size_list cs))
      then bind (xml2trsInput xml2uname (nth cs one_nat))
             (\ trs ->
               (if eq_xml (nth cs Zero_nat) terminationGoal
                 then bind (xml1element ['p', 'r', 'o', 'o', 'f']
                             (xml2trsProof xml2name xml2sname xml2lsname) id
                             (nth cs (nat 2)))
                        (\ p ->
                          returna (CertificationProblem True trs (Lefta p)))
                 else (if eq_xml (nth cs Zero_nat) nonTerminationGoal
                        then bind (xml1element ['p', 'r', 'o', 'o', 'f']
                                    (xml2trsDisproof xml2uname) id
                                    (nth cs (nat 2)))
                               (\ p ->
                                 returna
                                   (CertificationProblem False trs (Righta p)))
                        else errora
                               ['u', 'n', 'k', 'n', 'o', 'w', 'n', ' ', 'g',
                                 'o', 'a', 'l', ' ', 't', 'y', 'p', 'e', ' ',
                                 'i', 'n', ' ', 'c', 'e', 'r', 't', 'i', 'f',
                                 'i', 'c', 'a', 't', 'i', 'o', 'n', ' ', 'p',
                                 'r', 'o', 'b', 'l', 'e', 'm'])))
      else errora
             ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'p', 'a', 'r',
               's', 'e', ' ', 'r', 'o', 'o', 't', ' ', 'e', 'l', 'e', 'm', 'e',
               'n', 't']);

data Filtered a = FPair a Nat;

checkSubset :: forall a. (Eq a) => [a] -> [a] -> Eithera a ();
checkSubset xs ys =
  catcha (checkAllMI (checkF (\ x -> member x ys) id) xs)
    (\ x -> errora (snd x));

check_wf_trs ::
  forall a b.
    (Shows a, Eq b,
      Shows b) => [(Term a b, Term a b)] -> Eithera ([Char] -> [Char]) ();
check_wf_trs r =
  catcha
    (catcha
      (checkAllMI
        (\ rule ->
          catcha
            (bind (checkB (not (is_var (fst rule)))
                    (shows_string
                      ['v', 'a', 'r', 'i', 'a', 'b', 'l', 'e', ' ', 'l', 'e',
                        'f', 't', '-', 'h', 'a', 'n', 'd', ' ', 's', 'i', 'd',
                        'e', ' ', 'i', 'n']))
              (\ _ ->
                catcha
                  (checkSubset (vars_term_list (snd rule))
                    (vars_term_list (fst rule)))
                  (\ x ->
                    errora
                      (shows_string
                         ['f', 'r', 'e', 'e', ' ', 'v', 'a', 'r', 'i', 'a', 'b',
                           'l', 'e', ' ']
                        . showsb x .
                            shows_string
                              [' ', 'i', 'n', ' ', 'r', 'i', 'g', 'h', 't', '-',
                                'h', 'a', 'n', 'd', ' ', 's', 'i', 'd', 'e',
                                ' ', 'o', 'f']))))
            (\ x ->
              errora
                (x . shows_string [' ', 'r', 'u', 'l', 'e', ' '] .
                       shows_rule rule . shows_nl)))
        r)
      (\ x -> errora (snd x)))
    (\ x ->
      errora
        (shows_string
           ['t', 'h', 'e', ' ', 'T', 'R', 'S', ' ', 'i', 's', ' ', 'n', 'o',
             't', ' ', 'w', 'e', 'l', 'l', '-', 'f', 'o', 'r', 'm', 'e', 'd']
          . shows_nl . x));

check_DP ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => [(Term a b, Term a b)] ->
                    [(Term (Shp a) b, Term (Shp a) b)] ->
                      Eithera ([Char] -> [Char]) ();
check_DP r p =
  catcha
    (bind (check_wf_trs r)
      (\ _ ->
        catcha (checkSubset (dP_list r) p)
          (\ x ->
            errora
              (shows_string ['t', 'h', 'e', ' ', 'D', 'P', ' '] .
                shows_rule x .
                  shows_string
                    [' ', 'd', 'o', 'e', 's', ' ', 'n', 'o', 't', ' ', 'a', 'p',
                      'p', 'e', 'a', 'r', ' ', 'i', 'n', ' ', 't', 'h', 'e',
                      ' ', 'D', 'P', ' ', 'p', 'r', 'o', 'b', 'l', 'e', 'm']
                    . shows_nl))))
    (\ x ->
      errora
        (shows_string
           ['t', 'h', 'e', ' ', 'D', 'P', '-', 't', 'r', 'a', 'n', 's', 'f',
             'o', 'r', 'm', 'a', 't', 'i', 'o', 'n', ' ', 'i', 's', ' ', 'n',
             'o', 't', ' ', 'a', 'p', 'p', 'l', 'i', 'e', 'd', ' ', 'c', 'o',
             'r', 'r', 'e', 'c', 't', 'l', 'y', '.']
          . shows_nl . x));

ge_nat :: Nat -> Nat -> Bool;
ge_nat x y = less_eq_nat y x;

mat_mono :: forall a. Nat -> (a -> Bool) -> [[a]] -> Bool;
mat_mono sd m = mat_monoI m (Suc (minus_nat sd one_nat));

nat_name :: [Char];
nat_name = ['n', 'a', 't', 'u', 'r', 'a', 'l', 's'];

create_afI ::
  forall a b.
    (Eq a, Eq b) => a -> [((b, Nat), (a, [a]))] -> b -> Nat -> Nat -> Bool;
create_afI ze [] f n i = True;
create_afI ze (((g, m), (c, coeffs)) : rest) f n i =
  (if f == g && eq_nat m n
    then less_nat i (size_list coeffs) && not (nth coeffs i == ze)
    else create_afI ze rest f n i);

lpo_repr :: [Char] -> [Char];
lpo_repr = shows_string ['L', 'P', 'O', '+', 'A', 'F'];

check_sem_lab_proc ::
  forall a b c d.
    (Eq a, Shows a, Eq b, Shows b, Eq c,
      Shows c) => ([(Term (Lab a b) c, Term (Lab a b) c)] ->
                    [(Term (Lab a b) c, Term (Lab a b) c)] ->
                      Eithera ([Char] -> [Char]) ())
                    -> ([(Term (Lab a b) c, Term (Lab a b) c)] ->
                         [(Term (Lab a b) c, Term (Lab a b) c)] ->
                           Eithera ([Char] -> [Char]) ())
                         -> ((Eithera b [(Lab a b)], Eithera b [(Lab a b)]) ->
                              Bool)
                              -> (Lab a b -> [d] -> d) ->
                                   (Lab a b -> [d] -> Eithera b [(Lab a b)]) ->
                                     [(Term (Lab a b) c, Term (Lab a b) c)] ->
                                       [(Term (Lab a b) c, Term (Lab a b) c)] ->
 [(Term (Lab a b) c, Term (Lab a b) c)] ->
   [(Term (Lab a b) c, Term (Lab a b) c)] -> Eithera ([Char] -> [Char]) ();
check_sem_lab_proc check_lab check_model_lab gr i l p r lP lR =
  bind (bind (catcha (check_lab lP p)
               (\ x ->
                 errora
                   (shows_string
                      ['n', 'o', 't', ' ', 'a', 'l', 'l', ' ', 'l', 'a', 'b',
                        'e', 'l', 'e', 'd', ' ', 'D', 'P', 's', ' ', 'p', 'r',
                        'e', 's', 'e', 'n', 't', ':']
                     . shows_nl . x)))
         (\ _ ->
           catcha (check_model_lab lR r)
             (\ x ->
               errora
                 (shows_string
                    ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'i', 't', 'h',
                      ' ', 'l', 'a', 'b', 'e', 'l', 'e', 'd', ' ', 'T', 'R',
                      'S', ':']
                   . shows_nl . x))))
    (\ _ ->
      catcha (check_sl_only_lab_and_decr gr r lR)
        (\ x -> errora (x . shows_nl)));

list_diff :: forall a. (Eq a) => [a] -> [a] -> [a];
list_diff xs [] = xs;
list_diff xs (y : ys) = list_diff (removeAll y xs) ys;

rl_proc ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => [(Term (Lab a b) [Char], Term (Lab a b) [Char])] ->
                    [(Term (Lab a b) [Char], Term (Lab a b) [Char])] ->
                      [(Term (Lab a b) [Char], Term (Lab a b) [Char])] ->
                        [(Term (Lab a b) [Char], Term (Lab a b) [Char])] ->
                          Eithera ([Char] -> [Char]) ();
rl_proc p r lP lR =
  let {
    fs = list_union (funs_trs_list r)
           (list_diff (funs_trs_list p) (defs_list p));
  } in bind (checkB (not (nulla fs))
              (shows_string
                 ['r', 'o', 'o', 't', '-', 'l', 'a', 'b', 'e', 'l', 'i', 'n',
                   'g', ' ', 'r', 'e', 'q', 'u', 'i', 'r', 'e', 's', ' ', 'a',
                   't', ' ', 'l', 'e', 'a', 's', 't', ' ', 'o', 'n', 'e', ' ',
                   'f', 'u', 'n', 'c', 't', 'i', 'o', 'n', ' ', 's', 'y', 'm',
                   'b', 'o', 'l', ' ', 'i', 'n', ' ', 't', 'h', 'e', ' ', 's',
                   'i', 'g', 'n', 'a', 't', 'u', 'r', 'e']
                . shows_nl))
         (\ _ ->
           let {
             d = hd fs;
           } in check_sem_lab_proc
                  (check_sl_lab_trs (rl_inter_list fs d) rl_fun fs)
                  (check_sl_model_lab_trs (rl_inter_list fs d) rl_fun fs)
                  bot_fun (rl_inter_list fs d) rl_fun p r lP lR);

funas_term_list :: forall a b. (Eq a) => Term a b -> [(a, Nat)];
funas_term_list t = remdups (funas_term_list' t);

xml2labSharpName ::
  Nat -> Xml -> Eithera [Char] (Lab (Shp (Lab [Char] [Nat])) [Nat]);
xml2labSharpName Zero_nat = xmlChange xml2sharpName UnLab;
xml2labSharpName (Suc n) =
  xml2elements ['l', 'a', 'b', 'e', 'l', 'e', 'd', 'S', 'y', 'm', 'b', 'o', 'l']
    (xml2labSharpName n) (xml2label (xml2labSharpName n))
    (\ f a -> case a of {
                Lefta aa -> Lab f aa;
                Righta aa -> FunLab f aa;
              });

xmldoc2certProblem ::
  Xmldoc -> Eithera [Char] (CertificationProblem [Char] [Nat] [Char]);
xmldoc2certProblem (Xmldoc header xml) =
  debug ['0']
    ['p', 'a', 'r', 's', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 't', 'o', ' ',
      'c', 'e', 'r', 't', 'i', 'f', 'i', 'c', 'a', 't', 'i', 'o', 'n', ' ', 'p',
      'r', 'o', 'b', 'l', 'e', 'm']
    (xml2certProblem (xml2text ['n', 'a', 'm', 'e']) xml2plainName xml2sharpName
      xml2labSharpName xml);

parseCertProblem ::
  [Char] -> Eithera [Char] (CertificationProblem [Char] [Nat] [Char]);
parseCertProblem s = bind (parseXMLDoc s) xmldoc2certProblem;

checkAllLI_help ::
  forall a b. (a -> Nat -> Eithera b ()) -> [a] -> Nat -> Eithera b ();
checkAllLI_help uu [] uv = returna ();
checkAllLI_help f (x : xs) n =
  bind (f x n) (\ _ -> checkAllLI_help f xs (Suc n));

checkAllLI :: forall a b. (a -> Nat -> Eithera b ()) -> [a] -> Eithera b ();
checkAllLI f xs = checkAllLI_help f xs Zero_nat;

show_pvars :: forall a b. (Eq a, Shows a, Shows b) => a -> [(b, a)] -> [[Char]];
show_pvars on [] = [];
show_pvars on ((x, c) : vas) =
  append (if on == c then [] else showa c) (showa x) : show_pvars on vas;

af_entry_case :: forall a. (Nat -> a) -> ([Nat] -> a) -> Af_entry -> a;
af_entry_case f1 f2 (AFList list) = f2 list;
af_entry_case f1 f2 (Collapse nat) = f1 nat;

shows_afs :: forall a. (Shows a) => [((a, Nat), Af_entry)] -> [Char] -> [Char];
shows_afs af =
  shows_map
    (\ (a, b) ->
      let {
        (f, n) = a;
      } in (\ e ->
             shows_string ['p', 'i', '('] .
               showsb f .
                 shows_string ['/'] .
                   shows_nat n .
                     shows_string [')', ' ', '=', ' '] .
                       case e of {
                         Collapse i -> shows_nat (Suc i);
                         AFList ids -> shows_lista (mapa Suc ids);
                       }
                         . shows_nl)
        b)
    af;

block_term :: forall a b. a -> Term a b -> Term a b;
block_term f (Var x) = Var x;
block_term f (Fun g ts) = Fun g (mapa (\ t -> Fun f [t]) ts);

block_rule :: forall a b. a -> (Term a b, Term a b) -> (Term a b, Term a b);
block_rule f r = (block_term f (fst r), block_term f (snd r));

check_superset_of_blocked ::
  forall a.
    (Eq a,
      Shows a) => a -> [(Term a [Char], Term a [Char])] ->
                         [(Term a [Char], Term a [Char])] ->
                           Eithera ([Char] -> [Char]) ();
check_superset_of_blocked f r' r =
  catcha
    (catcha
      (catcha
        (checkAllMI (checkF (\ rule -> member (block_rule f rule) r') id) r)
        (\ x -> errora (snd x)))
      (\ x ->
        errora
          (shows_string ['t', 'h', 'e', ' ', 'r', 'u', 'l', 'e', ' '] .
            shows_rule (block_rule f x) .
              shows_string
                [' ', 'i', 's', ' ', 'm', 'i', 's', 's', 'i', 'n', 'g']
                . shows_nl)))
    (\ x ->
      errora
        (shows_trs r .
          shows_string
            ['i', 's', ' ', 'n', 'o', 't', ' ', 'a', ' ', 's', 'u', 'b', 's',
              'e', 't', ' ', 'o', 'f']
            . shows_trs r' . x . shows_nl));

check_no_defined_right_roots ::
  forall a.
    (Eq a,
      Shows a) => [(Term a [Char], Term a [Char])] ->
                    [a] -> Eithera ([Char] -> [Char]) ();
check_no_defined_right_roots trs fs =
  checkB (foralla (\ r -> not (member (the (root (snd r))) fs)) trs)
    (shows_string
       ['t', 'h', 'e', 'r', 'e', ' ', 'i', 's', ' ', 'a', ' ', 'r', 'i', 'g',
         'h', 't', '-', 'h', 'a', 'n', 'd', ' ', 's', 'i', 'd', 'e', ' ', 'i',
         'n', ' ', 't', 'h', 'e', ' ', 'D', 'P', 's', ' ', 'w', 'i', 't', 'h',
         ' ', 'd', 'e', 'f', 'i', 'n', 'e', 'd', ' ', 'r', 'o', 'o', 't', ' ',
         '(', 'w', '.', 'r', '.', 't', '.', ' ', 't', 'h', 'e', ' ', 'r', 'u',
         'l', 'e', 's', ')']
      . shows_nl);

check_no_vars ::
  forall a.
    (Shows a) => [(Term a [Char], Term a [Char])] ->
                   Eithera ([Char] -> [Char]) ();
check_no_vars trs =
  checkB (foralla (\ r -> not (is_var (fst r)) && not (is_var (snd r))) trs)
    (shows_string
       ['a', 'l', 'l', ' ', 'l', 'h', 's', 's', ' ', 'a', 'n', 'd', ' ', 'r',
         'h', 's', 's', ' ', 'n', 'e', 'e', 'd', ' ', 't', 'o', ' ', 'b', 'e',
         ' ', 'n', 'o', 'n', '-', 'v', 'a', 'r', 'i', 'a', 'b', 'l', 'e', ' ',
         't', 'e', 'r', 'm', 's']
      . shows_nl);

check_left_linear_trs ::
  forall a.
    (Shows a) => [(Term a [Char], Term a [Char])] ->
                   Eithera ([Char] -> [Char]) ();
check_left_linear_trs trs =
  checkB (foralla (\ r -> linear_term (fst r)) trs)
    (shows_lista trs .
      shows_nl .
        shows_string
          ['i', 's', ' ', 'n', 'o', 't', ' ', 'l', 'e', 'f', 't', '-', 'l', 'i',
            'n', 'e', 'a', 'r']
          . shows_nl);

funas_below_term_list' :: forall a b. Term a b -> [(a, Nat)];
funas_below_term_list' t = flat_map funas_term_list' (args t);

funas_below_rule_list' :: forall a b. (Term a b, Term a b) -> [(a, Nat)];
funas_below_rule_list' r =
  append (funas_below_term_list' (fst r)) (funas_below_term_list' (snd r));

funas_below_trs_list ::
  forall a b. (Eq a) => [(Term a b, Term a b)] -> [(a, Nat)];
funas_below_trs_list trs = remdups (flat_map funas_below_rule_list' trs);

fcc_proc ::
  forall a.
    (Eq a,
      Shows a) => [(Term a [Char], Term a [Char])] ->
                    [(Term a [Char], Term a [Char])] ->
                      a -> [(Ctxt a [Char])] ->
                             [(Term a [Char], Term a [Char])] ->
                               [(Term a [Char], Term a [Char])] ->
                                 Eithera ([Char] -> [Char]) ();
fcc_proc p r f fcs p' r' =
  let {
    ds = defs_list r;
    fa = (f, Suc Zero_nat);
    cf = More f [] Hole [];
    fcs' = cf : fcs;
    vs = vars_trs_list r;
    fs = list_union (funas_below_trs_list p) (funas_trs_list r);
    fas = fa : fs;
  } in bind (bind (bind (bind (bind (bind (bind
    (bind (bind (bind (bind (checkB (not (member fa fs))
                              (showsa fa .
                                shows_string
                                  [' ', 'i', 's', ' ', 'n', 'o', 't', ' ', 'f',
                                    'r', 'e', 's', 'h']
                                  . shows_nl))
                        (\ _ ->
                          checkB (not (member f ds))
                            (showsb f .
                              shows_string
                                [' ', 'i', 's', ' ', 'd', 'e', 'f', 'i', 'n',
                                  'e', 'd', ' ', 'i', 'n', ' ', 't', 'h', 'e',
                                  ' ', 'r', 'u', 'l', 'e', 's']
                                . shows_nl)))
                  (\ _ -> check_left_linear_trs r))
            (\ _ -> check_wf_trs r))
      (\ _ -> check_no_vars p))
    (\ _ -> check_no_defined_right_roots p ds))
                                      (\ _ ->
catcha (checkAllMI (check_flat_ctxt vs) fcs') (\ x -> errora (snd x))))
                                (\ _ ->
                                  catcha
                                    (checkAllMI (check_is_flat_ctxt vs fas)
                                      fcs')
                                    (\ x -> errora (snd x))))
                          (\ _ ->
                            catcha
                              (checkAllMI (check_flat_ctxt_complete fcs') fas)
                              (\ x -> errora (snd x))))
                    (\ _ ->
                      catcha (checkAllMI (check_rule_preserving fcs' r') r)
                        (\ x -> errora (snd x))))
              (\ _ ->
                catcha (checkAllMI (check_rule_reflecting fcs' r) r')
                  (\ x -> errora (snd x))))
         (\ _ -> check_superset_of_blocked f p' p);

get_arg :: forall a b. Term a b -> Nat -> Term a b;
get_arg t Zero_nat = t;
get_arg t (Suc n) = nth (args t) n;

int_mono :: Integer -> Bool;
int_mono x = 1 <= x;

max0_nat :: Nat -> Nat;
max0_nat x = x;

nat_mono :: Nat -> Bool;
nat_mono x = not (eq_nat x Zero_nat);

rat_mono :: Rat -> Bool;
rat_mono x = less_eq_rat (Fract 1 1) x;

lookup_rest :: forall a b. (Eq a) => a -> [(a, b)] -> Maybe (b, [(a, b)]);
lookup_rest x [] = Nothing;
lookup_rest x ((y, c) : ycs) =
  (if x == y then Just (c, ycs)
    else case lookup_rest x ycs of {
           Nothing -> Nothing;
           Just (d, yccs) -> Just (d, (y, c) : yccs);
         });

shows_lpoly ::
  forall a b.
    (Eq a, Shows a, Shows b) => a -> a -> L_poly b a -> [Char] -> [Char];
shows_lpoly ze on (LPoly c cs) =
  case show_pvars on cs of {
    [] -> showsb c;
    a : list ->
      (if c == ze then id else showsb c . shows_string [' ', '+', ' ']) .
        shows_list_gen shows_lista [] [] [' ', '+', ' '] [] (a : list);
  };

wf_fun_terms_list :: forall a b. (Eq a) => [(a, Nat)] -> Term a b -> Bool;
wf_fun_terms_list uu (Var uv) = True;
wf_fun_terms_list sig (Fun f ts) =
  member (f, size_list ts) sig && foralla (wf_fun_terms_list sig) ts;

apply_af_entry :: forall a b. a -> Af_entry -> [(Term a b)] -> Term a b;
apply_af_entry uu (Collapse i) ts = nth ts i;
apply_af_entry f (AFList is) ts = Fun f (mapa (nth ts) is);

afs_term ::
  forall a b. (a -> Nat -> Af_entry) -> Term a b -> Term (Filtered a) b;
afs_term pia (Fun f ts) =
  let {
    l = size_list ts;
  } in apply_af_entry (FPair f l) (pia f l) (mapa (afs_term pia) ts);
afs_term pia (Var x) = Var x;

search_len :: [Char] -> [Char] -> Nat -> Maybe ([Char], [Char]);
search_len t s n =
  (if takea n t == s then Just ([], dropa n t)
    else case t of {
           [] -> Nothing;
           h : tt ->
             case search_len tt s n of {
               Nothing -> Nothing;
               Just (bef, aft) -> Just (h : bef, aft);
             };
         });

reverse_trs :: forall a b. [(Term a b, Term a b)] -> [(Term a b, Term a b)];
reverse_trs r = mapa (\ lr -> (snd lr, fst lr)) r;

fresh_strings_list :: [Char] -> Nat -> [[Char]] -> Nat -> [[Char]];
fresh_strings_list name offset used n =
  takea n
    (filtera (\ s -> not (member s (remdups used)))
      (mapa (\ i -> append name (showa (plus_nat i offset)))
        (upt Zero_nat (plus_nat n (length_unique used)))));

shows_arctic :: Arctic -> [Char] -> [Char];
shows_arctic (Num_arc i) = shows_int i;
shows_arctic MinInfty = shows_string ['-', 'i', 'n', 'f'];

shows_list_arctic :: [Arctic] -> [Char] -> [Char];
shows_list_arctic asa = shows_list_aux shows_arctic asa;

instance Shows Arctic where {
  showsb = shows_arctic;
  shows_list = shows_list_arctic;
};

check_pvarsI ::
  forall a b.
    (Eq b) => a -> (a -> a -> Bool) -> [(b, a)] -> [(b, a)] -> Eithera b ();
check_pvarsI ze ge vas [] =
  catcha (checkAllMI (\ va -> checkB (ge (snd va) ze) (fst va)) vas)
    (\ x -> errora (snd x));
check_pvarsI ze ge vas ((x, b) : vbs) =
  let {
    (a', vas') =
      case lookup_rest x vas of {
        Nothing -> (ze, vas);
        Just (a, ba) -> (a, ba);
      };
  } in bind (checkB (ge a' b) x) (\ _ -> check_pvarsI ze ge vas' vbs);

check_lpoly_sI ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => Bool ->
                    a -> a -> (a -> a -> Bool) ->
                                (a -> a -> Bool) ->
                                  L_poly b a ->
                                    L_poly b a -> Eithera ([Char] -> [Char]) ();
check_lpoly_sI mode ze on ge gt (LPoly a vas) (LPoly b vbs) =
  catcha
    (bind (checkB (gt a b)
            (shows_string
              ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'h', 'e', 'n', ' ',
                'c', 'o', 'm', 'p', 'a', 'r', 'i', 'n', 'g', ' ', 'c', 'o', 'n',
                's', 't', 'a', 'n', 't', ' ', 'p', 'a', 'r', 't']))
      (\ _ ->
        catcha (check_pvarsI ze (if mode then ge else gt) vas vbs)
          (\ x ->
            errora
              (shows_string
                 ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'h', 'e', 'n',
                   ' ', 'c', 'o', 'm', 'p', 'a', 'r', 'i', 'n', 'g', ' ', 'c',
                   'o', 'e', 'f', 'f', 'i', 'c', 'i', 'e', 'n', 't', 's', ' ',
                   'o', 'f', ' ', 'v', 'a', 'r', 'i', 'a', 'b', 'l', 'e', ' ']
                . showsb x))))
    (\ x ->
      errora
        (shows_string
           ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'h', 'e', 'n', ' ',
             'c', 'o', 'm', 'p', 'a', 'r', 'i', 'n', 'g', ' ']
          . shows_lpoly ze on (LPoly a vas) .
              shows_string [' ', '>', ' '] .
                shows_lpoly ze on (LPoly b vbs) . shows_nl . x));

check_polo_sI ::
  forall a b c.
    (Eq a, Shows a, Shows b, Eq c,
      Shows c) => Bool ->
                    a -> a -> (a -> a -> a) ->
                                (a -> a -> a) ->
                                  (a -> a) ->
                                    (a -> a -> Bool) ->
                                      (a -> a -> Bool) ->
(b -> Nat -> (a, [a])) -> (Term b c, Term b c) -> Eithera ([Char] -> [Char]) ();
check_polo_sI mode ze on pl ti mz ge gt pia (s, t) =
  let {
    left = pleftI ze on pl ti mz pia s;
    right = prightI ze on pl ti mz pia t;
  } in catcha (check_lpoly_sI mode ze on ge gt left right)
         (\ x ->
           errora
             (shows_string
                ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'e', 'n',
                  's', 'u', 'r', 'e', ' ']
               . shows_term s .
                   shows_string [' ', '>', ' '] . shows_term t . shows_nl . x));

rev_term :: forall a b. Ctxt a b -> Term a b -> Term a b;
rev_term c (Var x) = ctxt_apply c (Var x);
rev_term c (Fun f [t]) = rev_term (More f [] c []) t;

rev_rule :: forall a b. (Term a b, Term a b) -> (Term a b, Term a b);
rev_rule (l, r) = (rev_term Hole l, rev_term Hole r);

filtered_fun :: forall a. Filtered a -> a;
filtered_fun (FPair f n) = f;

shows_filtered :: forall a. (Shows a) => Filtered a -> [Char] -> [Char];
shows_filtered ff = showsb (filtered_fun ff :: a);

shows_list_filtered ::
  forall a. (Shows a) => [(Filtered a)] -> [Char] -> [Char];
shows_list_filtered ffs = shows_list_aux shows_filtered ffs;

instance (Shows a) => Shows (Filtered a) where {
  showsb = shows_filtered;
  shows_list = shows_list_filtered;
};

afs_check ::
  forall a b.
    (Shows a,
      Shows b) => ([Char] -> [Char]) ->
                    (a -> Nat -> Af_entry) ->
                      ((Term (Filtered a) b, Term (Filtered a) b) ->
                        Eithera ([Char] -> [Char]) ())
                        -> (Term a b, Term a b) ->
                             Eithera ([Char] -> [Char]) ();
afs_check rel pia g lr =
  let {
    pl = afs_term pia (fst lr);
    pr = afs_term pia (snd lr);
  } in catcha (g (pl, pr))
         (\ x ->
           errora
             (shows_string
                ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'o', 'r',
                  'i', 'e', 'n', 't', ' ']
               . shows_term (fst lr) .
                   shows_string [' '] .
                     rel . shows_string [' '] .
                             shows_term (snd lr) .
                               shows_nl .
                                 shows_string ['p', 'i', '(', ' '] .
                                   shows_term (fst lr) .
                                     shows_string [' ', ')', ' ', '=', ' '] .
                                       shows_term pl .
 shows_nl .
   shows_string ['p', 'i', '(', ' '] .
     shows_term (snd lr) .
       shows_string [' ', ')', ' ', '=', ' '] . shows_term pr . shows_nl . x));

checkAllCombinations ::
  forall a b. (a -> a -> Eithera b ()) -> [a] -> Eithera ((a, a), b) ();
checkAllCombinations c xs =
  foldR (\ x ys res ->
          catcha
            (checkAllMI
              (\ y ->
                bind (catcha (c x y) (\ xa -> errora ((x, y), xa)))
                  (\ _ -> res))
              ys)
            (\ xa -> errora (snd xa)))
    xs (returna ());

check_graph_decomp ::
  forall a.
    (a -> [Char] -> [Char]) ->
      ((a, a) -> Bool) -> [(Bool, [a])] -> Eithera ([Char] -> [Char]) ();
check_graph_decomp ss g rcs =
  bind (catcha (checkAllCombinations (check_edges ss g) (mapa snd rcs))
         (\ x -> errora (snd x)))
    (\ _ ->
      catcha
        (checkAllMI (\ c -> check_edges ss g c c)
          (mapa snd (filtera (\ rc -> not (fst rc)) rcs)))
        (\ x -> errora (snd x)));

check_lpoly_nsI ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => a -> a -> (a -> a -> Bool) ->
                              L_poly b a ->
                                L_poly b a -> Eithera ([Char] -> [Char]) ();
check_lpoly_nsI ze on ge (LPoly a vas) (LPoly b vbs) =
  catcha
    (bind (checkB (ge a b)
            (shows_string
              ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'h', 'e', 'n', ' ',
                'c', 'o', 'm', 'p', 'a', 'r', 'i', 'n', 'g', ' ', 'c', 'o', 'n',
                's', 't', 'a', 'n', 't', ' ', 'p', 'a', 'r', 't', 's']))
      (\ _ ->
        catcha (check_pvarsI ze ge vas vbs)
          (\ x ->
            errora
              (shows_string
                 ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'h', 'e', 'n',
                   ' ', 'c', 'o', 'm', 'p', 'a', 'r', 'i', 'n', 'g', ' ', 'c',
                   'o', 'e', 'f', 'f', 'i', 'c', 'i', 'e', 'n', 't', 's', ' ',
                   'o', 'f', ' ', 'v', 'a', 'r', 'i', 'a', 'b', 'l', 'e', ' ']
                . showsb x))))
    (\ x ->
      errora
        (shows_string
           ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'h', 'e', 'n', ' ',
             'c', 'o', 'm', 'p', 'a', 'r', 'i', 'n', 'g', ' ']
          . shows_lpoly ze on (LPoly a vas) .
              shows_string [' ', '>', '=', ' '] .
                shows_lpoly ze on (LPoly b vbs) . shows_nl . x));

check_polo_nsI ::
  forall a b c.
    (Eq a, Shows a, Shows b, Eq c,
      Shows c) => a -> a -> (a -> a -> a) ->
                              (a -> a -> a) ->
                                (a -> a) ->
                                  (a -> a -> Bool) ->
                                    (b -> Nat -> (a, [a])) ->
                                      (Term b c, Term b c) ->
Eithera ([Char] -> [Char]) ();
check_polo_nsI ze on pl ti mz ge pia (s, t) =
  let {
    left = pleftI ze on pl ti mz pia s;
    right = prightI ze on pl ti mz pia t;
  } in catcha (check_lpoly_nsI ze on ge left right)
         (\ x ->
           errora
             (shows_string
                ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'e', 'n',
                  's', 'u', 'r', 'e', ' ']
               . shows_term s .
                   shows_string [' ', '>', '=', ' '] .
                     shows_term t . shows_nl . x));

to_lpoly_inter ::
  forall a b. (Eq b) => a -> [((b, Nat), (a, [a]))] -> b -> Nat -> (a, [a]);
to_lpoly_inter on i f n =
  case lookupa (f, n) i of {
    Nothing -> (on, replicatea n on);
    Just cas -> cas;
  };

internal_error :: [Char] -> [Char];
internal_error =
  shows_string
    ['i', 'n', 't', 'e', 'r', 'n', 'a', 'l', ' ', 'e', 'r', 'r', 'o', 'r']
    . shows_nl .
        shows_string
          ['p', 'l', 'e', 'a', 's', 'e', ' ', 'c', 'o', 'n', 't', 'a', 'c', 't',
            ' ', 'o', 'n', 'e', ' ', 'o', 'f', ' ', 't', 'h', 'e', ' ', 'C',
            'e', 'T', 'A', ' ', 'm', 'e', 'm', 'b', 'e', 'r', 's']
          . shows_nl .
              shows_string
                ['h', 't', 't', 'p', ':', '/', '/', 'c', 'l', '-', 'i', 'n',
                  'f', 'o', 'r', 'm', 'a', 't', 'i', 'k', '.', 'u', 'i', 'b',
                  'k', '.', 'a', 'c', '.', 'a', 't', '/', 's', 'o', 'f', 't',
                  'w', 'a', 'r', 'e', '/', 'c', 'e', 't', 'a']
                . shows_nl .
                    shows_string
                      ['c', 'e', 't', 'a', '@', 'i', 'n', 'f', 'o', 'r', 'm',
                        'a', 't', 'i', 'k', '.', 'u', 'i', 'b', 'k', '.', 'a',
                        'c', '.', 'a', 't']
                      . shows_nl;

checkRedPairProc ::
  forall a b.
    (Eq a,
      Eq b) => ((Term a b, Term a b) -> Eithera ([Char] -> [Char]) ()) ->
                 ((Term a b, Term a b) -> Eithera ([Char] -> [Char]) ()) ->
                   ([Char] -> [Char]) ->
                     ([Char] -> [Char]) ->
                       [(Term a b, Term a b)] ->
                         [(Term a b, Term a b)] ->
                           [(Term a b, Term a b)] ->
                             Eithera ([Char] -> [Char]) ();
checkRedPairProc checkSS checkNSS name representation p rP r =
  let {
    (_, nSp) = partition (\ foo -> isValue (checkSS foo)) p;
  } in catcha
         (bind (bind (catcha
                       (catcha (checkAllMI checkNSS r) (\ x -> errora (snd x)))
                       (\ x ->
                         errora
                           (shows_string
                              ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'h',
                                'e', 'n', ' ', 'o', 'r', 'i', 'e', 'n', 't',
                                'i', 'n', 'g', ' ', 'T', 'R', 'S']
                             . shows_nl . x)))
                 (\ _ ->
                   catcha
                     (catcha (checkAllMI checkNSS nSp) (\ x -> errora (snd x)))
                     (\ x ->
                       errora
                         (shows_string
                            ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'h',
                              'e', 'n', ' ', 'o', 'r', 'i', 'e', 'n', 't', 'i',
                              'n', 'g', ' ', 'D', 'P', 's']
                           . shows_nl . x))))
           (\ _ ->
             catcha (checkSubset nSp rP)
               (\ x ->
                 errora
                   (case checkSS x of {
                      Lefta err -> err;
                      Righta _ -> internal_error;
                    }))))
         (\ x ->
           errora
             (shows_string
                ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'a', 'p',
                  'p', 'l', 'y', ' ', 't', 'h', 'e', ' ', 'r', 'e', 'd', 'u',
                  'c', 't', 'i', 'o', 'n', ' ', 'p', 'a', 'i', 'r', ' ', 'p',
                  'r', 'o', 'c', 'e', 's', 's', 'o', 'r', ' ', 'w', 'i', 't',
                  'h', ' ', 't', 'h', 'e', ' ', 'f', 'o', 'l', 'l', 'o', 'w',
                  'i', 'n', 'g']
               . shows_nl . representation . shows_nl . x));

shows_lpo_repr :: forall a. (Shows a) => [((a, Nat), Nat)] -> [Char] -> [Char];
shows_lpo_repr prs =
  shows_string
    ['L', 'P', 'O', ' ', 'w', 'i', 't', 'h', ' ', 't', 'h', 'e', ' ', 'f', 'o',
      'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'p', 'r', 'e', 'c', 'e', 'd', 'e',
      'n', 'c', 'e']
    . shows_nl .
        shows_map
          (\ (a, b) ->
            let {
              (f, n) = a;
            } in (\ pr ->
                   shows_string
                     ['p', 'r', 'e', 'c', 'e', 'd', 'e', 'n', 'c', 'e', '(']
                     . showsb f .
                         shows_string ['['] .
                           shows_nat n .
                             shows_string [']', ')', ' ', '=', ' '] .
                               shows_nat pr . shows_nl)
              b)
          prs
          . shows_nl .
              shows_string
                ['p', 'r', 'e', 'c', 'e', 'd', 'e', 'n', 'c', 'e', '(', '_',
                  ')', ' ', '=', ' ', '0']
                . shows_nl;

shows_lpo_af ::
  forall a.
    (Shows a) => [((a, Nat), Nat)] ->
                   [((a, Nat), Af_entry)] -> [Char] -> [Char];
shows_lpo_af repr af =
  shows_lpo_repr repr .
    shows_nl .
      shows_string
        ['i', 'n', ' ', 'c', 'o', 'm', 'b', 'i', 'n', 'a', 't', 'i', 'o', 'n',
          ' ', 'w', 'i', 't', 'h', ' ', 't', 'h', 'e', ' ', 'f', 'o', 'l', 'l',
          'o', 'w', 'i', 'n', 'g', ' ', 'a', 'r', 'g', 'u', 'm', 'e', 'n', 't',
          ' ', 'f', 'i', 'l', 't', 'e', 'r']
        . shows_nl . shows_afs af;

check_supt ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => (Term a b, Term a b) -> Eithera ([Char] -> [Char]) ();
check_supt lr =
  catcha
    (checkB (suptp (fst lr) (snd lr))
      (shows_string
         ['t', 'h', 'e', ' ', 'r', 'h', 's', ' ', 'i', 's', ' ', 'n', 'o', 't',
           ' ', 'a', ' ', 'p', 'r', 'o', 'p', 'e', 'r', ' ', 's', 'u', 'b', 't',
           'e', 'r', 'm', ' ', 'o', 'f', ' ', 't', 'h', 'e', ' ', 'l', 'h', 's']
        . shows_nl))
    (\ x ->
      errora
        (shows_string
           ['C', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'o', 'r', 'i',
             'e', 'n', 't', ' ', 'r', 'u', 'l', 'e', ' ']
          . shows_rule lr . shows_nl . x));

extract_goal_input :: [Char] -> Maybe [Char];
extract_goal_input s =
  debug ['0']
    ['e', 'x', 't', 'r', 'a', 'c', 't', 'i', 'n', 'g', ' ', 'g', 'o', 'a', 'l',
      ' ', 'a', 'n', 'd', ' ', 'T', 'R', 'S', ' ', 'f', 'r', 'o', 'm', ' ', 'i',
      'n', 'p', 'u', 't', ' ', 's', 't', 'r', 'i', 'n', 'g']
    (case search_len s ['<', 'g', 'o', 'a', 'l', '>']
            (size_list ['<', 'g', 'o', 'a', 'l', '>'])
       of {
       Nothing -> Nothing;
       Just (_, mgoal) ->
         case search_len mgoal ['<', '/', 'g', 'o', 'a', 'l', '>']
                (size_list ['<', '/', 'g', 'o', 'a', 'l', '>'])
           of {
           Nothing -> Nothing;
           Just (goal, s') ->
             case search_len s' ['<', 'i', 'n', 'p', 'u', 't', '>']
                    (size_list ['<', 'i', 'n', 'p', 'u', 't', '>'])
               of {
               Nothing -> Nothing;
               Just (_, minput) ->
                 case search_len minput ['<', '/', 'i', 'n', 'p', 'u', 't', '>']
                        (size_list ['<', '/', 'i', 'n', 'p', 'u', 't', '>'])
                   of {
                   Nothing -> Nothing;
                   Just (input, _) ->
                     Just (concata
                            [['<', 'p', 'r', 'o', 'b', 'l', 'e', 'm', '>'],
                              ['<', 'g', 'o', 'a', 'l', '>'], goal,
                              ['<', '/', 'g', 'o', 'a', 'l', '>'],
                              ['<', 'i', 'n', 'p', 'u', 't', '>'], input,
                              ['<', '/', 'i', 'n', 'p', 'u', 't', '>'],
                              ['<', '/', 'p', 'r', 'o', 'b', 'l', 'e', 'm',
                                '>']]);
                 };
             };
         };
     });

check_not_wf_trs ::
  forall a b.
    (Shows a, Eq b,
      Shows b) => [(Term a b, Term a b)] -> Eithera ([Char] -> [Char]) ();
check_not_wf_trs r =
  checkB (not (isValue (check_wf_trs r)))
    (shows_string
       ['T', 'h', 'e', ' ', 'T', 'R', 'S', ' ', 'i', 's', ' ', 'w', 'e', 'l',
         'l', ' ', 'f', 'o', 'r', 'm', 'e', 'd']
      . shows_nl);

checkTRSDisproof ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => ([Char] -> [Char]) ->
                    [(Term a b, Term a b)] ->
                      TRSDisproof a b -> Eithera ([Char] -> [Char]) ();
checkTRSDisproof i r (Loop l) =
  debug (i []) ['L', 'o', 'o', 'p']
    (catcha (checkLoop r l)
      (\ x ->
        errora
          (i . shows_string
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'i', 'n', ' ', 'l',
                   'o', 'o', 'p', ' ', 'p', 'r', 'o', 'o', 'f']
                 . shows_nl . x)));
checkTRSDisproof i r NotWellFormed =
  debug (i [])
    ['n', 'o', 't', ' ', 'w', 'e', 'l', 'l', '-', 'f', 'o', 'r', 'm', 'e', 'd']
    (catcha (check_not_wf_trs r)
      (\ x ->
        errora
          (i . shows_string
                 [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'i', 'n', ' ', 'n',
                   'o', 't', '-', 'w', 'e', 'l', 'l', '-', 'f', 'o', 'r', 'm',
                   'e', 'd', ' ', 'p', 'r', 'o', 'o', 'f']
                 . shows_nl . x)));

init_lab :: forall a b. a -> Lab a b;
init_lab f = UnLab f;

unary_term :: forall a b. Term a b -> Bool;
unary_term (Var uu) = True;
unary_term (Fun uv [t]) = unary_term t;
unary_term (Fun v []) = False;
unary_term (Fun v (vb : vd : ve)) = False;

check_unary_signature ::
  forall a b. [(Term a b, Term a b)] -> Eithera ([Char] -> [Char]) ();
check_unary_signature r =
  catcha
    (catcha
      (checkAllMI (checkF (\ (l, ra) -> unary_term l && unary_term ra) id) r)
      (\ x -> errora (snd x)))
    (\ _ ->
      errora
        (shows_string
          ['t', 'h', 'e', ' ', 't', 'r', 's', ' ', 'd', 'o', 'e', 's', ' ', 'n',
            'o', 't', ' ', 'h', 'a', 'v', 'e', ' ', 'a', ' ', 'u', 'n', 'a',
            'r', 'y', ' ', 's', 'i', 'g', 'n', 'a', 't', 'u', 'r', 'e']));

check_string_rev ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => [(Term a b, Term a b)] ->
                    [(Term a b, Term a b)] -> Eithera ([Char] -> [Char]) ();
check_string_rev r rR =
  bind (check_unary_signature r)
    (\ _ ->
      catcha (checkSubset (mapa rev_rule r) rR)
        (\ x ->
          errora
            (shows_string ['t', 'h', 'e', ' ', 'r', 'u', 'l', 'e', ' '] .
              shows_rule x .
                shows_string
                  [' ', 'i', 's', ' ', 'm', 'i', 's', 's', 'i', 'n', 'g']
                  . shows_nl)));

shows_arithFun :: ArithFun -> [Char] -> [Char];
shows_arithFun (Arg i) = shows_string ['x', '_'] . shows_nat (Suc i);
shows_arithFun (Const n) = shows_nat n;
shows_arithFun (Sum fs) =
  shows_list_gen shows_arithFun [] [] [' ', '+', ' '] [] fs;
shows_arithFun (Prod fs) =
  shows_list_gen shows_arithFun [] [] [' ', '*', ' '] [] fs;
shows_arithFun (Min fs) =
  shows_list_gen shows_arithFun [] ['m', 'i', 'n', '('] [','] [')'] fs;
shows_arithFun (Max fs) =
  shows_list_gen shows_arithFun [] ['m', 'a', 'x', '('] [','] [')'] fs;

shows_sl_inter ::
  forall a b. (Shows a, Shows b) => Sl_inter a b -> [Char] -> [Char];
shows_sl_inter (SL_Inter n fnas) =
  shows_string
    ['c', 'a', 'r', 'r', 'i', 'e', 'r', ' ', '{', '0', ',', '.', '.', '.', ',']
    . shows_nat n .
        shows_string ['}'] .
          shows_nl .
            shows_map
              (\ (a, b) ->
                let {
                  (f, na) = a;
                } in (\ aa ->
                       shows_string ['['] .
                         shows_lab f .
                           shows_string ['/'] .
                             shows_nat na .
                               shows_string [']', ' ', '=', ' '] .
                                 shows_arithFun aa . shows_nl)
                  b)
              fnas;

take_default :: forall a. a -> [a] -> Nat -> a;
take_default def [] uu = def;
take_default uv (x : xs) Zero_nat = x;
take_default def (uw : xs) (Suc i) = take_default def xs i;

eval_arithFun_unbound :: [Nat] -> ArithFun -> Nat;
eval_arithFun_unbound nats (Arg i) = take_default Zero_nat nats i;
eval_arithFun_unbound nats (Const n) = n;
eval_arithFun_unbound nats (Sum []) = Zero_nat;
eval_arithFun_unbound nats (Sum (f : fs)) =
  plus_nat (eval_arithFun_unbound nats f) (eval_arithFun_unbound nats (Sum fs));
eval_arithFun_unbound nats (Prod []) = one_nat;
eval_arithFun_unbound nats (Prod (f : fs)) =
  times_nat (eval_arithFun_unbound nats f)
    (eval_arithFun_unbound nats (Prod fs));
eval_arithFun_unbound nats (Max [f]) = eval_arithFun_unbound nats f;
eval_arithFun_unbound nats (Max (f : v : va)) =
  maxa (eval_arithFun_unbound nats f)
    (eval_arithFun_unbound nats (Max (v : va)));
eval_arithFun_unbound nats (Min [f]) = eval_arithFun_unbound nats f;
eval_arithFun_unbound nats (Min (f : v : va)) =
  mina (eval_arithFun_unbound nats f)
    (eval_arithFun_unbound nats (Min (v : va)));

eval_arithFun :: Nat -> ArithFun -> [Nat] -> Nat;
eval_arithFun c f nats = mod_nat (eval_arithFun_unbound nats f) c;

sl_inter_to_inter ::
  forall a b. (Eq a, Eq b) => Sl_inter a b -> Lab a b -> [Nat] -> Nat;
sl_inter_to_inter (SL_Inter c ls) fl cs =
  case lookupa (fl, size_list cs) ls of {
    Nothing -> Zero_nat;
    Just f -> eval_arithFun (Suc c) f cs;
  };

get_largest_element :: forall a b. Sl_inter a b -> Nat;
get_largest_element (SL_Inter n uu) = n;

create_sem_lab_tt ::
  forall a b c.
    (Eq a, Shows a, Eq b, Shows b, Eq c,
      Shows c) => Sl_inter a b ->
                    (Lab a b -> [Nat] -> Eithera b [(Lab a b)]) ->
                      [(Term (Lab a b) c, Term (Lab a b) c)] ->
                        [(Term (Lab a b) c, Term (Lab a b) c)] ->
                          Eithera ([Char] -> [Char]) ();
create_sem_lab_tt sli l r lr =
  let {
    c = get_largest_element sli;
    ca = upt Zero_nat (Suc c);
    i = sl_inter_to_inter sli;
  } in catcha
         (check_sem_lab_tt (check_sl_lab_trs i l ca)
           (check_sl_model_lab_trs i l ca) (\ _ -> False) i l r lr)
         (\ x -> errora (shows_sl_inter sli . shows_nl . x));

unlab_of_trsI ::
  forall a b c.
    (Eq a, Eq b,
      Eq c) => [(Term (Lab a b) c, Term (Lab a b) c)] ->
                 [(Term (Lab a b) c, Term (Lab a b) c)];
unlab_of_trsI r =
  remdups
    (map_trsI unlab
      (filtera
        (\ (l, ra) ->
          eq_term l ra || not (eq_term (map_term unlab l) (map_term unlab ra)))
        r));

unlab_trs_tt ::
  forall a b c.
    (Eq a, Shows a, Eq b, Shows b, Eq c,
      Shows c) => [(Term (Lab a b) c, Term (Lab a b) c)] ->
                    [(Term (Lab a b) c, Term (Lab a b) c)] ->
                      Eithera ([Char] -> [Char]) ();
unlab_trs_tt r rR =
  let {
    u = unlab_of_trsI r;
  } in catcha (bind (checkSubset rR u) (\ _ -> checkSubset u rR))
         (\ _ ->
           errora
             (shows_rules rR .
               shows_nl .
                 shows_string
                   [' ', ' ', 'd', 'o', 'e', 's', ' ', 'n', 'o', 't', ' ', 'c',
                     'o', 'r', 'r', 'e', 's', 'p', 'o', 'n', 'd', ' ', 't', 'o',
                     ' ', 'u', 'n', 'l', 'a', 'b', 'e', 'l', 'e', 'd', ' ', 'T',
                     'R', 'S', ' ', ' ']
                   . shows_nl . shows_rules u . shows_nl));

check_mono_afs_help ::
  forall a b c. (Shows a) => ((a, b), c) -> [Char] -> [Char];
check_mono_afs_help ((f, a), e) =
  shows_string ['e', 'n', 't', 'r', 'y', ' ', 'f', 'o', 'r', ' '] .
    showsb f .
      shows_string
        [' ', 'n', 'o', 't', ' ', 'm', 'o', 'n', 'o', 't', 'o', 'n', 'i', 'c']
        . shows_nl;

mono_af_entry :: Nat -> Af_entry -> Bool;
mono_af_entry n (Collapse i) = less_eq_nat n one_nat;
mono_af_entry n (AFList ids) = foralla (\ i -> member i ids) (upt Zero_nat n);

check_mono_afs ::
  forall a.
    (Shows a) => [((a, Nat), Af_entry)] -> Eithera ([Char] -> [Char]) ();
check_mono_afs pia =
  catcha
    (catcha
      (checkAllMI (checkF (\ x -> mono_af_entry (snd (fst x)) (snd x)) id) pia)
      (\ x -> errora (snd x)))
    (\ x -> errora (check_mono_afs_help x));

checkRuleRemovalProc ::
  forall a b.
    (Eq a,
      Eq b) => Eithera ([Char] -> [Char]) () ->
                 ((Term a b, Term a b) -> Eithera ([Char] -> [Char]) ()) ->
                   ((Term a b, Term a b) -> Eithera ([Char] -> [Char]) ()) ->
                     ([Char] -> [Char]) ->
                       ([Char] -> [Char]) ->
                         [(Term a b, Term a b)] ->
                           [(Term a b, Term a b)] ->
                             Eithera ([Char] -> [Char]) ();
checkRuleRemovalProc checkMono checkSS checkNSS name representation r rR =
  let {
    (_, nSp) = partition (\ foo -> isValue (checkSS foo)) r;
  } in catcha
         (bind (bind (bind (catcha checkMono
                             (\ x ->
                               errora
                                 (shows_string
                                    ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ',
                                      'w', 'i', 't', 'h', ' ', 'm', 'o', 'n',
                                      'o', 't', 'o', 'n', 'i', 'c', 'i', 't',
                                      'y', ' ', 'o', 'f', ' ', 's', 't', 'r',
                                      'i', 'c', 't', ' ', 'o', 'r', 'd', 'e',
                                      'r']
                                   . shows_nl . x)))
                       (\ _ ->
                         catcha
                           (catcha (checkAllMI checkNSS r)
                             (\ x -> errora (snd x)))
                           (\ x ->
                             errora
                               (shows_string
                                  ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w',
                                    'h', 'e', 'n', ' ', 'o', 'r', 'i', 'e', 'n',
                                    't', 'i', 'n', 'g', ' ', 'r', 'u', 'l', 'e',
                                    's']
                                 . shows_nl . x))))
                 (\ _ ->
                   catcha (checkSubset nSp rR)
                     (\ x ->
                       errora
                         (case checkSS x of {
                            Lefta err -> err;
                            Righta _ -> internal_error;
                          }))))
           (\ _ ->
             catcha (checkSubset rR r)
               (\ x ->
                 errora
                   (case checkSS x of {
                      Lefta err -> err;
                      Righta _ -> internal_error;
                    }))))
         (\ x ->
           errora
             (shows_string
                ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'a', 'p',
                  'p', 'l', 'y', ' ', 't', 'h', 'e', ' ', 'r', 'u', 'l', 'e',
                  ' ', 'r', 'e', 'm', 'o', 'v', 'a', 'l', ' ', 'p', 'r', 'o',
                  'c', 'e', 's', 's', 'o', 'r', ' ', 'w', 'i', 't', 'h', ' ',
                  't', 'h', 'e', ' ', 'f', 'o', 'l', 'l', 'o', 'w', 'i', 'n',
                  'g']
               . shows_nl . representation . shows_nl . x));

check_wf_afs ::
  forall a.
    (Shows a) => [((a, Nat), Af_entry)] -> Eithera ([Char] -> [Char]) ();
check_wf_afs pia =
  catcha
    (checkAllMI
      (\ (a, b) ->
        let {
          (f, n) = a;
        } in (\ e ->
               catcha
                 (case e of {
                    Collapse i -> checkB (less_nat i n) i;
                    AFList ids ->
                      catcha (checkAllMI (checkF (\ i -> less_nat i n) id) ids)
                        (\ x -> errora (snd x));
                  })
                 (\ x ->
                   errora
                     (shows_string ['i', 'n', 'd', 'e', 'x', ' '] .
                       shows_nat x .
                         shows_string
                           [' ', 't', 'o', 'o', ' ', 'h', 'i', 'g', 'h', ' ',
                             'f', 'o', 'r', ' ', 'A', 'F', '-', 'e', 'n', 't',
                             'r', 'y', ' ', 'o', 'f']
                           . showsa (f, n) . shows_nl)))
          b)
      pia)
    (\ x -> errora (snd x));

default_af_entry :: Nat -> Af_entry;
default_af_entry n = AFList (upt Zero_nat n);

afs_list_to_afs ::
  forall a. (Eq a) => [((a, Nat), Af_entry)] -> a -> Nat -> Af_entry;
afs_list_to_afs pia f n =
  case lookupa (f, n) pia of {
    Nothing -> default_af_entry n;
    Just entry -> entry;
  };

lpo_repr_to_pr ::
  forall a. (Eq a) => [((a, Nat), Nat)] -> Filtered a -> Nat -> Nat;
lpo_repr_to_pr prs (FPair f a) ignored =
  case lookupa (f, a) prs of {
    Nothing -> Zero_nat;
    Just x -> x;
  };

get_max_arity :: forall a b. (Eq a) => [(Term a b, Term a b)] -> Nat;
get_max_arity r =
  max_list
    (mapa snd
      (concat_map (\ (l, ra) -> append (funas_term_list l) (funas_term_list ra))
        r));

nat_components ::
  (Nat -> Bool,
    (Nat, (Nat, (Nat -> Nat -> Nat,
                  (Nat -> Nat -> Nat,
                    (Nat -> Nat,
                      (Nat -> Nat -> Bool,
                        (Nat -> Nat -> Bool,
                          (Bool, ([Char], Nat -> Bool))))))))));
nat_components =
  ((\ _ -> True),
    (Zero_nat,
      (one_nat,
        (plus_nat,
          (times_nat,
            (max0_nat,
              (ge_nat,
                ((\ x y -> less_nat y x),
                  (True, (nat_name, (\ _ -> True)))))))))));

create_lpoly_repr ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => (a -> Bool,
                    (a, (a, (a -> a -> a,
                              (a -> a -> a,
                                (a -> a,
                                  (a -> a -> Bool,
                                    (a -> a -> Bool,
                                      (Bool, ([Char], a -> Bool))))))))))
                    -> [((b, Nat), (a, [a]))] -> [Char] -> [Char];
create_lpoly_repr c i =
  let {
    pia = to_lpoly_inter (get_comp_1 c) i;
  } in shows_string
         ['p', 'o', 'l', 'y', 'n', 'o', 'm', 'i', 'a', 'l', ' ', 'i', 'n', 't',
           'e', 'r', 'p', 'r', 'e', 't', 'r', 'a', 't', 'i', 'o', 'n', ' ', 'o',
           'v', 'e', 'r', ' ']
         . shows_string (get_comp_description c) .
             shows_nl .
               shows_sep
                 (\ (f, n) ->
                   let {
                     t = Fun f (mapa Var
                                 (fresh_strings_list ['x', '_'] one_nat [] n));
                   } in shows_string ['P', 'o', 'l', '('] .
                          shows_term t .
                            shows_string [')', ' ', '=', ' '] .
                              shows_lpoly (get_comp_0 c) (get_comp_1 c)
                                (pleftI (get_comp_0 c) (get_comp_1 c)
                                  (get_comp_plus c) (get_comp_times c)
                                  (get_comp_max0 c) pia t))
                 shows_nl (remdups (mapa fst i));

check_poly_monoI ::
  forall a b.
    (Shows a,
      Shows b) => a -> (a -> a -> Bool) ->
                         (a -> Bool) ->
                           [((b, Nat), (a, [a]))] ->
                             Eithera ([Char] -> [Char]) ();
check_poly_monoI ze ge cm =
  (\ xs ->
    catcha
      (checkAllMI
        (\ (a, b) ->
          let {
            (f, n) = a;
          } in (\ (c, cs) ->
                 catcha
                   (bind (bind (checkB (ge c ze)
                                 (shows_string
                                    ['c', 'o', 'n', 's', 't', 'a', 'n', 't',
                                      ' ', 'p', 'a', 'r', 't', ' ']
                                   . showsb c .
                                       shows_string
 [' ', 'm', 'u', 's', 't', ' ', 'b', 'e', ' ', 'a', 't', ' ', 'l', 'e', 'a',
   's', 't', ' ']
 . showsb ze . shows_nl))
                           (\ _ ->
                             checkB (less_eq_nat n (size_list cs))
                               (shows_string
                                  ['t', 'h', 'e', ' ', 'l', 'a', 's', 't', ' ',
                                    'a', 'r', 'g', 'u', 'm', 'e', 'n', 't', ' ',
                                    'i', 's', ' ', 'i', 'g', 'n', 'o', 'r', 'e',
                                    'd']
                                 . shows_nl)))
                     (\ _ ->
                       catcha
                         (checkAllMI
                           (\ d ->
                             checkB (cm d)
                               (shows_string
                                  ['c', 'o', 'e', 'f', 'f', 'i', 'c', 'i', 'e',
                                    'n', 't', ' ']
                                 . showsb d .
                                     shows_string
                                       [' ', 'i', 's', ' ', 'n', 'o', 't', ' ',
 'a', 'l', 'l', 'o', 'w', 'e', 'd']
                                       . shows_nl))
                           cs)
                         (\ x -> errora (snd x))))
                   (\ x ->
                     errora
                       (shows_string
                          ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'i',
                            't', 'h', ' ', 'm', 'o', 'n', 'o', 't', 'o', 'n',
                            'i', 'c', 'i', 't', 'y', ' ', 'd', 'u', 'e', ' ',
                            't', 'o', ' ', 'i', 'n', 't', 'e', 'r', 'p', 'r',
                            'e', 't', 'a', 't', 'i', 'o', 'n', ' ', 'o', 'f',
                            ' ']
                         . showsb f .
                             shows_string ['/'] . shows_nat n . shows_nl . x)))
            b)
        xs)
      (\ x -> errora (snd x)));

get_comp_wf_elem ::
  forall a.
    (a -> Bool,
      (a, (a, (a -> a -> a,
                (a -> a -> a,
                  (a -> a,
                    (a -> a -> Bool,
                      (a -> a -> Bool, (Bool, ([Char], a -> Bool))))))))))
      -> a -> Bool;
get_comp_wf_elem (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap))))))))))
  = wfe;

check_lpoly_coeffs ::
  forall a b.
    (Shows a,
      Shows b) => (a -> Bool) ->
                    (a -> Bool) ->
                      (a -> Bool) ->
                        [((b, Nat), (a, [a]))] -> Eithera ([Char] -> [Char]) ();
check_lpoly_coeffs wfe apos ch i =
  catcha
    (checkAllMI
      (\ (a, b) ->
        let {
          (f, n) = a;
        } in (\ (c, cs) ->
               catcha
                 (bind (bind (checkB (wfe c)
                               (shows_string
                                  ['c', 'o', 'n', 's', 't', 'a', 'n', 't', ' ',
                                    'p', 'a', 'r', 't', ' ']
                                 . showsb c .
                                     shows_string
                                       [' ', 'i', 's', ' ', 'n', 'o', 't', ' ',
 'w', 'e', 'l', 'l', '-', 'f', 'o', 'r', 'm', 'e', 'd']
                                       . shows_nl))
                         (\ _ ->
                           checkB (apos c || anya apos (takea n cs))
                             (shows_string
                                ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't',
                                  ' ', 'f', 'i', 'n', 'd', ' ', 'p', 'o', 's',
                                  'i', 't', 'i', 'v', 'e', ' ', 'e', 'n', 't',
                                  'r', 'y', ' ', 'w', 'h', 'i', 'c', 'h', ' ',
                                  'i', 's', ' ', 'r', 'e', 'q', 'u', 'i', 'r',
                                  'e', 'd', ' ', 'f', 'o', 'r', ' ', 'a', 'r',
                                  'c', 't', 'i', 'c', ' ', 'i', 'n', 't', 'e',
                                  'r', 'p', 'r', 'e', 't', 'a', 't', 'i', 'o',
                                  'n', 's']
                               . shows_nl)))
                   (\ _ ->
                     catcha
                       (checkAllMI
                         (\ aa ->
                           checkB (ch aa && wfe aa)
                             (shows_string
                                ['c', 'o', 'e', 'f', 'f', 'i', 'c', 'i', 'e',
                                  'n', 't', ' ']
                               . showsb aa .
                                   shows_string
                                     [' ', 'i', 's', ' ', 'n', 'o', 't', ' ',
                                       'a', 'l', 'l', 'o', 'w', 'e', 'd']
                                     . shows_nl))
                         cs)
                       (\ x -> errora (snd x))))
                 (\ x ->
                   errora
                     (shows_string
                        ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'i', 't',
                          'h', ' ', 'i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't',
                          'a', 't', 'i', 'o', 'n', ' ', 'o', 'f', ' ']
                       . showsb f .
                           shows_string ['/'] . shows_nat n . shows_nl . x)))
          b)
      i)
    (\ x -> errora (snd x));

create_rule_removal_proc_comp ::
  forall a b c.
    (Eq a, Shows a, Eq b, Shows b, Eq c,
      Shows c) => (a -> Bool,
                    (a, (a, (a -> a -> a,
                              (a -> a -> a,
                                (a -> a,
                                  (a -> a -> Bool,
                                    (a -> a -> Bool,
                                      (Bool, ([Char], a -> Bool))))))))))
                    -> (a -> Bool) ->
                         [((b, Nat), (a, [a]))] ->
                           [(Term b c, Term b c)] ->
                             [(Term b c, Term b c)] ->
                               Eithera ([Char] -> [Char]) ();
create_rule_removal_proc_comp c cm i r rR =
  let {
    pia = to_lpoly_inter (get_comp_1 c) i;
  } in bind (check_lpoly_coeffs (get_comp_wf_elem c) (get_comp_apos c)
              (\ x -> get_comp_ge c x (get_comp_0 c)) i)
         (\ _ ->
           checkRuleRemovalProc
             (check_poly_monoI (get_comp_0 c) (get_comp_ge c) cm i)
             (check_polo_sI (get_comp_mono c) (get_comp_0 c) (get_comp_1 c)
               (get_comp_plus c) (get_comp_times c) (get_comp_max0 c)
               (get_comp_ge c) (get_comp_gt c) pia)
             (check_polo_nsI (get_comp_0 c) (get_comp_1 c) (get_comp_plus c)
               (get_comp_times c) (get_comp_max0 c) (get_comp_ge c) pia)
             (shows_string
                ['p', 'o', 'l', 'y', 'n', 'o', 'm', 'i', 'a', 'l', ' ', 'o',
                  'r', 'd', 'e', 'r', ' ', '(']
               . shows_string (get_comp_description c) . shows_string [')'])
             (create_lpoly_repr c i) r rR);

create_rule_removal_proc ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => Redpair a ->
                    [(Term a b, Term a b)] ->
                      [(Term a b, Term a b)] -> Eithera ([Char] -> [Char]) ();
create_rule_removal_proc (Int_carrier i) =
  create_rule_removal_proc_comp int_components int_mono i;
create_rule_removal_proc (Nat_carrier i) =
  create_rule_removal_proc_comp nat_components nat_mono i;
create_rule_removal_proc (Rat_carrier delta i) =
  create_rule_removal_proc_comp (rat_components delta) rat_mono i;
create_rule_removal_proc (Arctic_carrier i) =
  (\ _ _ ->
    errora
      (shows_string
         ['a', 'r', 'c', 't', 'i', 'c', ' ', 'n', 'o', 't', ' ', 'm', 'o', 'n',
           'o', 't', 'o', 'n', 'i', 'c', ' ', 'f', 'o', 'r', ' ', 'T', 'R', 'S',
           's']
        . shows_nl));
create_rule_removal_proc (Arctic_rat_carrier delta i) =
  (\ _ _ ->
    errora
      (shows_string
         ['a', 'r', 'c', 't', 'i', 'c', ' ', 'n', 'o', 't', ' ', 'm', 'o', 'n',
           'o', 't', 'o', 'n', 'i', 'c', ' ', 'f', 'o', 'r', ' ', 'T', 'R', 'S',
           's']
        . shows_nl));
create_rule_removal_proc (Arc_mat_carrier n i) =
  (\ _ _ ->
    errora
      (shows_string
         ['a', 'r', 'c', 't', 'i', 'c', ' ', 'n', 'o', 't', ' ', 'm', 'o', 'n',
           'o', 't', 'o', 'n', 'i', 'c', ' ', 'f', 'o', 'r', ' ', 'T', 'R', 'S',
           's']
        . shows_nl));
create_rule_removal_proc (Arc_rat_mat_carrier delta n i) =
  (\ _ _ ->
    errora
      (shows_string
         ['a', 'r', 'c', 't', 'i', 'c', ' ', 'n', 'o', 't', ' ', 'm', 'o', 'n',
           'o', 't', 'o', 'n', 'i', 'c', ' ', 'f', 'o', 'r', ' ', 'T', 'R', 'S',
           's']
        . shows_nl));
create_rule_removal_proc (Int_mat_carrier n sd i) =
  create_rule_removal_proc_comp (mat_components n sd int_components)
    (mat_mono sd int_mono) i;
create_rule_removal_proc (Rat_mat_carrier delta n sd i) =
  create_rule_removal_proc_comp (mat_components n sd (rat_components delta))
    (mat_mono sd rat_mono) i;
create_rule_removal_proc (Lpo pri pia) =
  (\ r rR ->
    let {
      m = get_max_arity r;
      pr = lpo_repr_to_pr pri;
      piaa = afs_list_to_afs pia;
    } in bind (check_wf_afs pia)
           (\ _ ->
             checkRuleRemovalProc (check_mono_afs pia)
               (afs_check lpoS piaa (lpo_strict pr m))
               (afs_check lpoNS piaa (lpo_nstrict pr m)) lpo_repr
               (shows_lpo_af pri pia) r rR));

computeRuleMap ::
  forall a b.
    (Eq a) => [(Term a b, Term a b)] -> Maybe (a -> [(Term a b, Term a b)]);
computeRuleMap [] = Just (\ _ -> []);
computeRuleMap ((Fun f ts, r) : rules) =
  case computeRuleMap rules of {
    Nothing -> Nothing;
    Just rm -> Just (fun_upd rm f ((Fun f ts, r) : rm f));
  };
computeRuleMap ((Var uu, uv) : rules) = Nothing;

afs_list_to_af ::
  forall a. (Eq a) => [((a, Nat), Af_entry)] -> a -> Nat -> Nat -> Bool;
afs_list_to_af pia f n =
  (\ i ->
    case lookupa (f, n) pia of {
      Nothing -> True;
      Just (Collapse a) -> eq_nat i a;
      Just (AFList a) -> member i a;
    });

matchCapRMBelow ::
  forall a b.
    (Eq a,
      Eq b) => Term a b ->
                 Maybe (a -> [(Term a b, Term a b)]) -> Term a b -> Bool;
matchCapRMBelow l rm (Fun f ts) = gcMatchI (HFun f (mapa (capRM rm) ts)) l;

check_ur_closed_term_rm_af ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => (a -> [(Term a b, Term a b)]) ->
                    [(Term a b, Term a b)] ->
                      (a -> Nat -> Nat -> Bool) ->
                        Term a b -> Eithera ([Char] -> [Char]) ();
check_ur_closed_term_rm_af uu uv uw (Var x) = returna ();
check_ur_closed_term_rm_af rm ur pia (Fun f ts) =
  let {
    n = size_list ts;
  } in bind (checkAllLI
              (\ t i ->
                (if pia f n i then check_ur_closed_term_rm_af rm ur pia t
                  else returna ()))
              ts)
         (\ _ ->
           catcha
             (checkAllMI
               (\ lr ->
                 checkB
                   (member lr ur ||
                     not (matchCapRMBelow (fst lr) (Just rm) (Fun f ts)))
                   (shows_string
                      ['d', 'u', 'e', ' ', 't', 'o', ' ', 't', 'h', 'e', ' ',
                        's', 'u', 'b', 't', 'e', 'r', 'm', ' ']
                     . shows_term (Fun f ts) .
                         shows_string
                           [' ', 'o', 'f', ' ', 's', 'o', 'm', 'e', ' ', 'u',
                             's', 'a', 'b', 'l', 'e', ' ', 'r', 'h', 's', ',',
                             ' ', 'r', 'u', 'l', 'e', ' ']
                           . shows_rule lr .
                               shows_string
                                 [' ', 's', 'h', 'o', 'u', 'l', 'd', ' ', 'b',
                                   'e', ' ', 'u', 's', 'a', 'b', 'l', 'e',
                                   '.']))
               (rm f))
             (\ x -> errora (snd x)));

check_sct_ur_af_proc ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => ((Term a b, Term a b) -> Eithera ([Char] -> [Char]) ()) ->
                    ((Term a b, Term a b) -> Eithera ([Char] -> [Char]) ()) ->
                      ([Char] -> [Char]) ->
                        [(Term a b, Term a b)] ->
                          [((Term a b, Term a b), ([(Nat, Nat)], [(Nat, Nat)]))]
                            -> Maybe (a -> [(Term a b, Term a b)]) ->
                                 Maybe (a -> [(Term a b, Term a b)]) ->
                                   [(Term a b, Term a b)] ->
                                     (a -> Nat -> Nat -> Bool) ->
                                       Eithera ([Char] -> [Char]) ();
check_sct_ur_af_proc checkSS checkNSS representation p gs rm rrm ur pia =
  case rm of {
    Nothing ->
      errora
        (shows_string
           ['T', 'R', 'S', ' ', 'h', 'a', 's', ' ', 'v', 'a', 'r', 'i', 'a',
             'b', 'l', 'e', 's', ' ', 'a', 's', ' ', 'l', 'e', 'f', 't', '-',
             'h', 'a', 'n', 'd', ' ', 's', 'i', 'd', 'e', 's']
          . shows_nl);
    Just rma ->
      catcha
        (let {
           ggs = filtera (\ g -> member (fst g) p) gs;
         } in bind (bind (bind (bind (catcha
                                       (catcha
 (checkAllMI (\ lr -> check_ur_closed_term_rm_af rma ur pia (snd lr)) ur)
 (\ x -> errora (snd x)))
                                       (\ x ->
 errora
   (shows_string
      ['e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ', 'c', 'h', 'e',
        'c', 'k', 'i', 'n', 'g', ' ', 'c', 'l', 'o', 's', 'u', 'r', 'e', ' ',
        'p', 'r', 'o', 'p', 'e', 'r', 't', 'i', 'e', 's', ' ', 'o', 'f', ' ',
        'r', 'h', 's', ' ', 'o', 'f', ' ', 'u', 's', 'a', 'b', 'l', 'e', ' ',
        'r', 'u', 'l', 'e', 's']
     . shows_nl . x)))
                                 (\ _ ->
                                   catcha (checkSubset p (mapa fst ggs))
                                     (\ x ->
                                       errora
 (shows_string
    ['t', 'h', 'e', 'r', 'e', ' ', 'i', 's', ' ', 'n', 'o', ' ', 's', 'i', 'z',
      'e', '-', 'c', 'h', 'a', 'n', 'g', 'e', ' ', 'g', 'r', 'a', 'p', 'h', ' ',
      'f', 'o', 'r', ' ', 'D', 'P', ' ']
   . shows_rule x))))
                           (\ _ ->
                             catcha
                               (checkAllMI
                                 (\ (a, b) ->
                                   let {
                                     (s, t) = a;
                                   } in (\ (stri, nstri) ->
  catcha
    (bind (bind (bind (checkB
                        (not (is_var s) &&
                          (not (is_var t) && nulla (rma (the (root t)))))
                        (shows_string
                           ['r', 'o', 'o', 't', 's', ' ', 'o', 'f', ' ', 'D',
                             'P', ' ', 'm', 'u', 's', 't', ' ', 'b', 'e', ' ',
                             'c', 'o', 'n', 's', 't', 'r', 'u', 'c', 't', 'o',
                             'r', 's']
                          . shows_nl))
                  (\ _ ->
                    catcha
                      (checkAllMI
                        (\ i ->
                          checkB (less_eq_nat i (size_list (args s)))
                            (shows_string
                               ['l', 'e', 'f', 't', '-', 'i', 'n', 'd', 'e',
                                 'x', ' ', 't', 'o', ' ', 'l', 'a', 'r', 'g',
                                 'e']
                              . shows_nat i . shows_nl))
                        (remdups (mapa fst (append stri nstri))))
                      (\ x -> errora (snd x))))
            (\ _ ->
              catcha
                (checkAllMI
                  (\ j ->
                    checkB
                      (less_eq_nat j (size_list (args t)) &&
                        isValue
                          (check_ur_closed_term_rm_af rma ur pia (get_arg t j)))
                      (shows_string
                         ['r', 'i', 'g', 'h', 't', '-', 'i', 'n', 'd', 'e', 'x',
                           ' ', 't', 'o', ' ', 'l', 'a', 'r', 'g', 'e', ' ',
                           'o', 'r', ' ', 'a', 'r', 'g', 'u', 'm', 'e', 'n',
                           't', ' ', 'v', 'i', 'o', 'l', 'a', 't', 'e', 's',
                           ' ', 'u', 's', 'a', 'b', 'l', 'e', '-', 'r', 'u',
                           'l', 'e', 's', ' ', 'c', 'o', 'n', 'd', 'i', 't',
                           'i', 'o', 'n']
                        . shows_nat j))
                  (remdups (mapa snd (append stri nstri))))
                (\ x -> errora (snd x))))
      (\ _ ->
        let {
          _ = args s;
          _ = args t;
        } in bind (catcha
                    (checkAllMI
                      (\ (i, j) ->
                        checkB (isValue (checkSS (get_arg s i, get_arg t j)))
                          (shows_string
                             ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'i',
                               't', 'h', ' ', 'e', 'd', 'g', 'e', ' ']
                            . shows_nat i .
                                shows_string [' ', '-', 'S', '-', '>', ' '] .
                                  shows_nat j))
                      stri)
                    (\ x -> errora (snd x)))
               (\ _ ->
                 catcha
                   (checkAllMI
                     (\ (i, j) ->
                       checkB (isValue (checkNSS (get_arg s i, get_arg t j)))
                         (shows_string
                            ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'i',
                              't', 'h', ' ', 'e', 'd', 'g', 'e', ' ']
                           . shows_nat i .
                               shows_string [' ', '-', 'N', 'S', '-', '>', ' ']
                                 . shows_nat j))
                     nstri)
                   (\ x -> errora (snd x)))))
    (\ x ->
      errora
        (shows_string
           ['p', 'r', 'o', 'b', 'l', 'e', 'm', 's', ' ', 'w', 'i', 't', 'h',
             ' ', 'D', 'P', ' ']
          . shows_rule (s, t) . shows_nl . x)))
                                     b)
                                 ggs)
                               (\ x -> errora (snd x))))
                     (\ _ ->
                       catcha
                         (catcha (checkAllMI checkNSS ur)
                           (\ x -> errora (snd x)))
                         (\ x ->
                           errora
                             (shows_string
                                ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w',
                                  'h', 'e', 'n', ' ', 'o', 'r', 'i', 'e', 'n',
                                  't', 'i', 'n', 'g', ' ', 'u', 's', 'a', 'b',
                                  'l', 'e', ' ', 'r', 'u', 'l', 'e', 's']
                               . shows_nl . x))))
                (\ _ ->
                  checkB
                    (check_SCT
                      (\ (_, (t, (_, ct))) (u, (_, (cu, _))) ->
                        gcMatchI ct u && gcMatchI cu t)
                      (mapa (\ (a, b) ->
                              let {
                                (s, t) = a;
                              } in (\ (stri, nstri) ->
                                     let {
                                       e = (s, (t, (capRM rrm s, capRM rm t)));
                                     } in Scg e e stri nstri)
                                b)
                        ggs))
                    (shows_string
                       ['s', 'i', 'z', 'e', '-', 'c', 'h', 'a', 'n', 'g', 'e',
                         ' ', 'a', 'n', 'a', 'l', 'y', 's', 'i', 's', ' ', 'f',
                         'a', 'i', 'l', 'e', 'd']
                      . shows_nl)))
        (\ x ->
          errora
            (shows_string
               ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'a', 'p', 'p',
                 'l', 'y', ' ', 't', 'h', 'e', ' ', 's', 'i', 'z', 'e', '-',
                 'c', 'h', 'a', 'n', 'g', 'e', ' ', 'p', 'r', 'o', 'c', 'e',
                 's', 's', 'o', 'r', ' ', 'w', 'i', 't', 'h', ' ', 't', 'h',
                 'e', ' ', 'f', 'o', 'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ']
              . shows_nl .
                  representation .
                    shows_nl .
                      shows_string
                        ['f', 'o', 'r', ' ', 't', 'h', 'e', ' ', 'f', 'o', 'l',
                          'l', 'o', 'w', 'i', 'n', 'g', ' ', 'r', 'e', 'a', 's',
                          'o', 'n']
                        . shows_nl . x));
  };

shows_arctic_rat :: Arctic_rat -> [Char] -> [Char];
shows_arctic_rat (Num_arc_rat i) = shows_rat i;
shows_arctic_rat MinInfty_rat = shows_string ['-', 'i', 'n', 'f'];

shows_list_arctic_rat :: [Arctic_rat] -> [Char] -> [Char];
shows_list_arctic_rat asa = shows_list_aux shows_arctic_rat asa;

instance Shows Arctic_rat where {
  showsb = shows_arctic_rat;
  shows_list = shows_list_arctic_rat;
};

create_sct_ur_af_proc_comp ::
  forall a b c.
    (Eq a, Shows a, Eq b, Shows b, Eq c,
      Shows c) => (a -> Bool,
                    (a, (a, (a -> a -> a,
                              (a -> a -> a,
                                (a -> a,
                                  (a -> a -> Bool,
                                    (a -> a -> Bool,
                                      (Bool, ([Char], a -> Bool))))))))))
                    -> [((b, Nat), (a, [a]))] ->
                         [(Term b c, Term b c)] ->
                           Maybe (b -> [(Term b c, Term b c)]) ->
                             Maybe (b -> [(Term b c, Term b c)]) ->
                               [(Term b c, Term b c)] ->
                                 [((Term b c, Term b c),
                                    ([(Nat, Nat)], [(Nat, Nat)]))]
                                   -> Eithera ([Char] -> [Char]) ();
create_sct_ur_af_proc_comp c i p rm rrm ur graphs =
  let {
    pia = to_lpoly_inter (get_comp_1 c) i;
  } in bind (check_lpoly_coeffs (get_comp_wf_elem c) (get_comp_apos c)
              (\ x -> get_comp_ge c x (get_comp_0 c)) i)
         (\ _ ->
           check_sct_ur_af_proc
             (check_polo_sI (get_comp_mono c) (get_comp_0 c) (get_comp_1 c)
               (get_comp_plus c) (get_comp_times c) (get_comp_max0 c)
               (get_comp_ge c) (get_comp_gt c) pia)
             (check_polo_nsI (get_comp_0 c) (get_comp_1 c) (get_comp_plus c)
               (get_comp_times c) (get_comp_max0 c) (get_comp_ge c) pia)
             (shows_string
                ['p', 'o', 'l', 'y', 'n', 'o', 'm', 'i', 'a', 'l', ' ', 'o',
                  'r', 'd', 'e', 'r', ' ', '(']
               . shows_string (get_comp_description c) .
                   shows_string [')'] .
                     shows_nl . create_lpoly_repr c i . shows_nl)
             p graphs rm rrm ur (create_afI (get_comp_0 c) i));

create_sct_ur_af_proc ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => Redpair a ->
                    [(Term a b, Term a b)] ->
                      Maybe (a -> [(Term a b, Term a b)]) ->
                        Maybe (a -> [(Term a b, Term a b)]) ->
                          [(Term a b, Term a b)] ->
                            [((Term a b, Term a b),
                               ([(Nat, Nat)], [(Nat, Nat)]))]
                              -> Eithera ([Char] -> [Char]) ();
create_sct_ur_af_proc (Int_carrier i) =
  create_sct_ur_af_proc_comp int_components i;
create_sct_ur_af_proc (Nat_carrier i) =
  create_sct_ur_af_proc_comp nat_components i;
create_sct_ur_af_proc (Rat_carrier delta i) =
  create_sct_ur_af_proc_comp (rat_components delta) i;
create_sct_ur_af_proc (Arctic_carrier i) =
  create_sct_ur_af_proc_comp arctic_components i;
create_sct_ur_af_proc (Arctic_rat_carrier delta i) =
  create_sct_ur_af_proc_comp (arctic_rat_components delta) i;
create_sct_ur_af_proc (Int_mat_carrier n sd i) =
  create_sct_ur_af_proc_comp (mat_components n sd int_components) i;
create_sct_ur_af_proc (Rat_mat_carrier delta n sd i) =
  create_sct_ur_af_proc_comp (mat_components n sd (rat_components delta)) i;
create_sct_ur_af_proc (Arc_mat_carrier n i) =
  create_sct_ur_af_proc_comp (mat_arctic_components n arctic_components) i;
create_sct_ur_af_proc (Arc_rat_mat_carrier delta n i) =
  create_sct_ur_af_proc_comp
    (mat_arctic_components n (arctic_rat_components delta)) i;
create_sct_ur_af_proc (Lpo pri pia) =
  (\ p rm rrm ur graphs ->
    let {
      m = maxa (get_max_arity ur) (get_max_arity p);
      pr = lpo_repr_to_pr pri;
      piaa = afs_list_to_afs pia;
    } in bind (check_wf_afs pia)
           (\ _ ->
             check_sct_ur_af_proc (afs_check lpoS piaa (lpo_strict pr m))
               (afs_check lpoNS piaa (lpo_nstrict pr m)) (shows_lpo_af pri pia)
               p graphs rm rrm ur (afs_list_to_af pia)));

check_supteq ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => (Term a b, Term a b) -> Eithera ([Char] -> [Char]) ();
check_supteq lr =
  catcha
    (checkB (supteqp (fst lr) (snd lr))
      (shows_string
         ['t', 'h', 'e', ' ', 'r', 'h', 's', ' ', 'i', 's', ' ', 'n', 'o', 't',
           ' ', 'a', ' ', 's', 'u', 'b', 't', 'e', 'r', 'm', ' ', 'o', 'f', ' ',
           't', 'h', 'e', ' ', 'l', 'h', 's']
        . shows_nl))
    (\ x ->
      errora
        (shows_string
           ['C', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'o', 'r', 'i',
             'e', 'n', 't', ' ', 'r', 'u', 'l', 'e', ' ']
          . shows_rule lr . shows_nl . x));

check_sct_subterm_proc ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => [(Term a b, Term a b)] ->
                    [((Term a b, Term a b), ([(Nat, Nat)], [(Nat, Nat)]))] ->
                      Maybe (a -> [(Term a b, Term a b)]) ->
                        Maybe (a -> [(Term a b, Term a b)]) ->
                          Eithera ([Char] -> [Char]) ();
check_sct_subterm_proc p gs rm rrm =
  case rm of {
    Nothing ->
      errora
        (shows_string
           ['T', 'R', 'S', ' ', 'h', 'a', 's', ' ', 'v', 'a', 'r', 'i', 'a',
             'b', 'l', 'e', 's', ' ', 'a', 's', ' ', 'l', 'e', 'f', 't', '-',
             'h', 'a', 'n', 'd', ' ', 's', 'i', 'd', 'e', 's']
          . shows_nl);
    Just irm ->
      catcha
        (bind (bind (catcha (checkSubset p (mapa fst gs))
                      (\ x ->
                        errora
                          (shows_string
                             ['t', 'h', 'e', 'r', 'e', ' ', 'i', 's', ' ', 'n',
                               'o', ' ', 's', 'i', 'z', 'e', '-', 'c', 'h', 'a',
                               'n', 'g', 'e', ' ', 'g', 'r', 'a', 'p', 'h', ' ',
                               'f', 'o', 'r', ' ', 'D', 'P', ' ']
                            . shows_rule x . shows_nl)))
                (\ _ ->
                  catcha
                    (checkAllMI
                      (\ (a, b) ->
                        let {
                          (s, t) = a;
                        } in (\ (stri, nstri) ->
                               catcha
                                 (bind (checkB
 (not (is_var s) && (not (is_var t) && nulla (irm (the (root t)))))
 (shows_string
    ['r', 'o', 'o', 't', 's', ' ', 'o', 'f', ' ', 'D', 'P', ' ', 'm', 'u', 's',
      't', ' ', 'b', 'e', ' ', 'c', 'o', 'n', 's', 't', 'r', 'u', 'c', 't', 'o',
      'r', 's']
   . shows_nl))
                                   (\ _ ->
                                     let {
                                       n = size_list (args s);
                                       m = size_list (args t);
                                     } in bind
    (catcha
      (checkAllMI
        (\ (i, j) ->
          checkB
            (less_eq_nat i n &&
              (less_eq_nat j m &&
                isValue (check_supt (get_arg s i, get_arg t j))))
            (shows_string
               ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'i', 't', 'h', ' ',
                 'e', 'd', 'g', 'e', ' ']
              . shows_nat i .
                  shows_string [' ', '|', '>', ' '] . shows_nat j . shows_nl))
        stri)
      (\ x -> errora (snd x)))
    (\ _ ->
      catcha
        (checkAllMI
          (\ (i, j) ->
            checkB
              (less_eq_nat i n &&
                (less_eq_nat j m &&
                  isValue (check_supteq (get_arg s i, get_arg t j))))
              (shows_string
                 ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'i', 't', 'h',
                   ' ', 'e', 'd', 'g', 'e', ' ']
                . shows_nat i .
                    shows_string [' ', '|', '>', '=', ' '] .
                      shows_nat j . shows_nl))
          nstri)
        (\ x -> errora (snd x)))))
                                 (\ x ->
                                   errora
                                     (shows_string
['p', 'r', 'o', 'b', 'l', 'e', 'm', 's', ' ', 'w', 'i', 't', 'h', ' ', 'D', 'P',
  ' ']
                                       . shows_rule (s, t) . shows_nl . x)))
                          b)
                      gs)
                    (\ x -> errora (snd x))))
          (\ _ ->
            checkB
              (check_SCT
                (\ (_, (t, (_, ct))) (u, (_, (cu, _))) ->
                  gcMatchI ct u && gcMatchI cu t)
                (mapa (\ (a, b) ->
                        let {
                          (s, t) = a;
                        } in (\ (stri, nstri) ->
                               let {
                                 e = (s, (t, (capRM rrm s, capRM rm t)));
                               } in Scg e e stri nstri)
                          b)
                  gs))
              (shows_string
                 ['s', 'i', 'z', 'e', '-', 'c', 'h', 'a', 'n', 'g', 'e', ' ',
                   'a', 'n', 'a', 'l', 'y', 's', 'i', 's', ' ', 'f', 'a', 'i',
                   'l', 'e', 'd']
                . shows_nl)))
        (\ x ->
          errora
            (shows_string
               ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'a', 'p', 'p',
                 'l', 'y', ' ', 't', 'h', 'e', ' ', 's', 'i', 'z', 'e', '-',
                 'c', 'h', 'a', 'n', 'g', 'e', ' ', 'p', 'r', 'o', 'c', 'e',
                 's', 's', 'o', 'r', ' ', 'w', 'i', 't', 'h', ' ', 's', 'u',
                 'b', 't', 'e', 'r', 'm', '-', 'r', 'e', 'l', 'a', 't', 'i',
                 'o', 'n', ' ', 'f', 'o', 'r', ' ', 't', 'h', 'e', ' ', 'f',
                 'o', 'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'r', 'e', 'a',
                 's', 'o', 'n']
              . shows_nl . x));
  };

create_sp_representation :: forall a. (Shows a) => SpL a -> [Char] -> [Char];
create_sp_representation (SimpProj sp) =
  shows_map
    (\ (f, n) ->
      shows_string ['p', 'i', '('] .
        showsb f . shows_string [')', ' ', '=', ' '] . shows_nat n . shows_nl)
    sp;

lift2term :: forall a b. (a -> Nat) -> Term a b -> Term a b;
lift2term p (Var x) = Var x;
lift2term p (Fun f ss) = nth ss (p f);

check_supteq_proj ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => (a -> Nat) ->
                    (Term a b, Term a b) -> Eithera ([Char] -> [Char]) ();
check_supteq_proj p lr =
  catcha
    (checkB (supteqp (lift2term p (fst lr)) (lift2term p (snd lr)))
      (shows_string
         ['t', 'h', 'e', ' ', 'p', 'r', 'o', 'j', 'e', 'c', 't', 'e', 'd', ' ',
           'r', 'h', 's', ' ', 'i', 's', ' ', 'n', 'o', 't', ' ', 'a', ' ', 's',
           'u', 'b', 't', 'e', 'r', 'm', ' ', 'o', 'f', ' ', 't', 'h', 'e', ' ',
           'p', 'r', 'o', 'j', 'e', 'c', 't', 'e', 'd', ' ', 'l', 'h', 's']
        . shows_nl))
    (\ x ->
      errora
        (shows_string
           ['C', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'o', 'r', 'i',
             'e', 'n', 't', ' ', 'r', 'u', 'l', 'e', ' ']
          . showsa lr .
              shows_nl .
                shows_string ['i', '.', 'e', '.', ',', ' '] .
                  shows_term (lift2term p (fst lr)) .
                    shows_string [' ', '|', '>', '=', ' '] .
                      shows_term (lift2term p (snd lr)) . shows_nl . x));

check_projection ::
  forall a b.
    (Shows a,
      Shows b) => (a -> [(Term a b, Term a b)]) ->
                    (a -> Nat) ->
                      (Term a b, Term a b) -> Eithera ([Char] -> [Char]) ();
check_projection ds p (Fun f ss, Fun g ts) =
  let {
    ls = size_list ss;
    lt = size_list ts;
  } in bind (bind (bind (bind (checkB (less_nat Zero_nat ls)
                                (shows_string
                                   ['l', 'h', 's', ' ', 'i', 's', ' ', 'a', ' ',
                                     'c', 'o', 'n', 's', 't', 'a', 'n', 't']
                                  . shows_nl))
                          (\ _ ->
                            checkB (less_nat (p f) ls)
                              (shows_nat (Suc (p f)) .
                                shows_string
                                  [' ', 'i', 's', ' ', 'n', 'o', 't', ' ', 'a',
                                    'n', ' ', 'a', 'r', 'g', 'u', 'm', 'e', 'n',
                                    't', ' ', 'p', 'o', 's', 'i', 't', 'i', 'o',
                                    'n', ' ', 'o', 'f', ' ']
                                  . shows_term (Fun f ss) . shows_nl)))
                    (\ _ ->
                      checkB (less_nat Zero_nat lt)
                        (shows_string
                           ['r', 'h', 's', ' ', 'i', 's', ' ', 'a', ' ', 'c',
                             'o', 'n', 's', 't', 'a', 'n', 't']
                          . shows_nl)))
              (\ _ ->
                checkB (nulla (ds g))
                  (showsb g .
                    shows_string
                      [' ', 'i', 's', ' ', 'd', 'e', 'f', 'i', 'n', 'e', 'd',
                        ' ', 'i', 'n', ' ', 'R']
                      . shows_nl)))
         (\ _ ->
           checkB (less_nat (p g) lt)
             (shows_nat (Suc (p g)) .
               shows_string
                 [' ', 'i', 's', ' ', 'n', 'o', 't', ' ', 'a', 'n', ' ', 'a',
                   'r', 'g', 'u', 'm', 'e', 'n', 't', ' ', 'p', 'o', 's', 'i',
                   't', 'i', 'o', 'n', ' ', 'o', 'f', ' ']
                 . shows_term (Fun g ts) . shows_nl));
check_projection ds p (Var v, t) =
  errora
    (shows_string
       ['c', 'a', 'n', 'n', 'o', 't', ' ', 'p', 'r', 'o', 'j', 'e', 'c', 't',
         ' ', 'a', ' ', 'v', 'a', 'r', 'i', 'a', 'b', 'l', 'e']
      . shows_nl);
check_projection ds p (s, Var v) =
  errora
    (shows_string
       ['c', 'a', 'n', 'n', 'o', 't', ' ', 'p', 'r', 'o', 'j', 'e', 'c', 't',
         ' ', 'a', ' ', 'v', 'a', 'r', 'i', 'a', 'b', 'l', 'e']
      . shows_nl);

check_supt_proj ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => (a -> Nat) ->
                    (Term a b, Term a b) -> Eithera ([Char] -> [Char]) ();
check_supt_proj p lr =
  catcha
    (checkB (suptp (lift2term p (fst lr)) (lift2term p (snd lr)))
      (shows_string
         ['t', 'h', 'e', ' ', 'p', 'r', 'o', 'j', 'e', 'c', 't', 'e', 'd', ' ',
           'r', 'h', 's', ' ', 'i', 's', ' ', 'n', 'o', 't', ' ', 'a', ' ', 'p',
           'r', 'o', 'p', 'e', 'r', ' ', 's', 'u', 'b', 't', 'e', 'r', 'm', ' ',
           'o', 'f', ' ', 't', 'h', 'e', ' ', 'p', 'r', 'o', 'j', 'e', 'c', 't',
           'e', 'd', ' ', 'l', 'h', 's']
        . shows_nl))
    (\ x ->
      errora
        (shows_string
           ['C', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'o', 'r', 'i',
             'e', 'n', 't', ' ', 'r', 'u', 'l', 'e', ' ']
          . shows_rule lr .
              shows_nl .
                shows_string ['i', '.', 'e', '.', ',', ' '] .
                  shows_term (lift2term p (fst lr)) .
                    shows_string [' ', '|', '>', ' '] .
                      shows_term (lift2term p (snd lr)) . shows_nl . x));

check_spsc_proc ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => (a -> Nat) ->
                    ([Char] -> [Char]) ->
                      [(Term a b, Term a b)] ->
                        [(Term a b, Term a b)] ->
                          [(Term a b, Term a b)] ->
                            Maybe (a -> [(Term a b, Term a b)]) ->
                              Eithera ([Char] -> [Char]) ();
check_spsc_proc proj rep p rP r (Just rm) =
  let {
    (_, nSp) = partition (\ ra -> isValue (check_supt_proj proj ra)) p;
  } in catcha
         (bind (bind (catcha
                       (catcha (checkAllMI (check_projection rm proj) p)
                         (\ x -> errora (snd x)))
                       (\ x ->
                         errora
                           (shows_string
                              ['t', 'h', 'e', ' ', 's', 'u', 'p', 'p', 'l', 'i',
                                'e', 'd', ' ', 'p', 'r', 'o', 'j', 'e', 'c',
                                't', 'i', 'o', 'n', ' ', 'i', 's', ' ', 'm',
                                'a', 'l', 'f', 'o', 'r', 'm', 'e', 'd']
                             . shows_nl . x)))
                 (\ _ ->
                   catcha
                     (catcha (checkAllMI (check_supteq_proj proj) nSp)
                       (\ x -> errora (snd x)))
                     (\ x ->
                       errora
                         (shows_string
                            ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'h',
                              'e', 'n', ' ', 'o', 'r', 'i', 'e', 'n', 't', 'i',
                              'n', 'g', ' ', 'D', 'P', 's']
                           . shows_nl . x))))
           (\ _ ->
             catcha (checkSubset nSp rP)
               (\ x ->
                 errora
                   (case check_supt_proj proj x of {
                      Lefta err -> err;
                      Righta _ -> internal_error;
                    }))))
         (\ x ->
           errora
             (shows_string
                ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'a', 'p',
                  'p', 'l', 'y', ' ', 't', 'h', 'e', ' ', 's', 'u', 'b', 't',
                  'e', 'r', 'm', ' ', 'c', 'r', 'i', 't', 'e', 'r', 'i', 'o',
                  'n', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r', ' ',
                  'w', 'i', 't', 'h', ' ', 't', 'h', 'e', ' ', 'f', 'o', 'l',
                  'l', 'o', 'w', 'i', 'n', 'g', ' ', 's', 'i', 'm', 'p', 'l',
                  'e', ' ', 'p', 'r', 'o', 'j', 'e', 'c', 't', 'i', 'o', 'n']
               . shows_nl . rep . shows_nl . x));
check_spsc_proc proj rep p rP r Nothing = returna ();

create_sp :: forall a. (Eq a) => SpL a -> a -> Nat;
create_sp (SimpProj sp) f =
  case lookupa f sp of {
    Nothing -> Zero_nat;
    Just n -> n;
  };

create_spsc_proc ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => SpL a ->
                    [(Term a b, Term a b)] ->
                      [(Term a b, Term a b)] ->
                        [(Term a b, Term a b)] ->
                          Maybe (a -> [(Term a b, Term a b)]) ->
                            Eithera ([Char] -> [Char]) ();
create_spsc_proc sp p rP r rm =
  let {
    pia = create_sp sp;
  } in check_spsc_proc pia (create_sp_representation sp) p rP r rm;

create_sem_lab_proc ::
  forall a b c.
    (Eq a, Shows a, Eq b, Shows b, Eq c,
      Shows c) => Sl_inter a b ->
                    (Lab a b -> [Nat] -> Eithera b [(Lab a b)]) ->
                      [(Term (Lab a b) c, Term (Lab a b) c)] ->
                        [(Term (Lab a b) c, Term (Lab a b) c)] ->
                          [(Term (Lab a b) c, Term (Lab a b) c)] ->
                            [(Term (Lab a b) c, Term (Lab a b) c)] ->
                              Eithera ([Char] -> [Char]) ();
create_sem_lab_proc sli l r1 r2 r3 r4 =
  let {
    c = get_largest_element sli;
    ca = upt Zero_nat (Suc c);
    i = sl_inter_to_inter sli;
  } in catcha
         (check_sem_lab_proc (check_sl_lab_trs i l ca)
           (check_sl_model_lab_trs i l ca) (\ _ -> False) i l r1 r2 r3 r4)
         (\ x -> errora (shows_sl_inter sli . shows_nl . x));

check_unlab_proc ::
  forall a b c.
    (Eq a, Shows a, Eq b, Shows b, Eq c,
      Shows c) => [(Term (Lab a b) c, Term (Lab a b) c)] ->
                    [(Term (Lab a b) c, Term (Lab a b) c)] ->
                      [(Term (Lab a b) c, Term (Lab a b) c)] ->
                        [(Term (Lab a b) c, Term (Lab a b) c)] ->
                          Eithera ([Char] -> [Char]) ();
check_unlab_proc p r ulP ulR =
  let {
    ulr = unlab_of_trsI r;
  } in catcha
         (bind (bind (checkSubset (map_trsI unlab p) ulP)
                 (\ _ -> checkSubset ulr ulR))
           (\ _ -> checkSubset ulR ulr))
         (\ x ->
           errora
             (shows_string
                ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'i', 't', 'h',
                  ' ', 'r', 'u', 'l', 'e', ' ']
               . shows_rule x . shows_nl));

matchCapBelowI ::
  forall a b.
    (Eq a, Eq b) => Term a b -> [(Term a b, Term a b)] -> Term a b -> Bool;
matchCapBelowI l r (Fun f ts) = gcMatchI (HFun f (mapa (capI r) ts)) l;

check_ur_closed_term_af ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => [(Term a b, Term a b)] ->
                    [(Term a b, Term a b)] ->
                      (a -> Nat -> Nat -> Bool) ->
                        Term a b -> Eithera ([Char] -> [Char]) ();
check_ur_closed_term_af uu uv uw (Var x) = returna ();
check_ur_closed_term_af r ur pia (Fun f ts) =
  let {
    n = size_list ts;
  } in bind (checkAllLI
              (\ t i ->
                (if pia f n i then check_ur_closed_term_af r ur pia t
                  else returna ()))
              ts)
         (\ _ ->
           catcha
             (checkAllMI
               (\ lr ->
                 checkB
                   (member lr ur || not (matchCapBelowI (fst lr) r (Fun f ts)))
                   (shows_string
                      ['d', 'u', 'e', ' ', 't', 'o', ' ', 't', 'h', 'e', ' ',
                        's', 'u', 'b', 't', 'e', 'r', 'm', ' ']
                     . shows_term (Fun f ts) .
                         shows_string
                           [' ', 'o', 'f', ' ', 's', 'o', 'm', 'e', ' ', 'u',
                             's', 'a', 'b', 'l', 'e', ' ', 'r', 'h', 's', ',',
                             ' ', 'r', 'u', 'l', 'e', ' ']
                           . shows_rule lr .
                               shows_string
                                 [' ', 's', 'h', 'o', 'u', 'l', 'd', ' ', 'b',
                                   'e', ' ', 'u', 's', 'a', 'b', 'l', 'e',
                                   '.']))
               r)
             (\ x -> errora (snd x)));

check_ur_P_closed_af ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => [(Term a b, Term a b)] ->
                    [(Term a b, Term a b)] ->
                      [(Term a b, Term a b)] ->
                        (a -> Nat -> Nat -> Bool) ->
                          Eithera ([Char] -> [Char]) ();
check_ur_P_closed_af r ur p pia =
  bind (catcha
         (catcha
           (checkAllMI (\ lr -> check_ur_closed_term_af r ur pia (snd lr)) ur)
           (\ x -> errora (snd x)))
         (\ x ->
           errora
             (shows_string
                ['e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ', 'c',
                  'h', 'e', 'c', 'k', 'i', 'n', 'g', ' ', 'c', 'l', 'o', 's',
                  'u', 'r', 'e', ' ', 'p', 'r', 'o', 'p', 'e', 'r', 't', 'i',
                  'e', 's', ' ', 'o', 'f', ' ', 'r', 'h', 's', ' ', 'o', 'f',
                  ' ', 'u', 's', 'a', 'b', 'l', 'e', ' ', 'r', 'u', 'l', 'e',
                  's']
               . shows_nl . x)))
    (\ _ ->
      catcha
        (catcha
          (checkAllMI (\ st -> check_ur_closed_term_af r ur pia (snd st)) p)
          (\ x -> errora (snd x)))
        (\ x ->
          errora
            (shows_string
               ['e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ', 'c', 'h',
                 'e', 'c', 'k', 'i', 'n', 'g', ' ', 'c', 'l', 'o', 's', 'u',
                 'r', 'e', ' ', 'p', 'r', 'o', 'p', 'e', 'r', 't', 'i', 'e',
                 's', ' ', 'o', 'f', ' ', 'r', 'h', 's', ' ', 'o', 'f', ' ',
                 'D', 'P', 's']
              . shows_nl . x)));

check_ur_P_closed_rm_af ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => [(Term a b, Term a b)] ->
                    Maybe (a -> [(Term a b, Term a b)]) ->
                      [(Term a b, Term a b)] ->
                        [(Term a b, Term a b)] ->
                          (a -> Nat -> Nat -> Bool) ->
                            Eithera ([Char] -> [Char]) ();
check_ur_P_closed_rm_af r rmo ur p pia =
  case rmo of {
    Nothing -> check_ur_P_closed_af r ur p pia;
    Just rm ->
      bind (catcha
             (catcha
               (checkAllMI
                 (\ lr -> check_ur_closed_term_rm_af rm ur pia (snd lr)) ur)
               (\ x -> errora (snd x)))
             (\ x ->
               errora
                 (shows_string
                    ['e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ', 'c',
                      'h', 'e', 'c', 'k', 'i', 'n', 'g', ' ', 'c', 'l', 'o',
                      's', 'u', 'r', 'e', ' ', 'p', 'r', 'o', 'p', 'e', 'r',
                      't', 'i', 'e', 's', ' ', 'o', 'f', ' ', 'r', 'h', 's',
                      ' ', 'o', 'f', ' ', 'u', 's', 'a', 'b', 'l', 'e', ' ',
                      'r', 'u', 'l', 'e', 's']
                   . shows_nl . x)))
        (\ _ ->
          catcha
            (catcha
              (checkAllMI
                (\ st -> check_ur_closed_term_rm_af rm ur pia (snd st)) p)
              (\ x -> errora (snd x)))
            (\ x ->
              errora
                (shows_string
                   ['e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ', 'c',
                     'h', 'e', 'c', 'k', 'i', 'n', 'g', ' ', 'c', 'l', 'o', 's',
                     'u', 'r', 'e', ' ', 'p', 'r', 'o', 'p', 'e', 'r', 't', 'i',
                     'e', 's', ' ', 'o', 'f', ' ', 'r', 'h', 's', ' ', 'o', 'f',
                     ' ', 'D', 'P', 's']
                  . shows_nl . x)));
  };

checkMonoRedPairUrProc ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => Eithera ([Char] -> [Char]) () ->
                    ((Term a b, Term a b) -> Eithera ([Char] -> [Char]) ()) ->
                      ((Term a b, Term a b) -> Eithera ([Char] -> [Char]) ()) ->
                        ([Char] -> [Char]) ->
                          ([Char] -> [Char]) ->
                            [(Term a b, Term a b)] ->
                              [(Term a b, Term a b)] ->
                                [(Term a b, Term a b)] ->
                                  Maybe (a -> [(Term a b, Term a b)]) ->
                                    [(Term a b, Term a b)] ->
                                      [(Term a b, Term a b)] ->
Eithera ([Char] -> [Char]) ();
checkMonoRedPairUrProc checkMono checkSS checkNSS name representation p rP r rm
  rR ur =
  let {
    us = concat_map (funas_term_list . snd) (append p ur);
    filt = filtera (\ lr -> wf_fun_terms_list us (fst lr));
    (_, nSp) = partition (\ foo -> isValue (checkSS foo)) p;
    (_, nSr) = partition (\ foo -> isValue (checkSS foo)) ur;
  } in catcha
         (bind (bind (bind (bind (bind (bind
 (catcha checkMono
   (\ x ->
     errora
       (shows_string
          ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'i', 't', 'h', ' ', 'm',
            'o', 'n', 'o', 't', 'o', 'n', 'i', 'c', 'i', 't', 'y', ' ', 'o',
            'f', ' ', 's', 't', 'r', 'i', 'c', 't', ' ', 'o', 'r', 'd', 'e',
            'r']
         . shows_nl . x)))
 (\ _ ->
   catcha (check_ur_P_closed_rm_af r rm ur p pi_full)
     (\ x ->
       errora
         (shows_string
            ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'i', 't', 'h', ' ',
              'c', 'l', 'o', 's', 'u', 'r', 'e', ' ', 'p', 'r', 'o', 'p', 'e',
              'r', 't', 'i', 'e', 's', ' ', 'o', 'f', ' ', 'u', 's', 'a', 'b',
              'l', 'e', ' ', 'r', 'u', 'l', 'e', 's']
           . shows_nl . x))))
                                   (\ _ ->
                                     catcha
                                       (catcha (checkAllMI checkNSS nSr)
 (\ x -> errora (snd x)))
                                       (\ x ->
 errora
   (shows_string
      ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'h', 'e', 'n', ' ', 'o',
        'r', 'i', 'e', 'n', 't', 'i', 'n', 'g', ' ', 'u', 's', 'a', 'b', 'l',
        'e', ' ', 'r', 'u', 'l', 'e', 's']
     . shows_nl . x))))
                             (\ _ ->
                               catcha
                                 (catcha (checkAllMI checkNSS nSp)
                                   (\ x -> errora (snd x)))
                                 (\ x ->
                                   errora
                                     (shows_string
['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'h', 'e', 'n', ' ', 'o', 'r', 'i',
  'e', 'n', 't', 'i', 'n', 'g', ' ', 'D', 'P', 's']
                                       . shows_nl . x))))
                       (\ _ ->
                         catcha (checkSubset rR r)
                           (\ _ ->
                             errora
                               (shows_string
                                  ['p', 'r', 'o', 'b', 'l', 'e', 'm', ':', ' ',
                                    'a', 'd', 'd', 'i', 't', 'i', 'o', 'n', 'a',
                                    'l', ' ', 'r', 'u', 'l', 'e', '(', 's', ')',
                                    ' ', 't', 'o', ' ', 'T', 'R', 'S', ' ', 'a',
                                    'd', 'd', 'e', 'd']
                                 . shows_nl))))
                 (\ _ ->
                   catcha (checkSubset (filt nSr) rR)
                     (\ x ->
                       errora
                         (case checkSS x of {
                            Lefta err -> err;
                            Righta _ -> internal_error;
                          }))))
           (\ _ ->
             catcha (checkSubset (filt nSp) rP)
               (\ x ->
                 errora
                   (case checkSS x of {
                      Lefta err -> err;
                      Righta _ -> internal_error;
                    }))))
         (\ x ->
           errora
             (shows_string
                ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'a', 'p',
                  'p', 'l', 'y', ' ', 't', 'h', 'e', ' ', '(', 'm', 'o', 'n',
                  'o', 't', 'o', 'n', 'i', 'c', ')', ' ', 'r', 'e', 'd', 'u',
                  'c', 't', 'i', 'o', 'n', ' ', 'p', 'a', 'i', 'r', ' ', 'p',
                  'r', 'o', 'c', 'e', 's', 's', 'o', 'r', ' ', 'w', 'i', 't',
                  'h', ' ', 't', 'h', 'e', ' ', 'f', 'o', 'l', 'l', 'o', 'w',
                  'i', 'n', 'g']
               . shows_nl .
                   representation .
                     shows_nl .
                       shows_string
                         ['a', 'n', 'd', ' ', 't', 'h', 'e', ' ', 'f', 'o', 'l',
                           'l', 'o', 'w', 'i', 'n', 'g', ' ', 'u', 's', 'a',
                           'b', 'l', 'e', ' ', 'r', 'u', 'l', 'e', 's']
                         . shows_rules ur . shows_nl . x));

create_mono_redpair_ur_proc_comp ::
  forall a b c.
    (Eq a, Shows a, Eq b, Shows b, Eq c,
      Shows c) => (a -> Bool,
                    (a, (a, (a -> a -> a,
                              (a -> a -> a,
                                (a -> a,
                                  (a -> a -> Bool,
                                    (a -> a -> Bool,
                                      (Bool, ([Char], a -> Bool))))))))))
                    -> (a -> Bool) ->
                         [((b, Nat), (a, [a]))] ->
                           [(Term b c, Term b c)] ->
                             [(Term b c, Term b c)] ->
                               [(Term b c, Term b c)] ->
                                 Maybe (b -> [(Term b c, Term b c)]) ->
                                   [(Term b c, Term b c)] ->
                                     [(Term b c, Term b c)] ->
                                       Eithera ([Char] -> [Char]) ();
create_mono_redpair_ur_proc_comp c cm i p rP r rm rR ur =
  let {
    pia = to_lpoly_inter (get_comp_1 c) i;
  } in bind (check_lpoly_coeffs (get_comp_wf_elem c) (get_comp_apos c)
              (\ x -> get_comp_ge c x (get_comp_0 c)) i)
         (\ _ ->
           checkMonoRedPairUrProc
             (check_poly_monoI (get_comp_0 c) (get_comp_ge c) cm i)
             (check_polo_sI (get_comp_mono c) (get_comp_0 c) (get_comp_1 c)
               (get_comp_plus c) (get_comp_times c) (get_comp_max0 c)
               (get_comp_ge c) (get_comp_gt c) pia)
             (check_polo_nsI (get_comp_0 c) (get_comp_1 c) (get_comp_plus c)
               (get_comp_times c) (get_comp_max0 c) (get_comp_ge c) pia)
             (shows_string
                ['p', 'o', 'l', 'y', 'n', 'o', 'm', 'i', 'a', 'l', ' ', 'o',
                  'r', 'd', 'e', 'r', ' ', '(']
               . shows_string (get_comp_description c) . shows_string [')'])
             (create_lpoly_repr c i) p rP r rm rR ur);

create_mono_redpair_ur_proc ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => Redpair a ->
                    [(Term a b, Term a b)] ->
                      [(Term a b, Term a b)] ->
                        [(Term a b, Term a b)] ->
                          Maybe (a -> [(Term a b, Term a b)]) ->
                            [(Term a b, Term a b)] ->
                              [(Term a b, Term a b)] ->
                                Eithera ([Char] -> [Char]) ();
create_mono_redpair_ur_proc (Int_carrier i) =
  create_mono_redpair_ur_proc_comp int_components int_mono i;
create_mono_redpair_ur_proc (Nat_carrier i) =
  create_mono_redpair_ur_proc_comp nat_components nat_mono i;
create_mono_redpair_ur_proc (Rat_carrier delta i) =
  create_mono_redpair_ur_proc_comp (rat_components delta) rat_mono i;
create_mono_redpair_ur_proc (Arctic_carrier i) =
  (\ _ _ _ _ _ _ ->
    errora
      (shows_string
         ['a', 'r', 'c', 't', 'i', 'c', ' ', 'n', 'o', 't', ' ', 'm', 'o', 'n',
           'o', 't', 'o', 'n', 'i', 'c', ' ', 'f', 'o', 'r', ' ', 'T', 'R', 'S',
           's']
        . shows_nl));
create_mono_redpair_ur_proc (Arctic_rat_carrier delta i) =
  (\ _ _ _ _ _ _ ->
    errora
      (shows_string
         ['a', 'r', 'c', 't', 'i', 'c', ' ', 'n', 'o', 't', ' ', 'm', 'o', 'n',
           'o', 't', 'o', 'n', 'i', 'c', ' ', 'f', 'o', 'r', ' ', 'T', 'R', 'S',
           's']
        . shows_nl));
create_mono_redpair_ur_proc (Arc_mat_carrier n i) =
  (\ _ _ _ _ _ _ ->
    errora
      (shows_string
         ['a', 'r', 'c', 't', 'i', 'c', ' ', 'n', 'o', 't', ' ', 'm', 'o', 'n',
           'o', 't', 'o', 'n', 'i', 'c', ' ', 'f', 'o', 'r', ' ', 'T', 'R', 'S',
           's']
        . shows_nl));
create_mono_redpair_ur_proc (Arc_rat_mat_carrier delta n i) =
  (\ _ _ _ _ _ _ ->
    errora
      (shows_string
         ['a', 'r', 'c', 't', 'i', 'c', ' ', 'n', 'o', 't', ' ', 'm', 'o', 'n',
           'o', 't', 'o', 'n', 'i', 'c', ' ', 'f', 'o', 'r', ' ', 'T', 'R', 'S',
           's']
        . shows_nl));
create_mono_redpair_ur_proc (Int_mat_carrier n sd i) =
  create_mono_redpair_ur_proc_comp (mat_components n sd int_components)
    (mat_mono sd int_mono) i;
create_mono_redpair_ur_proc (Rat_mat_carrier d n sd i) =
  create_mono_redpair_ur_proc_comp (mat_components n sd (rat_components d))
    (mat_mono sd rat_mono) i;
create_mono_redpair_ur_proc (Lpo pri pia) =
  (\ p rP r rR rm ur ->
    let {
      m = maxa (get_max_arity r) (get_max_arity p);
      pr = lpo_repr_to_pr pri;
      piaa = afs_list_to_afs pia;
    } in bind (check_wf_afs pia)
           (\ _ ->
             checkMonoRedPairUrProc (check_mono_afs pia)
               (afs_check lpoS piaa (lpo_strict pr m))
               (afs_check lpoNS piaa (lpo_nstrict pr m)) lpo_repr
               (shows_lpo_af pri pia) p rP r rR rm ur));

checkMonoRedPairProc ::
  forall a b.
    (Eq a,
      Eq b) => Eithera ([Char] -> [Char]) () ->
                 ((Term a b, Term a b) -> Eithera ([Char] -> [Char]) ()) ->
                   ((Term a b, Term a b) -> Eithera ([Char] -> [Char]) ()) ->
                     ([Char] -> [Char]) ->
                       ([Char] -> [Char]) ->
                         [(Term a b, Term a b)] ->
                           [(Term a b, Term a b)] ->
                             [(Term a b, Term a b)] ->
                               [(Term a b, Term a b)] ->
                                 Eithera ([Char] -> [Char]) ();
checkMonoRedPairProc checkMono checkSS checkNSS name representation p rP r rR =
  let {
    (_, nSp) = partition (\ foo -> isValue (checkSS foo)) p;
    (_, nSr) = partition (\ foo -> isValue (checkSS foo)) r;
  } in catcha
         (bind (bind (bind (bind (bind (catcha checkMono
 (\ x ->
   errora
     (shows_string
        ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'i', 't', 'h', ' ', 'm',
          'o', 'n', 'o', 't', 'o', 'n', 'i', 'c', 'i', 't', 'y', ' ', 'o', 'f',
          ' ', 's', 't', 'r', 'i', 'c', 't', ' ', 'o', 'r', 'd', 'e', 'r']
       . shows_nl . x)))
                                   (\ _ ->
                                     catcha
                                       (catcha (checkAllMI checkNSS nSr)
 (\ x -> errora (snd x)))
                                       (\ x ->
 errora
   (shows_string
      ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'h', 'e', 'n', ' ', 'o',
        'r', 'i', 'e', 'n', 't', 'i', 'n', 'g', ' ', 'T', 'R', 'S']
     . shows_nl . x))))
                             (\ _ ->
                               catcha
                                 (catcha (checkAllMI checkNSS nSp)
                                   (\ x -> errora (snd x)))
                                 (\ x ->
                                   errora
                                     (shows_string
['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'h', 'e', 'n', ' ', 'o', 'r', 'i',
  'e', 'n', 't', 'i', 'n', 'g', ' ', 'D', 'P', 's']
                                       . shows_nl . x))))
                       (\ _ ->
                         catcha (checkSubset rR r)
                           (\ _ ->
                             errora
                               (shows_string
                                 ['p', 'r', 'o', 'b', 'l', 'e', 'm', ':', ' ',
                                   'a', 'd', 'd', 'i', 't', 'i', 'o', 'n', 'a',
                                   'l', ' ']))))
                 (\ _ ->
                   catcha (checkSubset nSr rR)
                     (\ x ->
                       errora
                         (case checkSS x of {
                            Lefta err -> err;
                            Righta _ -> internal_error;
                          }))))
           (\ _ ->
             catcha (checkSubset nSp rP)
               (\ x ->
                 errora
                   (case checkSS x of {
                      Lefta err -> err;
                      Righta _ -> internal_error;
                    }))))
         (\ x ->
           errora
             (shows_string
                ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'a', 'p',
                  'p', 'l', 'y', ' ', 't', 'h', 'e', ' ', 'r', 'e', 'd', 'u',
                  'c', 't', 'i', 'o', 'n', ' ', 'p', 'a', 'i', 'r', ' ', 'p',
                  'r', 'o', 'c', 'e', 's', 's', 'o', 'r', ' ', 'w', 'i', 't',
                  'h', ' ', 't', 'h', 'e', ' ', 'f', 'o', 'l', 'l', 'o', 'w',
                  'i', 'n', 'g']
               . shows_nl . representation . shows_nl . x));

create_mono_redpair_proc_comp ::
  forall a b c.
    (Eq a, Shows a, Eq b, Shows b, Eq c,
      Shows c) => (a -> Bool,
                    (a, (a, (a -> a -> a,
                              (a -> a -> a,
                                (a -> a,
                                  (a -> a -> Bool,
                                    (a -> a -> Bool,
                                      (Bool, ([Char], a -> Bool))))))))))
                    -> (a -> Bool) ->
                         [((b, Nat), (a, [a]))] ->
                           [(Term b c, Term b c)] ->
                             [(Term b c, Term b c)] ->
                               [(Term b c, Term b c)] ->
                                 [(Term b c, Term b c)] ->
                                   Eithera ([Char] -> [Char]) ();
create_mono_redpair_proc_comp c cm i p rP r rR =
  let {
    pia = to_lpoly_inter (get_comp_1 c) i;
  } in bind (check_lpoly_coeffs (get_comp_wf_elem c) (get_comp_apos c)
              (\ x -> get_comp_ge c x (get_comp_0 c)) i)
         (\ _ ->
           checkMonoRedPairProc
             (check_poly_monoI (get_comp_0 c) (get_comp_ge c) cm i)
             (check_polo_sI (get_comp_mono c) (get_comp_0 c) (get_comp_1 c)
               (get_comp_plus c) (get_comp_times c) (get_comp_max0 c)
               (get_comp_ge c) (get_comp_gt c) pia)
             (check_polo_nsI (get_comp_0 c) (get_comp_1 c) (get_comp_plus c)
               (get_comp_times c) (get_comp_max0 c) (get_comp_ge c) pia)
             (shows_string
                ['p', 'o', 'l', 'y', 'n', 'o', 'm', 'i', 'a', 'l', ' ', 'o',
                  'r', 'd', 'e', 'r', ' ', '(']
               . shows_string (get_comp_description c) . shows_string [')'])
             (create_lpoly_repr c i) p rP r rR);

create_mono_redpair_proc ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => Redpair a ->
                    [(Term a b, Term a b)] ->
                      [(Term a b, Term a b)] ->
                        [(Term a b, Term a b)] ->
                          [(Term a b, Term a b)] ->
                            Eithera ([Char] -> [Char]) ();
create_mono_redpair_proc (Int_carrier i) =
  create_mono_redpair_proc_comp int_components int_mono i;
create_mono_redpair_proc (Nat_carrier i) =
  create_mono_redpair_proc_comp nat_components nat_mono i;
create_mono_redpair_proc (Rat_carrier delta i) =
  create_mono_redpair_proc_comp (rat_components delta) rat_mono i;
create_mono_redpair_proc (Arctic_carrier i) =
  (\ _ _ _ _ ->
    errora
      (shows_string
         ['a', 'r', 'c', 't', 'i', 'c', ' ', 'n', 'o', 't', ' ', 'm', 'o', 'n',
           'o', 't', 'o', 'n', 'i', 'c', ' ', 'f', 'o', 'r', ' ', 'T', 'R', 'S',
           's']
        . shows_nl));
create_mono_redpair_proc (Arctic_rat_carrier d i) =
  (\ _ _ _ _ ->
    errora
      (shows_string
         ['a', 'r', 'c', 't', 'i', 'c', ' ', 'n', 'o', 't', ' ', 'm', 'o', 'n',
           'o', 't', 'o', 'n', 'i', 'c', ' ', 'f', 'o', 'r', ' ', 'T', 'R', 'S',
           's']
        . shows_nl));
create_mono_redpair_proc (Arc_mat_carrier n i) =
  (\ _ _ _ _ ->
    errora
      (shows_string
         ['a', 'r', 'c', 't', 'i', 'c', ' ', 'n', 'o', 't', ' ', 'm', 'o', 'n',
           'o', 't', 'o', 'n', 'i', 'c', ' ', 'f', 'o', 'r', ' ', 'T', 'R', 'S',
           's']
        . shows_nl));
create_mono_redpair_proc (Arc_rat_mat_carrier d n i) =
  (\ _ _ _ _ ->
    errora
      (shows_string
         ['a', 'r', 'c', 't', 'i', 'c', ' ', 'n', 'o', 't', ' ', 'm', 'o', 'n',
           'o', 't', 'o', 'n', 'i', 'c', ' ', 'f', 'o', 'r', ' ', 'T', 'R', 'S',
           's']
        . shows_nl));
create_mono_redpair_proc (Int_mat_carrier n sd i) =
  create_mono_redpair_proc_comp (mat_components n sd int_components)
    (mat_mono sd int_mono) i;
create_mono_redpair_proc (Rat_mat_carrier d n sd i) =
  create_mono_redpair_proc_comp (mat_components n sd (rat_components d))
    (mat_mono sd rat_mono) i;
create_mono_redpair_proc (Lpo pri pia) =
  (\ p rP r rR ->
    let {
      m = maxa (get_max_arity r) (get_max_arity p);
      pr = lpo_repr_to_pr pri;
      piaa = afs_list_to_afs pia;
    } in bind (check_wf_afs pia)
           (\ _ ->
             checkMonoRedPairProc (check_mono_afs pia)
               (afs_check lpoS piaa (lpo_strict pr m))
               (afs_check lpoNS piaa (lpo_nstrict pr m)) lpo_repr
               (shows_lpo_af pri pia) p rP r rR));

checkAllMIndex ::
  forall a b.
    (Nat -> a -> Nat) -> (Nat -> a -> Eithera b ()) -> [a] -> Eithera b ();
checkAllMIndex g f xs =
  snd (foldla (\ (n, mon) x -> (g n x, bind mon (\ _ -> f n x)))
        (one_nat, returna ()) xs);

checkDepGraphProc ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => [(Term a b, Term a b)] ->
                    [(Bool, [(Term a b, Term a b)])] ->
                      Maybe (a -> [(Term a b, Term a b)]) ->
                        Maybe (a -> [(Term a b, Term a b)]) ->
                          Eithera ([Char] -> [Char]) ();
checkDepGraphProc p dps rm rrm =
  bind (catcha (checkSubset p (concat_map snd dps))
         (\ x ->
           errora
             (shows_string
                ['D', 'e', 'p', 'e', 'n', 'd', 'e', 'n', 'c', 'y', ' ', 'P',
                  'a', 'i', 'r', ' ']
               . shows_rule x .
                   shows_string
                     [' ', 'i', 's', ' ', 'm', 'i', 's', 's', 'i', 'n', 'g',
                       ' ', 'i', 'n', ' ', 'd', 'e', 'c', 'o', 'm', 'p', 'o',
                       's', 'i', 't', 'i', 'o', 'n']
                     . shows_nl)))
    (\ _ ->
      catcha
        (check_graph_decomp (showsa . fst)
          (\ (a, b) ->
            let {
              (aa, ba) = a;
            } in let {
                   (_, t) = aa;
                 } in (\ (_, ct) (ab, bb) ->
                        let {
                          (u, _) = ab;
                        } in (\ (cu, _) -> gcMatchI ct u && gcMatchI cu t)
                          bb)
                   ba
              b)
          (mapa (\ (real, cs) ->
                  (real,
                    mapa (\ (s, t) -> ((s, t), (capRM rrm s, capRM rm t))) cs))
            dps))
        (\ x ->
          errora
            (shows_string
               ['o', 'u', 'r', ' ', 'e', 's', 't', 'i', 'm', 'a', 't', 'i', 'o',
                 'n', ' ', '(', 'E', 'D', 'G', '*', '*', '*', ')', ' ', 'c',
                 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 's', 'h', 'o',
                 'w', ' ', 't', 'h', 'a', 't', ' ', 'y', 'o', 'u', ' ', 'g',
                 'a', 'v', 'e', ' ', 'a', ' ', 'v', 'a', 'l', 'i', 'd', ' ',
                 'd', 'e', 'c', 'o', 'm', 'p', 'o', 's', 'i', 't', 'i', 'o',
                 'n', ' ', 'd', 'u', 'e', ' ', 't', 'o', ' ', 't', 'h', 'e',
                 ' ', 'f', 'o', 'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'r',
                 'e', 'a', 's', 'o', 'n']
              . shows_nl . x)));

checkRedPairUrAfProc ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => ((Term a b, Term a b) -> Eithera ([Char] -> [Char]) ()) ->
                    ((Term a b, Term a b) -> Eithera ([Char] -> [Char]) ()) ->
                      ([Char] -> [Char]) ->
                        ([Char] -> [Char]) ->
                          [(Term a b, Term a b)] ->
                            [(Term a b, Term a b)] ->
                              [(Term a b, Term a b)] ->
                                Maybe (a -> [(Term a b, Term a b)]) ->
                                  [(Term a b, Term a b)] ->
                                    (a -> Nat -> Nat -> Bool) ->
                                      Eithera ([Char] -> [Char]) ();
checkRedPairUrAfProc checkSS checkNSS name representation p rP r rm ur pia =
  let {
    (_, nSp) = partition (\ foo -> isValue (checkSS foo)) p;
  } in bind (bind (bind (catcha (check_ur_P_closed_rm_af r rm ur p pia)
                          (\ x ->
                            errora
                              (shows_string
                                 ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w',
                                   'i', 't', 'h', ' ', 'c', 'l', 'o', 's', 'u',
                                   'r', 'e', ' ', 'p', 'r', 'o', 'p', 'e', 'r',
                                   't', 'i', 'e', 's', ' ', 'o', 'f', ' ', 'u',
                                   's', 'a', 'b', 'l', 'e', ' ', 'r', 'u', 'l',
                                   'e', 's', ' ', 'w', '.', 'r', '.', 't', '.',
                                   ' ', 'a', 'r', 'g', 'u', 'm', 'e', 'n', 't',
                                   ' ', 'f', 'i', 'l', 't', 'e', 'r']
                                . shows_nl . x)))
                    (\ _ ->
                      catcha
                        (catcha (checkAllMI checkNSS ur)
                          (\ x -> errora (snd x)))
                        (\ x ->
                          errora
                            (shows_string
                               ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w',
                                 'h', 'e', 'n', ' ', 'o', 'r', 'i', 'e', 'n',
                                 't', 'i', 'n', 'g', ' ', 'u', 's', 'a', 'b',
                                 'l', 'e', ' ', 'r', 'u', 'l', 'e', 's']
                              . shows_nl . x))))
              (\ _ ->
                catcha
                  (catcha (checkAllMI checkNSS nSp) (\ x -> errora (snd x)))
                  (\ x ->
                    errora
                      (shows_string
                         ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'h', 'e',
                           'n', ' ', 'o', 'r', 'i', 'e', 'n', 't', 'i', 'n',
                           'g', ' ', 'D', 'P', 's']
                        . shows_nl . x))))
         (\ _ ->
           catcha
             (catcha (checkSubset nSp rP)
               (\ x ->
                 errora
                   (case checkSS x of {
                      Lefta err -> err;
                      Righta _ -> internal_error;
                    })))
             (\ x ->
               errora
                 (shows_string
                    ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'a', 'p',
                      'p', 'l', 'y', ' ', 't', 'h', 'e', ' ', 'r', 'e', 'd',
                      'u', 'c', 't', 'i', 'o', 'n', ' ', 'p', 'a', 'i', 'r',
                      ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r', ' ',
                      'w', 'i', 't', 'h', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                      'l', 'l', 'o', 'w', 'i', 'n', 'g']
                   . shows_nl .
                       representation .
                         shows_nl .
                           shows_string
                             ['a', 'n', 'd', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                               'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'u', 's',
                               'a', 'b', 'l', 'e', ' ', 'r', 'u', 'l', 'e', 's']
                             . shows_nl . shows_rules ur . shows_nl . x)));

create_redpair_ur_af_proc_comp ::
  forall a b c.
    (Eq a, Shows a, Eq b, Shows b, Eq c,
      Shows c) => (a -> Bool,
                    (a, (a, (a -> a -> a,
                              (a -> a -> a,
                                (a -> a,
                                  (a -> a -> Bool,
                                    (a -> a -> Bool,
                                      (Bool, ([Char], a -> Bool))))))))))
                    -> [((b, Nat), (a, [a]))] ->
                         [(Term b c, Term b c)] ->
                           [(Term b c, Term b c)] ->
                             [(Term b c, Term b c)] ->
                               Maybe (b -> [(Term b c, Term b c)]) ->
                                 [(Term b c, Term b c)] ->
                                   Eithera ([Char] -> [Char]) ();
create_redpair_ur_af_proc_comp c i p rP r rm ur =
  let {
    pia = to_lpoly_inter (get_comp_1 c) i;
  } in bind (check_lpoly_coeffs (get_comp_wf_elem c) (get_comp_apos c)
              (\ x -> get_comp_ge c x (get_comp_0 c)) i)
         (\ _ ->
           checkRedPairUrAfProc
             (check_polo_sI (get_comp_mono c) (get_comp_0 c) (get_comp_1 c)
               (get_comp_plus c) (get_comp_times c) (get_comp_max0 c)
               (get_comp_ge c) (get_comp_gt c) pia)
             (check_polo_nsI (get_comp_0 c) (get_comp_1 c) (get_comp_plus c)
               (get_comp_times c) (get_comp_max0 c) (get_comp_ge c) pia)
             (shows_string
                ['p', 'o', 'l', 'y', 'n', 'o', 'm', 'i', 'a', 'l', ' ', 'o',
                  'r', 'd', 'e', 'r', ' ', '(']
               . shows_string (get_comp_description c) . shows_string [')'])
             (create_lpoly_repr c i) p rP r rm ur
             (create_afI (get_comp_0 c) i));

create_redpair_ur_af_proc ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => Redpair a ->
                    [(Term a b, Term a b)] ->
                      [(Term a b, Term a b)] ->
                        [(Term a b, Term a b)] ->
                          Maybe (a -> [(Term a b, Term a b)]) ->
                            [(Term a b, Term a b)] ->
                              Eithera ([Char] -> [Char]) ();
create_redpair_ur_af_proc (Int_carrier i) =
  create_redpair_ur_af_proc_comp int_components i;
create_redpair_ur_af_proc (Nat_carrier i) =
  create_redpair_ur_af_proc_comp nat_components i;
create_redpair_ur_af_proc (Rat_carrier delta i) =
  create_redpair_ur_af_proc_comp (rat_components delta) i;
create_redpair_ur_af_proc (Arctic_carrier i) =
  create_redpair_ur_af_proc_comp arctic_components i;
create_redpair_ur_af_proc (Arctic_rat_carrier delta i) =
  create_redpair_ur_af_proc_comp (arctic_rat_components delta) i;
create_redpair_ur_af_proc (Int_mat_carrier n sd i) =
  create_redpair_ur_af_proc_comp (mat_components n sd int_components) i;
create_redpair_ur_af_proc (Rat_mat_carrier delta n sd i) =
  create_redpair_ur_af_proc_comp (mat_components n sd (rat_components delta)) i;
create_redpair_ur_af_proc (Arc_mat_carrier n i) =
  create_redpair_ur_af_proc_comp (mat_arctic_components n arctic_components) i;
create_redpair_ur_af_proc (Arc_rat_mat_carrier delta n i) =
  create_redpair_ur_af_proc_comp
    (mat_arctic_components n (arctic_rat_components delta)) i;
create_redpair_ur_af_proc (Lpo pri pia) =
  (\ p rP r rm ur ->
    let {
      m = maxa (get_max_arity r) (get_max_arity p);
      pr = lpo_repr_to_pr pri;
      piaa = afs_list_to_afs pia;
    } in bind (check_wf_afs pia)
           (\ _ ->
             checkRedPairUrAfProc (afs_check lpoS piaa (lpo_strict pr m))
               (afs_check lpoNS piaa (lpo_nstrict pr m)) lpo_repr
               (shows_lpo_af pri pia) p rP r rm ur (afs_list_to_af pia)));

create_redpair_proc_comp ::
  forall a b c.
    (Eq a, Shows a, Eq b, Shows b, Eq c,
      Shows c) => (a -> Bool,
                    (a, (a, (a -> a -> a,
                              (a -> a -> a,
                                (a -> a,
                                  (a -> a -> Bool,
                                    (a -> a -> Bool,
                                      (Bool, ([Char], a -> Bool))))))))))
                    -> [((b, Nat), (a, [a]))] ->
                         [(Term b c, Term b c)] ->
                           [(Term b c, Term b c)] ->
                             [(Term b c, Term b c)] ->
                               Eithera ([Char] -> [Char]) ();
create_redpair_proc_comp c i p rP r =
  let {
    pia = to_lpoly_inter (get_comp_1 c) i;
  } in bind (check_lpoly_coeffs (get_comp_wf_elem c) (get_comp_apos c)
              (\ x -> get_comp_ge c x (get_comp_0 c)) i)
         (\ _ ->
           checkRedPairProc
             (check_polo_sI (get_comp_mono c) (get_comp_0 c) (get_comp_1 c)
               (get_comp_plus c) (get_comp_times c) (get_comp_max0 c)
               (get_comp_ge c) (get_comp_gt c) pia)
             (check_polo_nsI (get_comp_0 c) (get_comp_1 c) (get_comp_plus c)
               (get_comp_times c) (get_comp_max0 c) (get_comp_ge c) pia)
             (shows_string
                ['p', 'o', 'l', 'y', 'n', 'o', 'm', 'i', 'a', 'l', ' ', 'o',
                  'r', 'd', 'e', 'r', ' ', '(']
               . shows_string (get_comp_description c) . shows_string [')'])
             (create_lpoly_repr c i) p rP r);

create_redpair_proc ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => Redpair a ->
                    [(Term a b, Term a b)] ->
                      [(Term a b, Term a b)] ->
                        [(Term a b, Term a b)] -> Eithera ([Char] -> [Char]) ();
create_redpair_proc (Int_carrier i) = create_redpair_proc_comp int_components i;
create_redpair_proc (Nat_carrier i) = create_redpair_proc_comp nat_components i;
create_redpair_proc (Rat_carrier delta i) =
  create_redpair_proc_comp (rat_components delta) i;
create_redpair_proc (Arctic_carrier i) =
  create_redpair_proc_comp arctic_components i;
create_redpair_proc (Arctic_rat_carrier d i) =
  create_redpair_proc_comp (arctic_rat_components d) i;
create_redpair_proc (Int_mat_carrier n sd i) =
  create_redpair_proc_comp (mat_components n sd int_components) i;
create_redpair_proc (Rat_mat_carrier delta n sd i) =
  create_redpair_proc_comp (mat_components n sd (rat_components delta)) i;
create_redpair_proc (Arc_mat_carrier n i) =
  create_redpair_proc_comp (mat_arctic_components n arctic_components) i;
create_redpair_proc (Arc_rat_mat_carrier d n i) =
  create_redpair_proc_comp (mat_arctic_components n (arctic_rat_components d))
    i;
create_redpair_proc (Lpo pri pia) =
  (\ p rP r ->
    let {
      m = maxa (get_max_arity r) (get_max_arity p);
      pr = lpo_repr_to_pr pri;
      piaa = afs_list_to_afs pia;
    } in bind (check_wf_afs pia)
           (\ _ ->
             checkRedPairProc (afs_check lpoS piaa (lpo_strict pr m))
               (afs_check lpoNS piaa (lpo_nstrict pr m)) lpo_repr
               (shows_lpo_af pri pia) p rP r));

checkDPProof ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => ([Char] -> [Char]) ->
                    [(Term (Lab a b) [Char], Term (Lab a b) [Char])] ->
                      [(Term (Lab a b) [Char], Term (Lab a b) [Char])] ->
                        Nat ->
                          Maybe (Lab a b ->
                                  [(Term (Lab a b) [Char],
                                     Term (Lab a b) [Char])])
                            -> Maybe (Lab a b ->
                                       [(Term (Lab a b) [Char],
  Term (Lab a b) [Char])])
                                 -> DPProof a b [Char] ->
                                      Eithera ([Char] -> [Char]) ();
checkDPProof i p r m rm rrm PisEmpty =
  debug (i []) ['P', ' ', 'i', 's', ' ', 'e', 'm', 'p', 't', 'y']
    (case p of {
       [] -> returna ();
       _ : _ ->
         errora
           (i . shows_string
                  [':', ' ', 'P', ' ', 'i', 's', ' ', 'n', 'o', 't', ' ', 'e',
                    'm', 'p', 't', 'y', ' ', 'i', 'n', ' ', 't', 'h', 'e', ' ',
                    'f', 'o', 'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'D', 'P',
                    '-', 'p', 'r', 'o', 'b', 'l', 'e', 'm', ' ']
                  . shows_nl . shows_dpp p r);
     });
checkDPProof i p r m rm rrm (RedPairProc redp rP prf) =
  debug (i []) ['R', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o', 'c']
    (bind (catcha (create_redpair_proc redp p rP r)
            (\ x ->
              errora
                (i . shows_string
                       [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e',
                         'n', ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ',
                         't', 'h', 'e', ' ', 'r', 'e', 'd', 'u', 'c', 't', 'i',
                         'o', 'n', ' ', 'p', 'a', 'i', 'r', ' ', 'p', 'r', 'o',
                         'c', 'e', 's', 's', 'o', 'r', ' ', 't', 'o', ' ', 's',
                         'w', 'i', 't', 'c', 'h', ' ', 'f', 'r', 'o', 'm']
                       . shows_nl .
                           shows_dpp p r .
                             shows_nl .
                               shows_string
                                 ['t', 'o', ' ', 't', 'h', 'e', ' ', 'D', 'P',
                                   ' ', 'p', 'r', 'o', 'b', 'l', 'e', 'm']
                                 . shows_nl . shows_dpp rP r . shows_nl . x)))
      (\ _ ->
        catcha (checkDPProof (i . shows_string ['.', '1']) rP r m rm rrm prf)
          (\ x ->
            errora
              (i . shows_string
                     [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l',
                       'o', 'w', ' ', 't', 'h', 'e', ' ', 'r', 'e', 'd', 'u',
                       'c', 't', 'i', 'o', 'n', ' ', 'p', 'a', 'i', 'r', ' ',
                       'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
                     . shows_nl . indent x))));
checkDPProof i p r m rm rrm (RedPairUrProc redp rP ur prf) =
  debug (i [])
    ['R', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o', 'c', ' ', '(', 'u', 's',
      'a', 'b', 'l', 'e', ' ', 'r', 'u', 'l', 'e', 's', ' ', '+', ' ', 'a', 'f',
      ')']
    (bind (catcha (create_redpair_ur_af_proc redp p rP r rm ur)
            (\ x ->
              errora
                (i . shows_string
                       [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e',
                         'n', ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ',
                         't', 'h', 'e', ' ', 'r', 'e', 'd', 'u', 'c', 't', 'i',
                         'o', 'n', ' ', 'p', 'a', 'i', 'r', ' ', 'p', 'r', 'o',
                         'c', 'e', 's', 's', 'o', 'r', ' ', '(', 'w', 'i', 't',
                         'h', ' ', 'u', 's', 'a', 'b', 'l', 'e', ' ', 'r', 'u',
                         'l', 'e', 's', ' ', 'a', 'n', 'd', ' ', 'a', 'f', ')',
                         ' ', 't', 'o', ' ', 's', 'w', 'i', 't', 'c', 'h', ' ',
                         'f', 'r', 'o', 'm', ' ']
                       . shows_nl .
                           shows_dpp p r .
                             shows_nl .
                               shows_string
                                 ['t', 'o', ' ', 't', 'h', 'e', ' ', 'D', 'P',
                                   ' ', 'p', 'r', 'o', 'b', 'l', 'e', 'm', ' ']
                                 . shows_nl . shows_dpp rP r . shows_nl . x)))
      (\ _ ->
        catcha (checkDPProof (i . shows_string ['.', '1']) rP r m rm rrm prf)
          (\ x ->
            errora
              (i . shows_string
                     [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l',
                       'o', 'w', ' ', 't', 'h', 'e', ' ', 'r', 'e', 'd', 'u',
                       'c', 't', 'i', 'o', 'n', ' ', 'p', 'a', 'i', 'r', ' ',
                       'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
                     . shows_nl . indent x))));
checkDPProof i p r m rm rrm (DepGraphProc edpts) =
  debug (i []) ['D', 'e', 'p', 'G', 'r', 'a', 'p', 'h', 'P', 'r', 'o', 'c']
    (bind (catcha
            (checkDepGraphProc p
              (mapa (\ x ->
                      (case fst x of {
                         Nothing -> False;
                         Just _ -> True;
                       },
                        snd x))
                edpts)
              rm rrm)
            (\ x ->
              errora
                (i . shows_string
                       [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'i',
                         'l', 'e', ' ', 't', 'r', 'y', 'i', 'n', 'g', ' ', 't',
                         'o', ' ', 'p', 'e', 'r', 'f', 'o', 'r', 'm', ' ', 'S',
                         'C', 'C', '-', 'd', 'e', 'c', 'o', 'm', 'p', 'o', 's',
                         'i', 't', 'i', 'o', 'n', ' ', ' ', 'o', 'n', ' ']
                       . shows_nl . shows_dpp p r . shows_nl . x)))
      (\ _ ->
        catcha
          (checkAllMIndex
            (\ j rp -> case fst rp of {
                         Nothing -> j;
                         Just _ -> Suc j;
                       })
            (\ j rp ->
              case fst rp of {
                Nothing -> returna ();
                Just a ->
                  checkDPProof (i . shows_string ['.'] . shows_nat j) (snd rp) r
                    m rm rrm a;
              })
            edpts)
          (\ x ->
            errora
              (i . shows_string
                     [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l',
                       'o', 'w', ' ', 't', 'h', 'e', ' ', 'd', 'e', 'p', 'e',
                       'n', 'd', 'e', 'n', 'c', 'y', ' ', 'g', 'r', 'a', 'p',
                       'h', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
                     . shows_nl . indent x))));
checkDPProof i p r m rm rrm (MonoRedPairProc redp rP rR prf) =
  debug (i [])
    ['M', 'o', 'n', 'o', 'R', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o', 'c']
    (bind (catcha (create_mono_redpair_proc redp p rP r rR)
            (\ x ->
              errora
                (i . shows_string
                       [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e',
                         'n', ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ',
                         't', 'h', 'e', ' ', 'm', 'o', 'n', 'o', 't', 'o', 'n',
                         'i', 'c', ' ', 'r', 'e', 'd', 'u', 'c', 't', 'i', 'o',
                         'n', ' ', 'p', 'a', 'i', 'r', ' ', 'p', 'r', 'o', 'c',
                         'e', 's', 's', 'o', 'r', ' ', 't', 'o', ' ', 's', 'w',
                         'i', 't', 'c', 'h', ' ', 'f', 'r', 'o', 'm', ' ']
                       . shows_nl .
                           shows_dpp p r .
                             shows_nl .
                               shows_string
                                 ['t', 'o', ' ', 't', 'h', 'e', ' ', 'D', 'P',
                                   ' ', 'p', 'r', 'o', 'b', 'l', 'e', 'm', ' ']
                                 . shows_nl . shows_dpp rP rR . shows_nl . x)))
      (\ _ ->
        catcha
          (checkDPProof (i . shows_string ['.', '1']) rP rR m
            (computeRuleMap rR) (computeRuleMap (reverse_trs rR)) prf)
          (\ x ->
            errora
              (i . shows_string
                     [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l',
                       'o', 'w', ' ', 't', 'h', 'e', ' ', 'm', 'o', 'n', 'o',
                       't', 'o', 'n', 'i', 'c', ' ', 'r', 'e', 'd', 'u', 'c',
                       't', 'i', 'o', 'n', ' ', 'p', 'a', 'i', 'r', ' ', 'p',
                       'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
                     . shows_nl . indent x))));
checkDPProof i p r m rm rrm (MonoRedPairUrProc redp rP rR ur prf) =
  debug (i [])
    ['M', 'o', 'n', 'o', 'R', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o', 'c',
      ' ', '(', 'u', 's', 'a', 'b', 'l', 'e', ' ', 'r', 'u', 'l', 'e', 's', ')']
    (bind (catcha (create_mono_redpair_ur_proc redp p rP r rm rR ur)
            (\ x ->
              errora
                (i . shows_string
                       [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e',
                         'n', ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ',
                         't', 'h', 'e', ' ', 'm', 'o', 'n', 'o', 't', 'o', 'n',
                         'i', 'c', ' ', 'r', 'e', 'd', 'u', 'c', 't', 'i', 'o',
                         'n', ' ', 'p', 'a', 'i', 'r', ' ', 'p', 'r', 'o', 'c',
                         'e', 's', 's', 'o', 'r', ' ', '(', 'w', 'i', 't', 'h',
                         ' ', 'u', 's', 'a', 'b', 'l', 'e', ' ', 'r', 'u', 'l',
                         'e', 's', ')', ' ', 't', 'o', ' ', 's', 'w', 'i', 't',
                         'c', 'h', ' ', 'f', 'r', 'o', 'm', ' ']
                       . shows_nl .
                           shows_dpp p r .
                             shows_nl .
                               shows_string
                                 ['t', 'o', ' ', 't', 'h', 'e', ' ', 'D', 'P',
                                   ' ', 'p', 'r', 'o', 'b', 'l', 'e', 'm']
                                 . shows_nl . shows_dpp rP rR . shows_nl . x)))
      (\ _ ->
        catcha
          (checkDPProof (i . shows_string ['.', '1']) rP rR m
            (computeRuleMap rR) (computeRuleMap (reverse_trs rR)) prf)
          (\ x ->
            errora
              (i . shows_string
                     [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l',
                       'o', 'w', ' ', 't', 'h', 'e', ' ', 'm', 'o', 'n', 'o',
                       't', 'o', 'n', 'i', 'c', ' ', 'r', 'e', 'd', 'u', 'c',
                       't', 'i', 'o', 'n', ' ', 'p', 'a', 'i', 'r', ' ', '(',
                       'w', 'i', 't', 'h', ' ', 'u', 's', 'a', 'b', 'l', 'e',
                       ' ', 'r', 'u', 'l', 'e', 's', ')', ' ', 'p', 'r', 'o',
                       'c', 'e', 's', 's', 'o', 'r']
                     . shows_nl . indent x))));
checkDPProof i p r (Suc m) rm rrm (UnlabProc ulP ulR prf) =
  debug (i []) ['U', 'n', 'l', 'a', 'b', 'P', 'r', 'o', 'c']
    (bind (check_unlab_proc p r ulP ulR)
      (\ _ ->
        catcha
          (checkDPProof (i . shows_string ['.', '1']) ulP ulR m
            (computeRuleMap ulR) (computeRuleMap (reverse_trs ulR)) prf)
          (\ x ->
            errora
              (i . shows_string
                     [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l',
                       'o', 'w', ' ', 't', 'h', 'e', ' ', 'u', 'n', 'l', 'a',
                       'b', 'e', 'l', 'i', 'n', 'g', ' ', 'p', 'r', 'o', 'c',
                       'e', 's', 's', 'o', 'r']
                     . shows_nl . indent x))));
checkDPProof i p r Zero_nat rm rrm (UnlabProc ulP ulR prf) =
  errora
    (shows_string
       ['u', 'n', 'l', 'a', 'b', 'e', 'l', 'i', 'n', 'g', ' ', 'n', 'o', 't',
         ' ', 'a', 'l', 'l', 'o', 'w', 'e', 'd', ' ', 'd', 'u', 'e', ' ', 't',
         'o', ' ', 'l', 'a', 'b', 'e', 'l', '-', 'd', 'e', 'p', 't', 'h', ' ',
         'o', 'f', ' ', '0']
      . shows_nl);
checkDPProof i p r m rm rrm (SemlabProc sli l lP lR prf) =
  debug (i []) ['S', 'e', 'm', 'l', 'a', 'b', 'P', 'r', 'o', 'c']
    (bind (catcha (create_sem_lab_proc sli l p r lP lR)
            (\ x ->
              errora
                (i . shows_string
                       [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e',
                         'n', ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ',
                         't', 'h', 'e', ' ', 's', 'e', 'm', 'a', 'n', 't', 'i',
                         'c', ' ', 'l', 'a', 'b', 'e', 'l', 'i', 'n', 'g', ' ',
                         'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
                       . shows_nl . x)))
      (\ _ ->
        catcha
          (checkDPProof (i . shows_string ['.', '1']) lP lR (Suc m)
            (computeRuleMap lR) (computeRuleMap (reverse_trs lR)) prf)
          (\ x ->
            errora
              (i . shows_string
                     [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l',
                       'o', 'w', ' ', 't', 'h', 'e', ' ', 's', 'e', 'm', 'a',
                       'n', 't', 'i', 'c', ' ', 'l', 'a', 'b', 'e', 'l', 'i',
                       'n', 'g', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o',
                       'r']
                     . shows_nl . indent x))));
checkDPProof i p r m rm rrm (RootlabProc lP lR prf) =
  debug (i []) ['R', 'o', 'o', 't', 'l', 'a', 'b', 'P', 'r', 'o', 'c']
    (bind (catcha (rl_proc p r lP lR)
            (\ x ->
              errora
                (i . shows_string
                       [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e',
                         'n', ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ',
                         't', 'h', 'e', ' ', 'r', 'o', 'o', 't', ' ', 'l', 'a',
                         'b', 'e', 'l', 'i', 'n', 'g', ' ', 'p', 'r', 'o', 'c',
                         'e', 's', 's', 'o', 'r']
                       . shows_nl . x)))
      (\ _ ->
        catcha
          (checkDPProof (i . shows_string ['.', '1']) lP lR (Suc m)
            (computeRuleMap lR) (computeRuleMap (reverse_trs lR)) prf)
          (\ x ->
            errora
              (i . shows_string
                     [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l',
                       'o', 'w', ' ', 't', 'h', 'e', ' ', 'r', 'o', 'o', 't',
                       '-', 'l', 'a', 'b', 'e', 'l', 'i', 'n', 'g', ' ', 'p',
                       'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
                     . shows_nl . indent x))));
checkDPProof i p r m rm rrm (SpscProc sp rP prf) =
  debug (i []) ['S', 'u', 'b', 't', 'e', 'r', 'm', 'P', 'r', 'o', 'c']
    (bind (catcha (create_spsc_proc sp p rP r rm)
            (\ x ->
              errora
                (i . shows_string
                       [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e',
                         'n', ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ',
                         't', 'h', 'e', ' ', 's', 'u', 'b', 't', 'e', 'r', 'm',
                         ' ', 'c', 'r', 'i', 't', 'e', 'r', 'i', 'o', 'n', ' ',
                         'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
                       . shows_nl . x)))
      (\ _ ->
        catcha (checkDPProof (i . shows_string ['.', '1']) rP r m rm rrm prf)
          (\ x ->
            errora
              (i . shows_string
                     [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l',
                       'o', 'w', ' ', 't', 'h', 'e', ' ', 's', 'u', 'b', 't',
                       'e', 'r', 'm', ' ', 'c', 'r', 'i', 't', 'e', 'r', 'i',
                       'o', 'n', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o',
                       'r']
                     . shows_nl . indent x))));
checkDPProof i p r m rm rrm (SizeChangeSubTerm graphs) =
  debug (i [])
    ['S', 'i', 'z', 'e', 'C', 'h', 'a', 'n', 'g', 'e', 'S', 'u', 'b', 'T', 'e',
      'r', 'm']
    (check_sct_subterm_proc p graphs rm rrm);
checkDPProof i p r m rm rrm (SizeChangeRedPair redp ur graphs) =
  debug (i [])
    ['S', 'i', 'z', 'e', 'C', 'h', 'a', 'n', 'g', 'e', 'R', 'e', 'd', 'P', 'a',
      'i', 'r']
    (create_sct_ur_af_proc redp p rm rrm ur graphs);
checkDPProof i p r m rm rrm (FccProc f fcs rP rR prf) =
  debug (i []) ['F', 'c', 'c', 'P', 'r', 'o', 'c']
    (bind (catcha (fcc_proc p r f fcs rP rR)
            (\ x ->
              errora
                (i . shows_string
                       [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e',
                         'n', ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ',
                         't', 'h', 'e', ' ', 'c', 'l', 'o', 's', 'u', 'r', 'e',
                         ' ', 'u', 'n', 'd', 'e', 'r', ' ', 'f', 'l', 'a', 't',
                         ' ', 'c', 'o', 'n', 't', 'e', 'x', 't', 's']
                       . shows_nl . x)))
      (\ _ ->
        catcha
          (checkDPProof (i . shows_string ['.', '1']) rP rR Zero_nat
            (computeRuleMap rR) (computeRuleMap (reverse_trs rR)) prf)
          (\ x ->
            errora
              (i . shows_string
                     [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l',
                       'o', 'w', ' ', 'f', 'l', 'a', 't', ' ', 'c', 'o', 'n',
                       't', 'e', 'x', 't', ' ', 'c', 'l', 'o', 's', 'u', 'r',
                       'e', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
                     . shows_nl . indent x))));

checkTRSProof ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => ([Char] -> [Char]) ->
                    [(Term (Lab a b) [Char], Term (Lab a b) [Char])] ->
                      Nat ->
                        TRSProof a b [Char] -> Eithera ([Char] -> [Char]) ();
checkTRSProof i r m (DPTrans p prf) =
  debug (i []) ['D', 'P', ' ', 't', 'r', 'a', 'n', 's']
    (bind (catcha (check_DP r p)
            (\ x ->
              errora
                (i . shows_string
                       [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e',
                         'n', ' ', 's', 'w', 'i', 't', 'c', 'h', 'i', 'n', 'g',
                         ' ', 'f', 'r', 'o', 'm', ' ', 't', 'h', 'e', ' ', 'T',
                         'R', 'S']
                       . shows_nl .
                           shows_trs r .
                             shows_nl .
                               shows_string
                                 ['t', 'o', ' ', 't', 'h', 'e', ' ', 'i', 'n',
                                   'i', 't', 'i', 'a', 'l', ' ', 'D', 'P', ' ',
                                   'p', 'r', 'o', 'b', 'l', 'e', 'm']
                                 . shows_dpp p (iD_list r) . shows_nl . x)))
      (\ _ ->
        catcha
          (checkDPProof (i . shows_string ['.', '1'])
            (mapa (map_rule init_lab) p) (mapa (map_rule init_lab) (iD_list r))
            Zero_nat (computeRuleMap (mapa (map_rule init_lab) (iD_list r)))
            (computeRuleMap
              (reverse_trs (mapa (map_rule init_lab) (iD_list r))))
            prf)
          (\ x ->
            errora
              (i . shows_string
                     [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l',
                       'o', 'w', ' ', 's', 'w', 'i', 't', 'c', 'h', ' ', 't',
                       'o', ' ', 'd', 'e', 'p', 'e', 'n', 'd', 'e', 'n', 'c',
                       'y', ' ', 'p', 'a', 'i', 'r', 's']
                     . shows_nl . indent x))));
checkTRSProof i r m (RuleRemoval rp rR prf) =
  debug (i []) ['r', 'u', 'l', 'e', ' ', 'r', 'e', 'm', 'o', 'v', 'a', 'l']
    (bind (catcha (create_rule_removal_proc rp r rR)
            (\ x ->
              errora
                (i . shows_string
                       [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'i', 'n', ' ',
                         'r', 'u', 'l', 'e', ' ', 'r', 'e', 'm', 'o', 'v', 'a',
                         'l', ' ', 'w', 'h', 'e', 'n', ' ', 's', 'w', 'i', 't',
                         'c', 'h', 'i', 'n', 'g', ' ', 'f', 'r', 'o', 'm', ' ',
                         't', 'h', 'e', ' ', 'T', 'R', 'S']
                       . shows_nl .
                           shows_trs r .
                             shows_nl .
                               shows_string
                                 ['t', 'o', ' ', 't', 'h', 'e', ' ', 'T', 'R',
                                   'S']
                                 . shows_nl . shows_trs rR . shows_nl . x)))
      (\ _ ->
        catcha (checkTRSProof (i . shows_string ['.', '1']) rR m prf)
          (\ x ->
            errora
              (i . shows_string
                     [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l',
                       'o', 'w', ' ', 't', 'h', 'e', ' ', 'r', 'u', 'l', 'e',
                       ' ', 'r', 'e', 'm', 'o', 'v', 'a', 'l', ' ', 'p', 'r',
                       'o', 'c', 'e', 's', 's', 'o', 'r']
                     . shows_nl . indent x))));
checkTRSProof i r (Suc m) (Unlab uR prf) =
  debug (i []) ['u', 'n', 'l', 'a', 'b', 'e', 'l']
    (bind (catcha (unlab_trs_tt r uR)
            (\ x ->
              errora
                (i . shows_string
                       [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'i', 'n', ' ',
                         'u', 'n', 'l', 'a', 'b', 'e', 'l', 'i', 'n', 'g', ' ',
                         'o', 'f', ' ', 'T', 'R', 'S', ' ', 'w', 'h', 'e', 'n',
                         ' ', 's', 'w', 'i', 't', 'c', 'h', 'i', 'n', 'g', ' ',
                         'f', 'r', 'o', 'm', ' ', 't', 'h', 'e', ' ', 'T', 'R',
                         'S']
                       . shows_nl .
                           shows_trs r .
                             shows_nl .
                               shows_string
                                 ['t', 'o', ' ', 't', 'h', 'e', ' ', 'T', 'R',
                                   'S']
                                 . shows_nl . shows_trs uR . shows_nl . x)))
      (\ _ ->
        catcha (checkTRSProof (i . shows_string ['.', '1']) uR m prf)
          (\ x ->
            errora
              (i . shows_string
                     [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l',
                       'o', 'w', ' ', 't', 'h', 'e', ' ', 'u', 'n', 'l', 'a',
                       'b', 'e', 'l', 'i', 'n', 'g']
                     . shows_nl . indent x))));
checkTRSProof i r Zero_nat (Unlab uR prf) =
  errora
    (shows_string
       ['u', 'n', 'l', 'a', 'b', 'e', 'l', 'i', 'n', 'g', ' ', 'n', 'o', 't',
         ' ', 'a', 'l', 'l', 'o', 'w', 'e', 'd', ' ', 'f', 'o', 'r', ' ', 'l',
         'a', 'b', 'e', 'l', ' ', 'd', 'e', 'p', 't', 'h', ' ', '0']
      . shows_nl);
checkTRSProof i r m RisEmpty =
  debug (i []) ['R', ' ', 'i', 's', ' ', 'e', 'm', 'p', 't', 'y']
    (checkB (nulla r)
      (i . shows_string
             [':', ' ', 'R', ' ', 'i', 's', ' ', 'n', 'o', 't', ' ', 'e', 'm',
               'p', 't', 'y', ',', ' ', 'b', 'u', 't', ' ', 'i', 't', ' ', 'i',
               's', ' ', 't', 'h', 'e', ' ', 'T', 'R', 'S']
             . shows_trs r . shows_nl));
checkTRSProof i r m (Semlab sli l lR prf) =
  debug (i []) ['S', 'e', 'm', 'l', 'a', 'b']
    (bind (catcha (create_sem_lab_tt sli l r lR)
            (\ x ->
              errora
                (i . shows_string
                       [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e',
                         'n', ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ',
                         's', 'e', 'm', 'a', 'n', 't', 'i', 'c', ' ', 'l', 'a',
                         'b', 'e', 'l', 'i', 'n', 'g']
                       . shows_nl . x)))
      (\ _ ->
        catcha (checkTRSProof (i . shows_string ['.', '1']) lR (Suc m) prf)
          (\ x ->
            errora
              (i . shows_string
                     [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l',
                       'o', 'w', ' ', 's', 'e', 'm', 'a', 'n', 't', 'i', 'c',
                       ' ', 'l', 'a', 'b', 'e', 'l', 'i', 'n', 'g']
                     . shows_nl . indent x))));
checkTRSProof i r m (Rootlab lR prf) =
  debug (i []) ['R', 'o', 'o', 't', 'l', 'a', 'b']
    (bind (catcha (rl_tt r lR)
            (\ x ->
              errora
                (i . shows_string
                       [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e',
                         'n', ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ',
                         'r', 'o', 'o', 't', '-', 'l', 'a', 'b', 'e', 'l', 'i',
                         'n', 'g']
                       . shows_nl . x)))
      (\ _ ->
        catcha (checkTRSProof (i . shows_string ['.', '1']) lR (Suc m) prf)
          (\ x ->
            errora
              (i . shows_string
                     [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l',
                       'o', 'w', ' ', 'r', 'o', 'o', 't', '-', 'l', 'a', 'b',
                       'e', 'l', 'i', 'n', 'g']
                     . shows_nl . indent x))));
checkTRSProof i r m (Fcc fcs rR prf) =
  debug (i []) ['F', 'c', 'c']
    (bind (catcha (fcc_tt r fcs rR)
            (\ x ->
              errora
                (i . shows_string
                       [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e',
                         'n', ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ',
                         'f', 'l', 'a', 't', ' ', 'c', 'o', 'n', 't', 'e', 'x',
                         't', ' ', 'c', 'l', 'o', 's', 'u', 'r', 'e']
                       . shows_nl . x)))
      (\ _ ->
        catcha (checkTRSProof (i . shows_string ['.', '1']) rR Zero_nat prf)
          (\ x ->
            errora
              (i . shows_string
                     [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l',
                       'o', 'w', ' ', 'f', 'l', 'a', 't', ' ', 'c', 'o', 'n',
                       't', 'e', 'x', 't', ' ', 'c', 'l', 'o', 's', 'u', 'r',
                       'e']
                     . shows_nl . indent x))));
checkTRSProof i r m (StringReversal rR prf) =
  debug (i [])
    ['S', 't', 'r', 'i', 'n', 'g', ' ', 'R', 'e', 'v', 'e', 'r', 's', 'a', 'l']
    (bind (catcha (check_string_rev r rR)
            (\ x ->
              errora
                (i . shows_string
                       [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e',
                         'n', ' ', 'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ',
                         's', 't', 'r', 'i', 'n', 'g', ' ', 'r', 'e', 'v', 'e',
                         'r', 's', 'a', 'l']
                       . shows_nl . x)))
      (\ _ ->
        catcha (checkTRSProof (i . shows_string ['.', '1']) rR Zero_nat prf)
          (\ x ->
            errora
              (i . shows_string
                     [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l',
                       'o', 'w', ' ', 's', 't', 'r', 'i', 'n', 'g', ' ', 'r',
                       'e', 'v', 'e', 'r', 's', 'a', 'l']
                     . shows_nl . indent x))));

certifyCertProblem ::
  forall a b.
    (Eq a, Shows a, Eq b,
      Shows b) => CertificationProblem a b [Char] -> CertificationResult;
certifyCertProblem (CertificationProblem True trs (Lefta prf)) =
  case checkTRSProof (shows_string ['1']) (mapa (map_rule init_lab) trs)
         Zero_nat prf
    of {
    Lefta err ->
      Error ((shows_string
                ['t', 'e', 'r', 'm', 'i', 'n', 'a', 't', 'i', 'o', 'n', ' ',
                  'p', 'r', 'o', 'o', 'f', ' ', 'n', 'o', 't', ' ', 'a', 'c',
                  'c', 'e', 'p', 't', 'e', 'd']
               . shows_nl . err)
              []);
    Righta _ -> Certified True;
  };
certifyCertProblem (CertificationProblem False trs (Righta prf)) =
  case checkTRSDisproof (shows_string ['1']) trs prf of {
    Lefta err ->
      Error ((shows_string
                ['n', 'o', 'n', '-', 't', 'e', 'r', 'm', 'i', 'n', 'a', 't',
                  'i', 'o', 'n', ' ', 'p', 'r', 'o', 'o', 'f', ' ', 'n', 'o',
                  't', ' ', 'a', 'c', 'c', 'e', 'p', 't', 'e', 'd']
               . shows_nl . err)
              []);
    Righta _ -> Certified False;
  };
certifyCertProblem (CertificationProblem False va (Lefta v)) =
  Error ['N', 'o', 't', ' ', 'a', 'l', 'l', 'o', 'w', 'e', 'd', ' ', 't', 'o',
          ' ', 'g', 'i', 'v', 'e', ' ', 'a', ' ', 't', 'e', 'r', 'm', 'i', 'n',
          'a', 't', 'i', 'o', 'n', ' ', 'p', 'r', 'o', 'o', 'f', ' ', 'f', 'o',
          'r', ' ', 'a', ' ', 'n', 'o', 'n', '-', 't', 'e', 'r', 'm', 'i', 'n',
          'a', 't', 'i', 'o', 'n', ' ', 'g', 'o', 'a', 'l', ' ', 'o', 'r', ' ',
          'v', 'i', 'c', 'e', '-', 'v', 'e', 'r', 's', 'a'];
certifyCertProblem (CertificationProblem True va (Righta vc)) =
  Error ['N', 'o', 't', ' ', 'a', 'l', 'l', 'o', 'w', 'e', 'd', ' ', 't', 'o',
          ' ', 'g', 'i', 'v', 'e', ' ', 'a', ' ', 't', 'e', 'r', 'm', 'i', 'n',
          'a', 't', 'i', 'o', 'n', ' ', 'p', 'r', 'o', 'o', 'f', ' ', 'f', 'o',
          'r', ' ', 'a', ' ', 'n', 'o', 'n', '-', 't', 'e', 'r', 'm', 'i', 'n',
          'a', 't', 'i', 'o', 'n', ' ', 'g', 'o', 'a', 'l', ' ', 'o', 'r', ' ',
          'v', 'i', 'c', 'e', '-', 'v', 'e', 'r', 's', 'a'];

xml_certificationProblem ::
  forall a b c. (Shows a, Shows c) => CertificationProblem a b c -> Xml;
xml_certificationProblem (CertificationProblem term rls prf) =
  let {
    inp = Xml ['i', 'n', 'p', 'u', 't'] []
            [Xml ['t', 'r', 's', 'I', 'n', 'p', 'u', 't'] []
               [Xml ['t', 'r', 's'] [] [xml_rules rls] Nothing] Nothing]
            Nothing;
    goal = (if isLeft prf then terminationGoal else nonTerminationGoal);
  } in Xml ['p', 'r', 'o', 'b', 'l', 'e', 'm'] [] [goal, inp] Nothing;

eqWhiteSpace :: [Char] -> [Char] -> Bool;
eqWhiteSpace s1 s2 =
  debug ['0']
    ['c', 'o', 'm', 'p', 'a', 'r', 'i', 'n', 'g', ' ', 'w', 'h', 'e', 't', 'h',
      'e', 'r', ' ', 'p', 'a', 'r', 's', 'e', 'd', ' ', 'i', 'n', 'p', 'u', 't',
      ' ', 'c', 'o', 'r', 'r', 'e', 's', 'p', 'o', 'n', 'd', 's', ' ', 't', 'o',
      ' ', 'r', 'e', 'a', 'l', ' ', 'i', 'n', 'p', 'u', 't']
    (filtera (\ c -> not (member c wspace)) s1 ==
      filtera (\ c -> not (member c wspace)) s2);

certifyProof :: [Char] -> CertificationResult;
certifyProof problemString =
  case parseCertProblem problemString of {
    Lefta err ->
      Error (append
              ['e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'i', 'l', 'e', ' ', 'p',
                'a', 'r', 's', 'i', 'n', 'g']
              (append [newline] err));
    Righta cp ->
      case extract_goal_input problemString of {
        Nothing ->
          Error ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'f', 'i',
                  'n', 'd', ' ', 'g', 'o', 'a', 'l', ' ', 'a', 'n', 'd', ' ',
                  'i', 'n', 'p', 'u', 't', ' ', 'i', 'n', ' ', 's', 't', 'r',
                  'i', 'n', 'g', ' ', 'f', 'o', 'r', ' ', 'c', 'o', 'm', 'p',
                  'a', 'r', 'i', 's', 'o', 'n', ' ', 'w', 'i', 't', 'h', ' ',
                  'p', 'a', 'r', 's', 'e', 'd', ' ', 'r', 'e', 's', 'u', 'l',
                  't'];
        Just inpProblem ->
          (if eqWhiteSpace inpProblem (showa (xml_certificationProblem cp))
            then certifyCertProblem cp
            else Error (concata
                         [['p', 'a', 'r', 's', 'e', 'd', ' ', 'p', 'r', 'o',
                            'b', 'l', 'e', 'm', ' ', 'd', 'o', 'e', 's', ' ',
                            'n', 'o', 't', ' ', 'c', 'o', 'r', 'r', 'e', 's',
                            'p', 'o', 'n', 'd', ' ', 't', 'o', ' ', 'i', 'n',
                            'p', 'u', 't'],
                           [newline], ['i', 'n', 'p', 'u', 't', ':'], [newline],
                           filtera (\ c -> not (member c wspace)) inpProblem,
                           [newline], [newline],
                           ['p', 'a', 'r', 's', 'e', 'd', ':'], [newline],
                           filtera (\ c -> not (member c wspace))
                             (showa (xml_certificationProblem cp)),
                           [newline]]));
      };
  };

}
